@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2021 Intel Corporation
+* Copyright 2018-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -41,26 +41,53 @@ status_t cvt_primitive_args(const primitive_desc_t *pd, int nargs,
         // allows dummy arguments
         if (mem == nullptr) continue;
 
+        VCONDCHECK(primitive, exec, check, primitive, args.count(arg) == 0,
+                invalid_arguments,
+                "The same argument kind %d is passed multiple times", arg);
+
         switch (pd->arg_usage(arg)) {
             case primitive_desc_t::arg_usage_t::input:
-                if (args.count(arg) != 0) return invalid_arguments;
                 args[arg] = {mem, true};
                 n_inputs++;
                 extra_inputs += (arg == DNNL_ARG_ATTR_OUTPUT_SCALES)
-                        || (arg & DNNL_ARG_ATTR_ZERO_POINTS);
+                        || (arg & DNNL_ARG_ATTR_ZERO_POINTS)
+                        || (arg & DNNL_ARG_ATTR_SCALES)
+                        // 1x1 + dw conv fusion
+                        || (arg
+                                == (DNNL_ARG_ATTR_POST_OP_DW
+                                        | DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC))
+                        || (arg
+                                == (DNNL_ARG_ATTR_POST_OP_DW
+                                        | DNNL_ARG_ATTR_SCALES
+                                        | DNNL_ARG_WEIGHTS))
+                        || (arg
+                                == (DNNL_ARG_ATTR_POST_OP_DW
+                                        | DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST))
+                        || (arg == DNNL_ARG_ATTR_DROPOUT_PROBABILITY)
+                        || (arg == DNNL_ARG_ATTR_DROPOUT_SEED)
+                        || (arg == DNNL_ARG_ATTR_ROUNDING_SEED);
                 break;
             case primitive_desc_t::arg_usage_t::output:
-                if (args.count(arg) != 0) return invalid_arguments;
                 args[arg] = {mem, false};
                 n_outputs++;
-                extra_outputs += (arg == DNNL_ARG_SCRATCHPAD);
+                extra_outputs += (arg == DNNL_ARG_SCRATCHPAD)
+                        || (arg == DNNL_ARG_ATTR_DROPOUT_MASK);
+                break;
+            case primitive_desc_t::arg_usage_t::unused:
+                VINFO(primitive, exec, check, primitive,
+                        "unused primitive execution argument (%d)", arg);
                 break;
-            case primitive_desc_t::arg_usage_t::unused: break;
         }
     }
 
-    if (n_inputs != pd->n_inputs() + extra_inputs) return invalid_arguments;
-    if (n_outputs != pd->n_outputs() + extra_outputs) return invalid_arguments;
+    VCONDCHECK(primitive, exec, check, primitive,
+            (n_inputs == pd->n_inputs() + extra_inputs), invalid_arguments,
+            "bad number of inputs (expected %d got %d)",
+            pd->n_inputs() + extra_inputs, n_inputs);
+    VCONDCHECK(primitive, exec, check, primitive,
+            (n_outputs == pd->n_outputs() + extra_outputs), invalid_arguments,
+            "bad number of outputs (expected %d got %d)",
+            pd->n_outputs() + extra_outputs, n_outputs);
 
     return success;
 }
@@ -98,7 +125,8 @@ void exec_ctx_t::register_memory_mapping(void *handle, void *host_ptr) {
     memory_mapping_.insert({handle, host_ptr});
 }
 
-void *exec_ctx_t::host_ptr(int arg, bool do_zeropad, status_t *status_) const {
+void *exec_ctx_t::host_ptr(
+        int arg, bool do_zeropad, status_t *status_, int index) const {
     status_t status = status::success;
     if (status_) *status_ = status;
 
@@ -108,17 +136,19 @@ void *exec_ctx_t::host_ptr(int arg, bool do_zeropad, status_t *status_) const {
     if (do_zeropad) status = mem->zero_pad(*this);
     if (status_) *status_ = status;
 
-    auto *mem_storage = mem->memory_storage();
+    auto *mem_storage = mem->memory_storage(index);
     return host_ptr(mem_storage);
 }
 
 void *exec_ctx_t::host_ptr(const memory_storage_t *mem_storage) const {
     if (!mem_storage || mem_storage->is_null()) return nullptr;
 
-    void *handle = mem_storage->data_handle();
+    void *handle = mem_storage->root_storage()->data_handle();
     void *base_ptr = nullptr;
     if (memory_mapping_.count(handle) > 0) {
         base_ptr = memory_mapping_.at(handle);
+        base_ptr = reinterpret_cast<char *>(base_ptr)
+                + mem_storage->base_offset();
     } else {
         assert(mem_storage->is_host_accessible());
         base_ptr = handle;
