@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,13 +14,12 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <algorithm>
 #include "primitive_desc.hpp"
 #include "type_helpers.hpp"
 #include "utils.hpp"
 
-#include "pooling_pd.hpp"
-#include "shuffle_pd.hpp"
-
+#include "dnnl_thread.hpp"
 #include "engine.hpp"
 #include "primitive_hashing.hpp"
 
@@ -28,108 +27,22 @@ namespace dnnl {
 namespace impl {
 namespace primitive_hashing {
 
-key_t::key_t(const primitive_desc_t *pd, const engine_t *engine, int impl_nthr)
-    : primitive_kind_(pd->kind())
-    , op_desc_(pd->op_desc())
-    , attr_(pd->attr())
-    , impl_id_(pd->impl_id())
-    , impl_nthr_(impl_nthr)
-    , engine_kind_(engine ? engine->kind() : engine_kind::any_engine)
-    , runtime_kind_(engine ? engine->runtime_kind() : runtime_kind::none)
-    , device_id_(engine ? engine->device_id() : device_id_t(0, 0, 0))
-    , thread_id_(std::this_thread::get_id()) {
-    init_mds(pd);
-}
+key_t::key_t(const engine_t *engine, const op_desc_t *op_desc,
+        const primitive_attr_t *attr, int pd_iterator_offset,
+        const std::vector<memory_desc_t> &hint_mds, int skip_idx)
+    : primitive_kind_(op_desc->primitive_kind)
+    , op_desc_(op_desc)
+    , attr_(attr)
+    , pd_iterator_offset_(pd_iterator_offset)
+    , impl_nthr_(dnnl_get_max_threads())
+    , skip_idx_(skip_idx)
+    , hint_mds_(hint_mds)
+    , engine_id_(engine->engine_id())
+    , thread_id_(std::this_thread::get_id()) {}
 
-void key_t::init_mds(const primitive_desc_t *pd) {
-    // Put only **relevant** memory descriptors to the list that might
-    // affect the equality. The current cases are:
-    // - Backward pooling and shuffle (rationale: implementation might depend
-    //   on the fwd_hint_pd).
-    //
-    // Later this list can be extended. For instance, currently we don't store
-    // convolution mds, because nthrs + op_desc (even with format=`any`) +
-    // attributes fully define particular implementation.
-    //
-    // XXX: There is too much knowledge about in the internals...
-
-    switch ((int)primitive_kind_) {
-        case primitive_kind::batch_normalization: {
-            break;
-        }
-        case primitive_kind::binary: {
-            break;
-        }
-        case primitive_kind::concat: {
-            break;
-        }
-        case primitive_kind::convolution: {
-            break;
-        }
-        case primitive_kind::deconvolution: {
-            break;
-        }
-        case primitive_kind::eltwise: {
-            break;
-        }
-        case primitive_kind::gemm: {
-            break;
-        }
-        case primitive_kind::inner_product: {
-            break;
-        }
-        case primitive_kind::layer_normalization: {
-            break;
-        }
-        case primitive_kind::lrn: {
-            break;
-        }
-        case primitive_kind::matmul: {
-            break;
-        }
-        case primitive_kind::pooling_v2: {
-            auto typed_pd = utils::downcast<const pooling_pd_t *>(pd);
-            if (!typed_pd->is_fwd()) {
-                mds.push_back(*typed_pd->diff_dst_md(0));
-                mds.push_back(*typed_pd->diff_src_md(0));
-            }
-            break;
-        }
-        case primitive_kind::prelu: {
-            break;
-        }
-        case primitive_kind::reduction: {
-            break;
-        }
-        case primitive_kind::reorder: {
-            break;
-        }
-        case primitive_kind::resampling: {
-            break;
-        }
-        case primitive_kind::rnn: {
-            break;
-        }
-        case primitive_kind::shuffle: {
-            auto typed_pd = utils::downcast<const shuffle_pd_t *>(pd);
-            if (!typed_pd->is_fwd()) {
-                mds.push_back(*typed_pd->diff_dst_md(0));
-                mds.push_back(*typed_pd->diff_src_md(0));
-            }
-            break;
-        }
-        case primitive_kind::softmax: {
-            break;
-        }
-        case primitive_kind::sum: {
-            break;
-        }
-        case primitive_kind::zero_pad: {
-            break;
-        }
-        default: assert(!"unknown primitive_kind");
-    }
-}
+key_t::key_t(const primitive_desc_t *pd, const engine_t *engine)
+    : key_t(engine, pd->op_desc(), pd->attr(), pd->pd_iterator_offset(),
+            pd->hint_mds(false /* is_hint */), pd->skip_idx()) {}
 
 bool key_t::operator==(const key_t &rhs) const {
     DNNL_SHORT_CIRCUIT_SELF_COMPARISON(rhs);
@@ -137,40 +50,51 @@ bool key_t::operator==(const key_t &rhs) const {
     bool ret = true
         // Less expensive comparisons come first
         && primitive_kind_ == rhs.primitive_kind_
-        && engine_kind_ == rhs.engine_kind_
-        && runtime_kind_ == rhs.runtime_kind_
-        && device_id_ == rhs.device_id_
-        && mds.size() == rhs.mds.size()
-        && impl_id_ == rhs.impl_id_
+        && engine_id_ == rhs.engine_id_
+        && hint_mds_.size() == rhs.hint_mds_.size()
+        && pd_iterator_offset_ == rhs.pd_iterator_offset_
         && impl_nthr_ == rhs.impl_nthr_
-        && (*attr_) == (*rhs.attr_);
+        && skip_idx_ == rhs.skip_idx_
+        && (*attr_) == (*rhs.attr_)
+        && std::equal(
+            hint_mds_.begin(), hint_mds_.end(), rhs.hint_mds_.begin());
 
-    if (!ret) return false;
+    if (!ret) {
+        // ANCHOR: HASHING_DEBUGINFO_16.
+        VDEBUGINFO(16, primitive, hashing, "operator==,ret=%d", ret);
+        return ret;
+    }
 
 #define CASE(pkind) \
     case primitive_kind::pkind: \
-        ret = cast_to_desc<pkind##_desc_t>(op_desc_) \
-                == cast_to_desc<pkind##_desc_t>(rhs.op_desc_); \
+        ret = *op_desc_t::to_desc<pkind##_desc_t>(op_desc_) \
+                == *op_desc_t::to_desc<pkind##_desc_t>(rhs.op_desc_); \
         break;
 
         switch ((int)primitive_kind_) {
             CASE(batch_normalization)
             CASE(binary)
             CASE(concat)
-            CASE(convolution)
+            // Use a custom comparison function that ignores alg_kind.
+            case primitive_kind::convolution:
+                ret = compare_conv_opdesc(*op_desc_t::to_desc<convolution_desc_t>(op_desc_),
+                *op_desc_t::to_desc<convolution_desc_t>(rhs.op_desc_));
+            break;
             CASE(deconvolution)
             CASE(eltwise)
             CASE(gemm)
+            CASE(group_normalization)
             CASE(inner_product)
             CASE(layer_normalization)
             CASE(lrn)
             CASE(matmul)
-            CASE(pooling_v2)
+            CASE(pooling)
             CASE(prelu)
             CASE(reduction)
             CASE(reorder)
             CASE(resampling)
             CASE(rnn)
+            CASE(sdpa)
             CASE(shuffle)
             CASE(softmax)
             CASE(sum)
@@ -180,12 +104,9 @@ bool key_t::operator==(const key_t &rhs) const {
 #undef CASE
     // clang-format on
 
-    if (!ret) return false;
-
-    for (size_t i = 0; i < mds.size(); ++i)
-        if (mds[i] != rhs.mds[i]) return false;
-
-    return true;
+    // ANCHOR: HASHING_DEBUGINFO_16.
+    VDEBUGINFO(16, primitive, hashing, "operator==,ret=%d", ret);
+    return ret;
 }
 
 // Combine hash of each memory_desc_t data member
@@ -198,7 +119,7 @@ size_t get_md_hash(const memory_desc_t &md) {
     seed = hash_combine(seed, md.offset0);
     seed = hash_combine(seed, static_cast<size_t>(md.format_kind));
     // format desc
-    switch (md.format_kind) {
+    switch ((int)md.format_kind) {
         case format_kind::undef:
         case format_kind::any: break;
         case format_kind::blocked:
@@ -226,6 +147,13 @@ size_t get_md_hash(const memory_desc_t &md) {
             seed = hash_combine(seed, md.format_desc.wino_desc.adj_scale);
             seed = hash_combine(seed, md.format_desc.wino_desc.size);
             break;
+        case format_kind::cublaslt_blocked:
+            seed = hash_combine(seed,
+                    static_cast<size_t>(md.format_desc.cublaslt_blocked_desc
+                                                .cublaslt_format));
+            seed = hash_combine(
+                    seed, (md.format_desc.cublaslt_blocked_desc.size));
+            break;
         case format_kind::rnn_packed:
             seed = hash_combine(seed,
                     static_cast<size_t>(md.format_desc.rnn_packed_desc.format));
@@ -245,6 +173,18 @@ size_t get_md_hash(const memory_desc_t &md) {
                     seed, md.format_desc.rnn_packed_desc.offset_compensation);
             seed = hash_combine(seed, md.format_desc.rnn_packed_desc.size);
             break;
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+        case format_kind::sparse:
+            seed = hash_combine(seed,
+                    static_cast<size_t>(md.format_desc.sparse_desc.encoding));
+            seed = hash_combine(seed, md.format_desc.sparse_desc.nnz);
+            seed = get_array_hash(seed,
+                    md.format_desc.sparse_desc.metadata_types,
+                    sparse_desc_t::max_metadata_types);
+            // User cannot initialize `packed_desc` therefore `packed_desc`
+            // is always zero initialized.
+            break;
+#endif
         default: assert(!"unknown format_kind");
     }
 
@@ -264,47 +204,89 @@ size_t get_md_hash(const memory_desc_t &md) {
                 & dnnl_memory_extra_flag_compensation_conv_asymmetric_src) {
             seed = hash_combine(seed, md.extra.asymm_compensation_mask);
         }
+
+        if (md.extra.flags
+                & dnnl_memory_extra_flag_compensation_gpu_conv_asymmetric_src) {
+            seed = get_array_hash(seed, md.extra.idhw, 3);
+            seed = get_array_hash(seed, md.extra.odhw, 3);
+            seed = get_array_hash(seed, md.extra.pdhw, 3);
+            seed = get_array_hash(seed, md.extra.ddhw, 3);
+            seed = hash_combine(seed, md.extra.dst_size);
+        }
     }
     // Combined hash for a memory descriptor
     return seed;
 }
 
+// Generate a hash for runtime scales
+size_t get_runtime_scale_hash(const runtime_scales_t &scales) {
+    size_t seed = 0;
+    seed = hash_combine(seed, scales.mask_);
+    // scales: groups
+    const int ndims = scales.ndims_;
+    seed = hash_combine(seed, ndims);
+    if (ndims > 0) seed = get_array_hash(seed, scales.group_dims_, ndims);
+    // scales: data type
+    seed = hash_combine(seed, static_cast<size_t>(scales.data_type_));
+    return seed;
+}
+
+// Generate a hash for zero points
+size_t get_zero_points_hash(const zero_points_t &zps) {
+    size_t seed = 0;
+    for (int arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST})
+        if (!zps.has_default_values(arg)) {
+            // zero_points: arg
+            seed = hash_combine(seed, arg);
+
+            int mask = 0;
+            zps.get(arg, &mask);
+            // zero_points: mask
+            seed = hash_combine(seed, mask);
+            // zero points: groups
+            const int ndims = zps.get_groups_ndims(arg);
+            seed = hash_combine(seed, ndims);
+            if (ndims > 0)
+                seed = get_array_hash(seed, zps.get_groups(arg), ndims);
+            // zero points: data type
+            seed = hash_combine(
+                    seed, static_cast<size_t>(zps.get_data_type(arg)));
+        }
+    return seed;
+}
+
 // Combine hash of each primitive_attr_t data member
 size_t get_attr_hash(const primitive_attr_t &attr) {
     size_t seed = 0;
     // scratchpad_mode
     seed = hash_combine(seed, static_cast<size_t>(attr.scratchpad_mode_));
-
-    if (!attr.output_scales_.has_default_values()) {
-        // output_scales: mask
-        seed = hash_combine(seed, attr.output_scales_.mask_);
-        // output_scales: count
-        seed = hash_combine(seed, attr.output_scales_.count_);
-        // output_scales: scales[:]
-        seed = get_array_hash(
-                seed, attr.output_scales_.scales_, attr.output_scales_.count_);
-    } else if (!attr.scales_.has_default_values()) {
-        // go through scales for all arguments
-        for (const auto &p : attr.scales_.scales_) {
-            seed = hash_combine(seed, p.second.mask_);
-            seed = hash_combine(seed, p.second.count_);
-            seed = get_array_hash(seed, p.second.scales_, p.second.count_);
+    // fpmath_mode
+    seed = hash_combine(seed, static_cast<size_t>(attr.fpmath_.mode_));
+    seed = hash_combine(seed, static_cast<size_t>(attr.fpmath_.apply_to_int_));
+    // deterministic
+    seed = hash_combine(seed, static_cast<size_t>(attr.deterministic_));
+    // acc_mode
+    seed = hash_combine(seed, static_cast<size_t>(attr.acc_mode_));
+    // rounding_mode
+    if (!attr.rounding_mode_.has_default_values()) {
+        for (const auto &e : attr.rounding_mode_.rounding_modes_map_) {
+            seed = hash_combine(seed, e.first);
+            seed = hash_combine(seed, static_cast<size_t>(e.second));
         }
     }
-    // zero_points
-    for (int arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST})
-        if (!attr.zero_points_.has_default_values(arg)) {
-            dim_t count = 0;
-            int mask = 0;
-            const int *zero_points = nullptr;
-            attr.zero_points_.get(arg, &count, &mask, &zero_points);
-            // zero_points: count
-            seed = hash_combine(seed, count);
-            // zero_points: mask
-            seed = hash_combine(seed, mask);
-            // zero_points: zero_points[:]
-            seed = get_array_hash(seed, zero_points, count);
+
+    if (!attr.scales_.has_default_values()) {
+        // go through scales for all arguments
+        for (const auto &p : attr.scales_.scales_) {
+            // scales: arg
+            seed = hash_combine(seed, p.first);
+            // scales: mask
+            seed = hash_combine(seed, get_runtime_scale_hash(p.second));
         }
+    }
+
+    seed = hash_combine(seed, get_zero_points_hash(attr.zero_points_));
+
     // post_ops: entry[:]
     for (int i = 0; i < attr.post_ops_.len(); i++) {
         const auto &entry = attr.post_ops_.entry_[i];
@@ -318,28 +300,32 @@ size_t get_attr_hash(const primitive_attr_t &attr) {
                 break;
             case primitive_kind::sum:
                 seed = hash_combine(seed, entry.sum.scale);
+                seed = hash_combine(seed, entry.sum.zero_point);
                 seed = hash_combine(seed, static_cast<size_t>(entry.sum.dt));
                 break;
             case primitive_kind::convolution:
+                seed = hash_combine(
+                        seed, static_cast<size_t>(entry.depthwise_conv.kernel));
                 seed = hash_combine(
                         seed, static_cast<size_t>(entry.depthwise_conv.stride));
+                seed = hash_combine(seed,
+                        static_cast<size_t>(entry.depthwise_conv.padding));
                 seed = hash_combine(
                         seed, static_cast<size_t>(entry.depthwise_conv.wei_dt));
                 seed = hash_combine(seed,
                         static_cast<size_t>(entry.depthwise_conv.bias_dt));
                 seed = hash_combine(
                         seed, static_cast<size_t>(entry.depthwise_conv.dst_dt));
-                if (entry.depthwise_conv.scales) {
-                    seed = hash_combine(seed, entry.depthwise_conv.mask);
-                    seed = hash_combine(seed, entry.depthwise_conv.count);
-                    seed = get_array_hash(seed, entry.depthwise_conv.scales,
-                            entry.depthwise_conv.count);
-                }
                 break;
             case primitive_kind::binary:
                 seed = hash_combine(
                         seed, static_cast<size_t>(entry.binary.alg));
-                seed = hash_combine(seed, get_md_hash(entry.binary.src1_desc));
+                seed = hash_combine(
+                        seed, get_md_hash(entry.binary.user_src1_desc));
+                break;
+            case primitive_kind::prelu:
+                seed = hash_combine(
+                        seed, static_cast<size_t>(entry.prelu.mask));
                 break;
             default: assert(!"unknown post_op");
         }
@@ -356,6 +342,13 @@ size_t get_attr_hash(const primitive_attr_t &attr) {
         seed = get_array_hash(seed, attr.rnn_weights_qparams_.scales_,
                 attr.rnn_weights_qparams_.count_);
     }
+    if (attr.gpu_attr_) {
+        seed = hash_combine(seed, attr.gpu_attr_->get_hash());
+    }
+    if (!attr.dropout_.has_default_values()) {
+        seed = hash_combine(
+                seed, get_md_hash(attr.dropout_.user_dropout_desc_));
+    }
     // Combined hash for attributes
     return seed;
 }
@@ -366,13 +359,13 @@ size_t get_desc_hash(const concat_desc_t &desc) {
     // Kinds
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
     // Memory descriptors
-    seed = hash_combine(seed, get_md_hash(desc.dst_md));
+    seed = hash_combine(seed, get_md_hash(*desc.dst_md));
     // N
     seed = hash_combine(seed, desc.n);
     // Concat dimension
     seed = hash_combine(seed, desc.concat_dimension);
     // Array of mds
-    seed = get_array_hash(seed, desc.src_mds.data(), desc.n);
+    seed = get_array_hash(seed, desc.src_mds);
     // Combined hash for concat desc
     return seed;
 }
@@ -383,10 +376,12 @@ size_t get_desc_hash(const batch_normalization_desc_t &desc) {
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
     seed = hash_combine(seed, static_cast<size_t>(desc.prop_kind));
     // Memory descriptors
-    seed = hash_combine(seed, get_md_hash(desc.data_desc));
-    seed = hash_combine(seed, get_md_hash(desc.diff_data_desc));
-    seed = hash_combine(seed, get_md_hash(desc.data_scaleshift_desc));
-    seed = hash_combine(seed, get_md_hash(desc.diff_data_scaleshift_desc));
+    seed = hash_combine(seed, get_md_hash(desc.src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.scaleshift_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_scaleshift_desc));
     seed = hash_combine(seed, get_md_hash(desc.stat_desc));
     // Epsilon
     seed = hash_combine(seed, desc.batch_norm_epsilon);
@@ -404,6 +399,8 @@ size_t get_desc_hash(const binary_desc_t &desc) {
     // Memory descriptors
     seed = hash_combine(seed, get_md_hash(desc.src_desc[0]));
     seed = hash_combine(seed, get_md_hash(desc.src_desc[1]));
+    if (desc.alg_kind == alg_kind::binary_select)
+        seed = hash_combine(seed, get_md_hash(desc.src_desc[2]));
     seed = hash_combine(seed, get_md_hash(desc.dst_desc));
     // Combined hash for binary op desc
     return seed;
@@ -415,7 +412,18 @@ size_t get_desc_hash(const convolution_desc_t &desc) {
     // Kinds
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
     seed = hash_combine(seed, static_cast<size_t>(desc.prop_kind));
-    seed = hash_combine(seed, static_cast<size_t>(desc.alg_kind));
+
+    // Ignore `alg_kind` to keep hash value consistent for any algorithm.
+    //
+    // Background: when a convolution primitive descriptor is created for
+    // the algorithm `auto` we overwrite `alg_kind` field in `op_desc` when
+    // store it in the primitive descriptor. Because of that, the `op_desc`
+    // stored in the primitive descriptor is different from the one user
+    // passed to oneDNN API. Because of the difference the requested
+    // primitive descriptor cannot be found in the cache if we hash/compare
+    // `alg_kind`.
+    //seed = hash_combine(seed, static_cast<size_t>(desc.alg_kind));
+
     // Memory descriptors
     seed = hash_combine(seed, get_md_hash(desc.src_desc));
     seed = hash_combine(seed, get_md_hash(desc.diff_src_desc));
@@ -432,6 +440,8 @@ size_t get_desc_hash(const convolution_desc_t &desc) {
     seed = get_array_hash(seed, desc.padding[1], DNNL_MAX_NDIMS);
     // Accumulator type
     seed = hash_combine(seed, static_cast<size_t>(desc.accum_data_type));
+    // Internal member
+    seed = hash_combine(seed, static_cast<size_t>(desc.use_inversion));
     // Combined hash for (de-)convolution desc
     return seed;
 }
@@ -444,8 +454,10 @@ size_t get_desc_hash(const eltwise_desc_t &desc) {
     seed = hash_combine(seed, static_cast<size_t>(desc.prop_kind));
     seed = hash_combine(seed, static_cast<size_t>(desc.alg_kind));
     // Memory descriptors
-    seed = hash_combine(seed, get_md_hash(desc.data_desc));
-    seed = hash_combine(seed, get_md_hash(desc.diff_data_desc));
+    seed = hash_combine(seed, get_md_hash(desc.src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_dst_desc));
     // Alpha, beta
     seed = hash_combine(seed, desc.alpha);
     seed = hash_combine(seed, desc.beta);
@@ -463,10 +475,35 @@ size_t get_desc_hash(const gemm_desc_t &desc) {
     seed = hash_combine(seed, get_md_hash(desc.bias_desc));
     // Accumulator type
     seed = hash_combine(seed, static_cast<size_t>(desc.acc_type));
+    seed = hash_combine(seed, static_cast<size_t>(desc.sum_ab));
+    seed = hash_combine(seed, static_cast<size_t>(desc.sum_ab_type));
     // Combined hash for gemm desc
     return seed;
 }
 
+size_t get_desc_hash(const group_normalization_desc_t &desc) {
+    size_t seed = 0;
+    // Kinds
+    seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
+    seed = hash_combine(seed, static_cast<size_t>(desc.prop_kind));
+    // Memory descriptors
+    seed = hash_combine(seed, get_md_hash(desc.src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.scaleshift_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_scaleshift_desc));
+    seed = hash_combine(seed, get_md_hash(desc.dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.stat_desc));
+    // Groups
+    seed = hash_combine(seed, desc.groups);
+    // Epsilon
+    seed = hash_combine(seed, desc.group_norm_epsilon);
+    // Flags
+    seed = hash_combine(seed, desc.flags);
+    // Combined hash for group_normalization desc
+    return seed;
+}
+
 size_t get_desc_hash(const inner_product_desc_t &desc) {
     size_t seed = 0;
     // Kinds
@@ -487,17 +524,18 @@ size_t get_desc_hash(const inner_product_desc_t &desc) {
     return seed;
 }
 
-// Layer normalization
 size_t get_desc_hash(const layer_normalization_desc_t &desc) {
     size_t seed = 0;
     // Kinds
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
     seed = hash_combine(seed, static_cast<size_t>(desc.prop_kind));
     // Memory descriptors
-    seed = hash_combine(seed, get_md_hash(desc.data_desc));
-    seed = hash_combine(seed, get_md_hash(desc.diff_data_desc));
+    seed = hash_combine(seed, get_md_hash(desc.src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_src_desc));
     seed = hash_combine(seed, get_md_hash(desc.data_scaleshift_desc));
     seed = hash_combine(seed, get_md_hash(desc.diff_data_scaleshift_desc));
+    seed = hash_combine(seed, get_md_hash(desc.dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_dst_desc));
     seed = hash_combine(seed, get_md_hash(desc.stat_desc));
     // Epsilon
     seed = hash_combine(seed, desc.layer_norm_epsilon);
@@ -514,8 +552,10 @@ size_t get_desc_hash(const lrn_desc_t &desc) {
     seed = hash_combine(seed, static_cast<size_t>(desc.prop_kind));
     seed = hash_combine(seed, static_cast<size_t>(desc.alg_kind));
     // Memory descriptors
-    seed = hash_combine(seed, get_md_hash(desc.data_desc));
-    seed = hash_combine(seed, get_md_hash(desc.diff_data_desc));
+    seed = hash_combine(seed, get_md_hash(desc.src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_dst_desc));
     // Local size
     seed = hash_combine(seed, desc.local_size);
     // Alpha, beta
@@ -542,7 +582,7 @@ size_t get_desc_hash(const matmul_desc_t &desc) {
     return seed;
 }
 
-size_t get_desc_hash(const pooling_v2_desc_t &desc) {
+size_t get_desc_hash(const pooling_desc_t &desc) {
     size_t seed = 0;
     // Kinds
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
@@ -556,9 +596,9 @@ size_t get_desc_hash(const pooling_v2_desc_t &desc) {
     // Strides, dilates, padding
     seed = get_array_hash(seed, desc.strides, DNNL_MAX_NDIMS);
     seed = get_array_hash(seed, desc.kernel, DNNL_MAX_NDIMS);
-    seed = get_array_hash(seed, desc.dilation, DNNL_MAX_NDIMS);
     seed = get_array_hash(seed, desc.padding[0], DNNL_MAX_NDIMS);
     seed = get_array_hash(seed, desc.padding[1], DNNL_MAX_NDIMS);
+    seed = get_array_hash(seed, desc.dilation, DNNL_MAX_NDIMS);
     // Accumulator type
     seed = hash_combine(seed, static_cast<size_t>(desc.accum_data_type));
     // Combined hash for pooling desc
@@ -571,11 +611,13 @@ size_t get_desc_hash(const prelu_desc_t &desc) {
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
     seed = hash_combine(seed, static_cast<size_t>(desc.prop_kind));
     // Memory descriptors
-    seed = hash_combine(seed, get_md_hash(desc.data_desc));
-    seed = hash_combine(seed, get_md_hash(desc.diff_data_desc));
+    seed = hash_combine(seed, get_md_hash(desc.src_desc));
     seed = hash_combine(seed, get_md_hash(desc.weights_desc));
+    seed = hash_combine(seed, get_md_hash(desc.dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_src_desc));
     seed = hash_combine(seed, get_md_hash(desc.diff_weights_desc));
-    // Combined hash for pooling desc
+    seed = hash_combine(seed, get_md_hash(desc.diff_dst_desc));
+    // Combined hash for prelu desc
     return seed;
 }
 
@@ -599,11 +641,12 @@ size_t get_desc_hash(const reorder_desc_t &desc) {
     // Kinds
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
     // Memory descriptors
-    seed = hash_combine(seed, get_md_hash(desc.src_md));
-    seed = hash_combine(seed, get_md_hash(desc.dst_md));
+    seed = hash_combine(seed, get_md_hash(*desc.src_md));
+    seed = hash_combine(seed, get_md_hash(*desc.dst_md));
     // Kinds of source and destination engines
     seed = hash_combine(seed, static_cast<size_t>(desc.src_engine_kind));
     seed = hash_combine(seed, static_cast<size_t>(desc.dst_engine_kind));
+    seed = hash_combine(seed, desc.is_cross_engine);
     // Combined hash for reorder desc
     return seed;
 }
@@ -672,7 +715,8 @@ size_t get_desc_hash(const shuffle_desc_t &desc) {
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
     seed = hash_combine(seed, static_cast<size_t>(desc.prop_kind));
     // Memory descriptors
-    seed = hash_combine(seed, get_md_hash(desc.data_desc));
+    seed = hash_combine(seed, get_md_hash(desc.src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.dst_desc));
     // Axis
     seed = hash_combine(seed, desc.axis);
     // Groupe size
@@ -686,9 +730,12 @@ size_t get_desc_hash(const softmax_desc_t &desc) {
     // Kinds
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
     seed = hash_combine(seed, static_cast<size_t>(desc.prop_kind));
+    seed = hash_combine(seed, static_cast<size_t>(desc.alg_kind));
     // Memory descriptors
-    seed = hash_combine(seed, get_md_hash(desc.data_desc));
-    seed = hash_combine(seed, get_md_hash(desc.diff_desc));
+    seed = hash_combine(seed, get_md_hash(desc.src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_src_desc));
+    seed = hash_combine(seed, get_md_hash(desc.dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.diff_dst_desc));
     // Axis
     seed = hash_combine(seed, desc.softmax_axis);
     // Combined hash for softmax desc
@@ -700,15 +747,13 @@ size_t get_desc_hash(const sum_desc_t &desc) {
     // Kinds
     seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
     // Memory descriptors
-    seed = hash_combine(seed, get_md_hash(desc.dst_md));
+    seed = hash_combine(seed, get_md_hash(*desc.dst_md));
     // N
     seed = hash_combine(seed, desc.n);
     // Scales
-    if (!desc.scales.empty()) {
-        seed = get_array_hash(seed, desc.scales.data(), desc.n);
-    }
+    if (desc.scales) { seed = get_array_hash(seed, desc.scales, desc.n); }
     // Array of mds
-    seed = get_array_hash(seed, desc.src_mds.data(), desc.n);
+    seed = get_array_hash(seed, desc.src_mds);
     // Combined hash for sum desc
     return seed;
 }
@@ -720,6 +765,29 @@ size_t get_desc_hash(const zero_pad_desc_t &desc) {
     return seed;
 }
 
+size_t get_desc_hash(const sdpa_desc_t &desc) {
+    size_t seed = 0;
+    // Kinds
+    seed = hash_combine(seed, static_cast<size_t>(desc.primitive_kind));
+    // Memory descriptors
+    seed = hash_combine(seed, get_md_hash(desc.q_desc));
+    seed = hash_combine(seed, get_md_hash(desc.k_desc));
+    seed = hash_combine(seed, get_md_hash(desc.v_desc));
+    seed = hash_combine(seed, get_runtime_scale_hash(desc.kq_scales));
+    seed = hash_combine(seed, get_zero_points_hash(desc.kq_zero_points));
+    seed = hash_combine(seed, get_runtime_scale_hash(desc.vs_scales));
+    seed = hash_combine(seed, get_zero_points_hash(desc.vs_zero_points));
+    seed = hash_combine(seed, get_md_hash(desc.dst_desc));
+    seed = hash_combine(seed, get_md_hash(desc.attn_mask_desc));
+    // Scale type
+    seed = hash_combine(seed, static_cast<size_t>(desc.scale_dt));
+    seed = hash_combine(seed, desc.invert_scale);
+    seed = hash_combine(seed, desc.kv_head_number);
+    seed = hash_combine(seed, desc.causal_mask);
+    // Combined hash for sdpa desc
+    return seed;
+}
+
 } // namespace primitive_hashing
 } // namespace impl
 } // namespace dnnl
