@@ -1,6 +1,6 @@
 /*******************************************************************************
-* Copyright 2021 Intel Corporation
-* Copyright 2021 FUJITSU LIMITED
+* Copyright 2021-2022 Intel Corporation
+* Copyright 2021-2022 FUJITSU LIMITED
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -76,8 +76,11 @@ void jit_sve_512_x8s8s32x_fwd_kernel::cvt2ps(data_type_t type_in,
         case data_type::s32:
             if (mask_flag)
                 ld1w(vmm.s, ktail_mask / T_z, ptr(reg_addr));
-            else
-                ld1w(vmm.s, mask_all_one, ptr(reg_addr));
+            else {
+                // Because reg_addr maybe disaligned,
+                // `ldr(vmm, ptr(reg_addr))` can't be used.
+                ld1w(vmm.s, mask_all_one / T_z, ptr(reg_addr));
+            }
             break;
         case data_type::s8:
             sub(reg_stack, reg_stack, 64);
@@ -137,6 +140,7 @@ void jit_sve_512_x8s8s32x_fwd_kernel::store_output(
     for (int k = 0; k < nb_oc_block; k++) {
         const bool mask_flag
                 = last_oc_block_flag && k == nb_oc_block - 1 && mask_gflag;
+        PReg mask = mask_flag ? ktail_mask : mask_all_one;
         int scale_offset = jcp.is_oc_scale * (sizeof(float) * k * oc_block);
         if (jcp.with_bias) {
             int bias_offset = jcp.typesize_bia * k * oc_block;
@@ -152,7 +156,7 @@ void jit_sve_512_x8s8s32x_fwd_kernel::store_output(
         /* optimization under specific conditions: preload scale_offset data */
         if (!jcp.is_fast_depthwise && jcp.signed_input) {
             auto reg_addr = get_comp_addr_reg(reg_ptr_scales, scale_offset);
-            ld1w(vmm_pre_load.s, mask_all_one, ptr(reg_addr));
+            ld1w(vmm_pre_load.s, mask / T_z, ptr(reg_addr));
         }
         /* add to accum: compensation, bias and permute */
         for (int j = 0; j < ur_w; j++) {
@@ -190,22 +194,12 @@ void jit_sve_512_x8s8s32x_fwd_kernel::store_output(
             if (!jcp.is_fast_depthwise && jcp.signed_input) {
                 /* optimization under specific conditions: optimize using preloaded scale_offset data */
                 fmul(vmm.s, vmm.s, vmm_pre_load.s);
-                if (mask_flag) {
-                    not_(mask_tmp.b, mask_all_one.b, ktail_mask.b);
-                    mov(vmm.s, mask_tmp / T_m, 0);
-                }
             } else {
                 auto reg_addr = get_comp_addr_reg(reg_ptr_scales, scale_offset);
-                sub(reg_stack, reg_stack, 64);
-                str(vmm_tmp, ptr(reg_stack));
-                ld1w(vmm_tmp.s, mask_all_one, ptr(reg_addr));
+                st1w(vmm_tmp.s, mask_all_one / T_z, ptr(reg_stack, -1, MUL_VL));
+                ld1w(vmm_tmp.s, mask / T_z, ptr(reg_addr));
                 fmul(vmm.s, vmm.s, vmm_tmp.s);
-                ldr(vmm_tmp, ptr(reg_stack));
-                add(reg_stack, reg_stack, 64);
-                if (mask_flag) {
-                    not_(mask_tmp.b, mask_all_one.b, ktail_mask.b);
-                    mov(vmm.s, mask_tmp / T_m, 0);
-                }
+                ld1w(vmm_tmp.s, mask_all_one / T_z, ptr(reg_stack, -1, MUL_VL));
             }
         }
     }
@@ -238,7 +232,7 @@ void jit_sve_512_x8s8s32x_fwd_kernel::store_output(
     }
 
     // Properly saturate the accumulators for integer datatypes
-    if (one_of(jcp.dst_dt, u8, data_type::s8, s32)) {
+    if (one_of(jcp.dst_dt, data_type::u8, data_type::s8, data_type::s32)) {
         if (jcp.dst_dt == data_type::u8) {
             eor(vmm_zero.d, vmm_zero.d, vmm_zero.d);
         }
@@ -273,15 +267,7 @@ void jit_sve_512_x8s8s32x_fwd_kernel::store_output(
             auto base = reg_out;
             auto re = get_offset(aux_output_offset);
 
-            auto reg_tmp_adr = ((j % 4) == 0) ? reg_tmp0_adr
-                                              : ((j % 4) == 1)
-                            ? reg_tmp1_adr
-                            : ((j % 4) == 2) ? reg_tmp2_adr : reg_tmp3_adr;
-            auto reg_tmp_imm = ((j % 4) == 0) ? reg_tmp0_imm
-                                              : ((j % 4) == 1)
-                            ? reg_tmp1_imm
-                            : ((j % 4) == 2) ? reg_tmp2_imm : reg_tmp3_imm;
-            add_imm(reg_tmp_adr, base, re, reg_tmp_imm);
+            add_imm(reg_tmp0_adr, base, re, reg_tmp0_imm);
 
             auto vmm = vmm_out(j, k);
 
@@ -289,16 +275,16 @@ void jit_sve_512_x8s8s32x_fwd_kernel::store_output(
             switch (jcp.dst_dt) {
                 case data_type::f32:
                 case data_type::s32:
-                    st1w(vmm.s, _mask, ptr(reg_tmp_adr));
+                    st1w(vmm.s, _mask, ptr(reg_tmp0_adr));
                     break;
                 case data_type::s8:
                     smin(vmm.s, 127);
                     smax(vmm.s, -128);
-                    st1b(vmm.s, _mask, ptr(reg_tmp_adr));
+                    st1b(vmm.s, _mask, ptr(reg_tmp0_adr));
                     break;
                 case data_type::u8:
                     umin(vmm.s, 255);
-                    st1b(vmm.s, _mask, ptr(reg_tmp_adr));
+                    st1b(vmm.s, _mask, ptr(reg_tmp0_adr));
                     break;
                 default: assert(!"unknown dst_dt");
             }
@@ -573,21 +559,9 @@ void jit_sve_512_x8s8s32x_fwd_kernel::compute_ker(int ur_w, int pad_l,
                                 ld1rw(vmm_inp(jj, nb_oc_block).s, mask_all_one,
                                         ptr(base, static_cast<int32_t>(re)));
                             else {
-                                auto reg_tmp_adr = ((jj % 4) == 0)
-                                        ? reg_tmp0_adr
-                                        : ((jj % 4) == 1) ? reg_tmp1_adr
-                                                          : ((jj % 4) == 2)
-                                                        ? reg_tmp2_adr
-                                                        : reg_tmp3_adr;
-                                auto reg_tmp_imm = ((jj % 4) == 0)
-                                        ? reg_tmp0_imm
-                                        : ((jj % 4) == 1) ? reg_tmp1_imm
-                                                          : ((jj % 4) == 2)
-                                                        ? reg_tmp2_imm
-                                                        : reg_tmp3_imm;
-                                add_imm(reg_tmp_adr, base, re, reg_tmp_imm);
+                                add_imm(reg_tmp0_adr, base, re, reg_tmp0_imm);
                                 ld1rw(vmm_inp(jj, nb_oc_block).s, mask_all_one,
-                                        ptr(reg_tmp_adr));
+                                        ptr(reg_tmp0_adr));
                             }
                         }
                         if (!jcp.signed_input)
@@ -1231,8 +1205,8 @@ status_t jit_sve_512_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     if (kernel_outside_src) return status::unimplemented;
 
     jcp.signed_input = (src_d.data_type() == data_type::s8) ? true : false;
-    jcp.need_saturation
-            = utils::one_of(dst_d.data_type(), u8, data_type::s8, s32);
+    jcp.need_saturation = utils::one_of(
+            dst_d.data_type(), data_type::u8, data_type::s8, data_type::s32);
     jcp.is_depthwise = true && with_groups && everyone_is(1, jcp.ic, jcp.oc);
 
     if (jcp.is_depthwise && is_3d)
@@ -1452,12 +1426,8 @@ status_t jit_sve_512_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
 
     pick_loop_order(jcp, jcp.nthr);
 
-    const auto &oscales = attr.output_scales_;
-    jcp.is_oc_scale = oscales.mask_ == 1 << 1;
-
-    // only common and per-oc-channel scales are supported
-    const bool oscales_ok = one_of(oscales.mask_, 0, 1 << 1);
-    if (!oscales_ok) return status::unimplemented;
+    // TODO: enable quantization.
+    jcp.is_oc_scale = false;
 
     jcp.wei_adj_scale
             = (weights_d.extra().flags & memory_extra_flags::scale_adjust)
