@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@
 #include <math.h>
 
 #include "common/c_types_map.hpp"
+#include "common/compiler_workarounds.hpp"
 #include "common/dnnl_thread.hpp"
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
@@ -33,6 +34,7 @@
 #include "cpu/binary_injector_utils.hpp"
 #include "cpu/matmul/gemm_bf16_matmul.hpp"
 #include "cpu/matmul/matmul_utils.hpp"
+#include "cpu/scale_utils.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -49,21 +51,42 @@ status_t gemm_bf16_matmul_t<dst_type>::pd_t::init(engine_t *engine) {
                         && is_bias_1xN());
     };
 
-    bool ok = src_md()->data_type == src_type
+    VDISPATCH_MATMUL(is_dense_format_kind(), VERBOSE_UNSUPPORTED_SPARSE_CFG);
+    VDISPATCH_MATMUL(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+
+    const bool problem_dt_correct = src_md()->data_type == src_type
             && weights_md()->data_type == weights_type
             && desc()->accum_data_type == acc_type
             && dst_md()->data_type == dst_type
-            && platform::has_data_type_support(data_type::bf16) && check_bias()
-            && attr()->has_default_values(
-                    primitive_attr_t::skip_mask_t::oscale_runtime
-                    | primitive_attr_t::skip_mask_t::post_ops)
-            && set_default_formats()
-            && gemm_based::check_gemm_compatible_formats(*this);
-    if (!ok) return status::unimplemented;
+            && platform::has_data_type_support(data_type::bf16);
 
-    CHECK(check_and_configure_attributes());
+    VDISPATCH_MATMUL(problem_dt_correct, VERBOSE_UNSUPPORTED_DT_CFG);
+    VDISPATCH_MATMUL(check_bias(), VERBOSE_UNSUPPORTED_BIAS_CFG);
+#if DNNL_X64
+    VDISPATCH_MATMUL(x64::mayiuse(x64::avx512_core), VERBOSE_UNSUPPORTED_ISA);
+#endif
 
-    gemm_based::book_acc_scratchpad(*this, params_, sizeof(acc_data_t));
+    VDISPATCH_MATMUL(attr()->has_default_values(
+                             primitive_attr_t::skip_mask_t::scales_runtime
+                             | primitive_attr_t::skip_mask_t::post_ops),
+            VERBOSE_UNSUPPORTED_ATTR);
+    VDISPATCH_MATMUL(attr()->post_ops_.check_sum_consistency(dst_type,
+                             /* is_int8 */ false),
+            VERBOSE_UNSUPPORTED_POSTOP);
+
+    VDISPATCH_MATMUL(set_default_formats(), VERBOSE_UNSUPPORTED_TAG);
+    bool po_format_ok = attr_.set_default_formats(dst_md(0)) == status::success;
+    VDISPATCH_MATMUL(po_format_ok, VERBOSE_UNSUPPORTED_POSTOP);
+
+    VDISPATCH_MATMUL(gemm_based::check_gemm_compatible_formats(*this),
+            VERBOSE_INCOMPATIBLE_GEMM_FMT);
+
+    CHECK(check_and_configure_attributes(engine));
+
+    nthr_ = dnnl_get_max_threads();
+    gemm_based::book_acc_scratchpad(*this, params_, sizeof(acc_data_t), nthr_);
+    auto scratchpad = scratchpad_registry().registrar();
+    book_precomputed_scales(scratchpad, attr()->scales_, N());
 
     return status::success;
 }
@@ -73,36 +96,65 @@ static bool should_gemm_execute_sum_po(const gemm_based::params_t &params,
     const auto &po = params.pp_attr_.post_ops_;
     static constexpr int sum_idx = 0;
     return po.len() > 0 && po.contain(primitive_kind::sum, sum_idx)
-            && dst_type == data_type::f32 && params.gemm_applies_output_scales_;
+            && dst_type == data_type::f32 && params.gemm_applies_output_scales_
+            && po.entry_[sum_idx].sum.zero_point == 0;
 }
 
 template <impl::data_type_t dst_type>
-status_t gemm_bf16_matmul_t<dst_type>::pd_t::check_and_configure_attributes() {
-    auto check_attr_oscale = [&]() -> bool {
-        const auto &oscale = attr()->output_scales_;
-        return oscale.mask_ == 0
-                || (oscale.mask_ == (1 << 1) && batched() == false);
+status_t gemm_bf16_matmul_t<dst_type>::pd_t::check_and_configure_attributes(
+        engine_t *engine) {
+    auto check_attr_scales = [&]() -> bool {
+        bool ok = attr_scales_ok();
+        if (!attr()->scales_.get(DNNL_ARG_SRC).has_default_values()
+                && !attr()->scales_.get(DNNL_ARG_WEIGHTS).has_default_values()
+                && attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_ != 0) {
+            // This case requires scratchpad with unknown size
+            if (N() == DNNL_RUNTIME_DIM_VAL) ok = false;
+        }
+        return ok;
     };
 
     auto check_attr_post_ops = [&]() -> bool {
         using namespace primitive_kind;
         const auto &post_ops = attr()->post_ops_;
-        return cpu::inner_product_utils::post_ops_ok(post_ops, dst_md());
+        static const bcast_set_t enabled_bcast_strategy {
+                broadcasting_strategy_t::scalar,
+                broadcasting_strategy_t::per_oc,
+                broadcasting_strategy_t::per_oc_spatial,
+                broadcasting_strategy_t::per_mb_spatial,
+                broadcasting_strategy_t::per_mb_w,
+                broadcasting_strategy_t::per_w,
+                broadcasting_strategy_t::no_broadcast};
+        const bool is_binary_po_per_oc
+                = binary_injector_utils::bcast_strategy_present(
+                        binary_injector_utils::extract_bcast_strategies(
+                                post_ops.entry_, dst_md()),
+                        broadcasting_strategy_t::per_oc);
+        const bool has_prelu = post_ops.find(prelu) != -1;
+        return cpu::inner_product_utils::post_ops_ok(
+                       post_ops, dst_md(), enabled_bcast_strategy)
+                && IMPLICATION(is_binary_po_per_oc,
+                        gemm_based::check_gemm_binary_per_oc_compatible_formats(
+                                *this))
+                && IMPLICATION(N() == DNNL_RUNTIME_DIM_VAL, !has_prelu);
     };
 
     // check basic attributes
-    if (!check_attr_oscale()) return status::unimplemented;
+    VDISPATCH_MATMUL(check_attr_scales(), VERBOSE_UNSUPPORTED_SCALES_CFG);
 
     // set state
     CHECK(params_.pp_attr_.copy_from(*attr()));
     params_.gemm_applies_output_scales_
-            = attr()->output_scales_.mask_ == 0 && !with_bias();
+            = attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_ == 0 && !with_bias();
 
-    if (params_.gemm_applies_output_scales_)
-        params_.pp_attr_.output_scales_.set(1.f);
+    if (params_.gemm_applies_output_scales_) {
+        params_.pp_attr_.scales_.reset(DNNL_ARG_SRC);
+        params_.pp_attr_.scales_.reset(DNNL_ARG_WEIGHTS);
+    }
 
     // check post-ops
-    if (!check_attr_post_ops()) return status::unimplemented;
+    VDISPATCH_MATMUL(check_attr_post_ops(), VERBOSE_UNSUPPORTED_POSTOP);
+
     const bool sum_po_via_gemm_beta
             = should_gemm_execute_sum_po(params_, dst_type);
     // set state
@@ -132,39 +184,56 @@ bool gemm_bf16_matmul_t<dst_type>::should_skip_sum_po() const noexcept {
 template <impl::data_type_t dst_type>
 status_t gemm_bf16_matmul_t<dst_type>::execute_ref(
         const exec_ctx_t &ctx) const {
+    using namespace binary_injector_utils;
     auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
     auto weights = CTX_IN_MEM(const weights_data_t *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
     auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
-    const auto post_ops_binary_rhs_arg_vec
-            = binary_injector_utils::prepare_binary_args(
-                    this->pd()->attr()->post_ops_, ctx);
-
-    DEFINE_SCALES_BUFFER(scales);
+    const auto &po = this->pd()->attr()->post_ops_;
+    const auto post_ops_binary_rhs_arg_vec = prepare_binary_args(po, ctx);
 
     const auto src_d = ctx.memory_mdw(DNNL_ARG_SRC, pd()->src_md());
     const auto weights_d = ctx.memory_mdw(DNNL_ARG_WEIGHTS, pd()->weights_md());
     const auto dst_d = ctx.memory_mdw(DNNL_ARG_DST, pd()->dst_md());
 
-    matmul_helper_t helper(src_d, weights_d, dst_d);
     const int ndims = pd()->ndims();
+
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    auto scratchpad = ctx.get_scratchpad_grantor();
+    const int wei_scale_mask
+            = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const float *scales = precompute_scales(scratchpad, src_scales, wei_scales,
+            src_d.dims()[ndims - 1], dst_d.dims()[ndims - 1], false,
+            wei_scale_mask != 0, pd()->attr());
+
+    if (src_d.has_zero_dim() || weights_d.has_zero_dim()
+            || dst_d.has_zero_dim())
+        return status::success;
+
+    matmul_helper_t helper(src_d, weights_d, dst_d);
     const int batch_ndims = ndims - 2;
     dim_t M = helper.M();
     const dim_t N = helper.N();
     const dim_t K = helper.K();
     const dim_t batch = helper.batch();
+    const dim_t batch_without_dim0
+            = helper.ndims() > 3 ? batch / dst_d.dims()[0] : 0;
+    const dim_t batch_without_dim01
+            = helper.ndims() > 4 ? batch_without_dim0 / dst_d.dims()[1] : 1;
     const char transA = helper.transA();
     const char transB = helper.transB();
     const dim_t lda = helper.lda();
     const dim_t ldb = helper.ldb();
     const dim_t ldc = helper.ldc();
+    const int nthr = pd()->nthr_;
 
     const gemm_based::params_t &params = pd()->params();
-    const bool can_fuse_src_batch_dims = pd()->has_runtime_dims_or_strides()
-            ? helper.can_fuse_src_batch_dims()
-            : params.can_fuse_src_batch_dims_;
-    const dim_t acc_stride = gemm_based::get_scratchpad_size(
-            batch, M, N, can_fuse_src_batch_dims);
+    const bool use_single_gemm_call = pd()->has_runtime_dims_or_strides()
+            ? helper.use_single_gemm_call_optimization(po)
+            : params.use_single_gemm_call_optimization_;
     bool dst_is_acc = params.dst_is_acc_;
     acc_data_t *acc = dst_is_acc
             ? (acc_data_t *)dst
@@ -173,11 +242,10 @@ status_t gemm_bf16_matmul_t<dst_type>::execute_ref(
     // case: dynamic sizes
     bool need_free_acc = false;
     if (acc == nullptr) {
-        acc = (acc_data_t *)malloc(sizeof(acc_data_t) * acc_stride
-                        * ((can_fuse_src_batch_dims || batch == 1)
-                                        ? 1
-                                        : (dim_t)dnnl_get_max_threads()),
-                64);
+        const size_t buf_elements = gemm_based::get_scratchpad_num_elements(
+                batch, M, N, use_single_gemm_call, nthr);
+        acc = (acc_data_t *)malloc(sizeof(acc_data_t) * buf_elements, 64);
+
         if (acc == nullptr) return status::out_of_memory;
         need_free_acc = true;
     }
@@ -185,10 +253,12 @@ status_t gemm_bf16_matmul_t<dst_type>::execute_ref(
     const float alpha = params.get_gemm_alpha(scales);
     const float beta = params.gemm_beta_;
     const dim_t acc_ldc = dst_is_acc ? ldc : N;
+    const int scale_idx_mult
+            = this->pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_
+            == (1 << (ndims - 1));
 
     std::atomic<status_t> st(status::success);
-    const bool parallel_over_batch = batch > 1 && !can_fuse_src_batch_dims;
-    if (parallel_over_batch) {
+    if (!use_single_gemm_call) {
         const int src_mask
                 = utils::get_dims_mask(dst_d.dims(), src_d.dims(), ndims);
         const int wei_mask
@@ -198,11 +268,10 @@ status_t gemm_bf16_matmul_t<dst_type>::execute_ref(
                 : types::data_type_size(pd()->weights_md(1)->data_type);
         const size_t work_amount = (size_t)batch * M * N;
         const size_t work_per_batch = (size_t)M * N;
+        const dim_t acc_stride = gemm_based::get_scratchpad_block_elements(
+                batch, M, N, use_single_gemm_call, nthr);
 
-        // NOTE: inside lambda, type cast variables captured by reference using
-        // either c-like "(type)var" or functional "type(var)" notation in order
-        // to avoid gcc bug with c++14 standard. Otherwise, capture by value.
-        parallel(0, [=, &st](int ithr, int nthr) {
+        parallel(nthr, [&](int ithr, int nthr) {
             size_t t_work_start {0}, t_work_end {0};
             balance211(work_amount, nthr, ithr, t_work_start, t_work_end);
 
@@ -236,20 +305,24 @@ status_t gemm_bf16_matmul_t<dst_type>::execute_ref(
                 if (!reuse_acc) curr_acc = acc + dst_off;
                 dim_t gemm_M {0}, gemm_N {0};
 
+                size_t matrix_offset;
                 const size_t rem_work = t_work_end - i_work;
                 if (rem_work >= work_per_batch && cur_m == 0 && cur_n == 0) {
                     // parallel over batch
                     gemm_M = M;
                     gemm_N = N;
+                    matrix_offset = 0;
                 } else if (rem_work >= (size_t)N && cur_n == 0) {
                     // parallel over M
                     gemm_M = nstl::min(
                             (size_t)(M - cur_m), (size_t)(rem_work / N));
                     gemm_N = N;
+                    matrix_offset = cur_n + cur_m * N;
                 } else {
                     // parallel over N
                     gemm_M = 1;
                     gemm_N = nstl::min((size_t)(N - cur_n), rem_work);
+                    matrix_offset = cur_n + cur_m * N;
                 }
 
                 status_t st_thr = gemm_bf16bf16f32(&transB, &transA, &gemm_N,
@@ -263,15 +336,24 @@ status_t gemm_bf16_matmul_t<dst_type>::execute_ref(
                 if (params.has_pp_kernel_) {
                     const float *pp_scales
                             = params.get_post_processing_scales(scales);
-
+                    const size_t dst_logical_off = i_work;
+                    const size_t dim1_off = helper.ndims() > 3
+                            ? ((cur_b % batch_without_dim0)
+                                    / batch_without_dim01)
+                            : cur_m;
+                    // offset for case with post-op broadcast_channel
+                    const size_t matrix_per_first_batch_off = helper.ndims() > 3
+                            ? M * N * (cur_b / batch_without_dim0)
+                                    + matrix_offset
+                            : 0;
+                    const ptrdiff_t oc_off = i_work % N;
                     (*pp_kernel_)(curr_dst, curr_acc,
-                            bias
-                                    + static_cast<ptrdiff_t>(i_work % N)
-                                            * bia_dt_size,
-                            pp_scales, 0, gemm_M * gemm_N,
+                            bias + oc_off * bia_dt_size,
+                            pp_scales + oc_off * scale_idx_mult, dst_scales[0],
+                            0, dst_logical_off, dim1_off, gemm_M * gemm_N,
                             static_cast<size_t>(N), ldc, nullptr,
-                            post_ops_binary_rhs_arg_vec.data(), dst, ctx,
-                            *pd()->dst_md());
+                            post_ops_binary_rhs_arg_vec.data(), dst,
+                            matrix_per_first_batch_off, ctx, *pd()->dst_md());
                 }
                 i_work += gemm_M * gemm_N;
             }
@@ -287,12 +369,15 @@ status_t gemm_bf16_matmul_t<dst_type>::execute_ref(
             const bool force_sequential = pp_kernel_->sequential_kernel();
             const float *pp_scales = params.get_post_processing_scales(scales);
 
-            parallel(force_sequential ? 1 : 0, [&](int ithr, int nthr) {
+            parallel(force_sequential ? 1 : nthr, [&](int ithr, int nthr) {
                 size_t start {}, end {};
                 balance211((size_t)(M * N), nthr, ithr, start, end);
-                (*pp_kernel_)(dst, acc, bias, pp_scales, start, end, (size_t)N,
-                        ldc, nullptr, post_ops_binary_rhs_arg_vec.data(), dst,
-                        ctx, *pd()->dst_md());
+                const size_t dst_logical_off = start;
+                const size_t dim1_off = start % N;
+                (*pp_kernel_)(dst, acc, bias, pp_scales, dst_scales[0], start,
+                        dst_logical_off, dim1_off, end, (size_t)N, ldc, nullptr,
+                        post_ops_binary_rhs_arg_vec.data(), dst, 0, ctx,
+                        *pd()->dst_md());
             });
         }
     }
