@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -101,8 +101,9 @@ struct reducer_2d_driver_t : public jit_generator {
     using data_t = typename prec_traits<data_type>::type;
 
     reducer_2d_driver_t(int n_src, size_t src_ld, size_t src_step,
-            size_t dst_step, bool nullify_dst)
-        : n_src_(n_src)
+            size_t dst_step, bool nullify_dst, const char *name)
+        : jit_generator(name)
+        , n_src_(n_src)
         , src_ld_(src_ld)
         , src_step_(src_step)
         , dst_step_(dst_step)
@@ -159,7 +160,7 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type> {
     reducer_2d_driver_f_s_32_t(int n_src, size_t src_ld, size_t src_step,
             size_t dst_step, bool nullify_dst)
         : reducer_2d_driver_t<data_type>(
-                n_src, src_ld, src_step, dst_step, nullify_dst) {}
+                n_src, src_ld, src_step, dst_step, nullify_dst, jit_name()) {}
 
     void nullify_dst(int nloads, int load_len) {
         UNUSED(load_len);
@@ -263,7 +264,7 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type> {
     }
 
     void generate() override {
-        assert(isa == avx2 || isa == avx512_common || isa == avx512_mic);
+        static_assert(isa == avx2 || isa == avx512_core, "unsupported CPU ISA");
 
         this->preamble();
 
@@ -287,8 +288,8 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type> {
 template <impl::data_type_t data_type>
 inline reducer_2d_driver_t<data_type> *create_reduce_2d_drv(int n_src,
         size_t src_ld, size_t src_step, size_t dst_step, bool nullify_dst) {
-    if (mayiuse(avx512_common))
-        return new reducer_2d_driver_f_s_32_t<data_type, avx512_common>(
+    if (mayiuse(avx512_core))
+        return new reducer_2d_driver_f_s_32_t<data_type, avx512_core>(
                 n_src, src_ld, src_step, dst_step, nullify_dst);
     else if (mayiuse(avx2))
         return new reducer_2d_driver_f_s_32_t<data_type, avx2>(
@@ -551,9 +552,8 @@ template struct cpu_reducer_2d_t<data_type::s32>;
 /* accumulator section */
 
 template <impl::data_type_t data_type>
-cpu_accumulator_1d_t<data_type>::cpu_accumulator_1d_t() : drv_(nullptr) {
-    drv_ = create_reduce_2d_drv<data_type>(1, 0, 0, 0, false);
-}
+cpu_accumulator_1d_t<data_type>::cpu_accumulator_1d_t()
+    : drv_(create_reduce_2d_drv<data_type>(1, 0, 0, 0, false)) {}
 
 template <impl::data_type_t data_type>
 cpu_accumulator_1d_t<data_type>::~cpu_accumulator_1d_t() {
