@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -23,10 +23,10 @@ namespace impl {
 namespace cpu {
 namespace x64 {
 
-jit_avx512_core_s16_48x8_copy_an_kern::jit_avx512_core_s16_48x8_copy_an_kern()
+jit_avx512_core_s16_24x8_copy_an_kern::jit_avx512_core_s16_24x8_copy_an_kern()
     : jit_generator(jit_name()) {}
 
-void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
+void jit_avx512_core_s16_24x8_copy_an_kern::generate() {
 
 #ifndef _WIN32
 #define M rdi
@@ -63,52 +63,46 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
     inLocalLabel();
     {
 
-        Xbyak::Label l1e8;
+        Xbyak::Label l118;
+        Xbyak::Label l18c;
+        Xbyak::Label l1c0;
+        Xbyak::Label l1d0;
+        Xbyak::Label l1dc;
+        Xbyak::Label l1f0;
         Xbyak::Label l24;
-        Xbyak::Label l2c8;
-        Xbyak::Label l2fc;
-        Xbyak::Label l30c;
+        Xbyak::Label l28c;
+        Xbyak::Label l2e0;
+        Xbyak::Label l308;
         Xbyak::Label l318;
-        Xbyak::Label l32c;
+        Xbyak::Label l324;
+        Xbyak::Label l338;
         Xbyak::Label l38;
-        Xbyak::Label l44c;
-        Xbyak::Label l4e8;
-        Xbyak::Label l510;
-        Xbyak::Label l520;
-        Xbyak::Label l52c;
-        Xbyak::Label l540;
-        Xbyak::Label l5dc;
-        Xbyak::Label l630;
-        Xbyak::Label l658;
+        Xbyak::Label l3e0;
+        Xbyak::Label l43c;
+        Xbyak::Label l470;
+        Xbyak::Label l48c;
+        Xbyak::Label l49c;
+        Xbyak::Label l4a8;
+        Xbyak::Label l4b8;
+        Xbyak::Label l534;
+        Xbyak::Label l57c;
+        Xbyak::Label l5a8;
+        Xbyak::Label l5c4;
+        Xbyak::Label l5d4;
+        Xbyak::Label l5e0;
+        Xbyak::Label l5f0;
         Xbyak::Label l668;
-        Xbyak::Label l674;
-        Xbyak::Label l688;
-        Xbyak::Label l730;
-        Xbyak::Label l78c;
-        Xbyak::Label l7c0;
+        Xbyak::Label l6ac;
+        Xbyak::Label l6d8;
+        Xbyak::Label l6f4;
+        Xbyak::Label l702;
+        Xbyak::Label l70c;
+        Xbyak::Label l71c;
+        Xbyak::Label l794;
         Xbyak::Label l7dc;
-        Xbyak::Label l7ec;
-        Xbyak::Label l7f8;
-        Xbyak::Label l808;
-        Xbyak::Label l884;
-        Xbyak::Label l8cc;
-        Xbyak::Label l8f8;
-        Xbyak::Label l914;
-        Xbyak::Label l924;
-        Xbyak::Label l930;
-        Xbyak::Label l940;
-        Xbyak::Label l9b8;
-        Xbyak::Label l9fc;
-        Xbyak::Label la28;
-        Xbyak::Label la44;
-        Xbyak::Label la52;
-        Xbyak::Label la5c;
-        Xbyak::Label la6c;
-        Xbyak::Label lae4;
-        Xbyak::Label lb2c;
-        Xbyak::Label lb5c;
-        Xbyak::Label lb74;
-        Xbyak::Label lb84;
+        Xbyak::Label l80c;
+        Xbyak::Label l824;
+        Xbyak::Label l834;
 
         preamble();
 #ifdef _WIN32
@@ -124,16 +118,16 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         lea(LDA3, ptr[LDA + LDA * 2]);
         sub(A, -128);
         sub(B, -128);
-        cmp(N, 0x30);
-        jl(l30c, T_NEAR);
+        cmp(N, 0x18);
+        jl(l1d0, T_NEAR);
         align(4);
 
         L(l24);
         mov(A1, A);
-        add(A, 0x60);
+        add(A, 0x30);
         mov(I, M);
         sar(I, 0x2);
-        jle(l1e8, T_NEAR);
+        jle(l118, T_NEAR);
         align(4);
 
         L(l38);
@@ -155,259 +149,89 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         vpunpckhwd(xmm3, xmm0, xmm1);
         vperm2f128(ymm2, ymm2, ymm3, 0x20);
         vmovdqu(yword[B - 0x40], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x50]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x50]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x20], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x40]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x40]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x30]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x30]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B + 0x20], ymm2);
         lea(A1, ptr[A1 + LDA * 2]);
         vmovdqu(xmm0, xword[A1 - 0x80]);
         vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         vpunpcklwd(xmm2, xmm0, xmm1);
         vpunpckhwd(xmm3, xmm0, xmm1);
         vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B + 0x40], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x70]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x70]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B + 0x60], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x60]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x60]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B + 0x80], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x50]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x50]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B + 0xa0], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x40]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x40]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B + 0xc0], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x30]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x30]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B + 0xe0], ymm2);
-        lea(A1, ptr[A1 + LDA * 2]);
-        sub(B, -384);
-        dec(I);
-        jg(l38, T_NEAR);
-        align(4);
-
-        L(l1e8);
-        test(M, 0x2);
-        jle(l2c8, T_NEAR);
-        vmovdqu(xmm0, xword[A1 - 0x80]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x80], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x70]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x70]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x60], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x60]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x60]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x40], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x50]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x50]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
         vmovdqu(yword[B - 0x20], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x40]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x40]);
+        vmovdqu(xmm0, xword[A1 - 0x70]);
+        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x70]);
         vpunpcklwd(xmm2, xmm0, xmm1);
         vpunpckhwd(xmm3, xmm0, xmm1);
         vperm2f128(ymm2, ymm2, ymm3, 0x20);
         vmovdqu(yword[B], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x30]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x30]);
+        vmovdqu(xmm0, xword[A1 - 0x60]);
+        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x60]);
         vpunpcklwd(xmm2, xmm0, xmm1);
         vpunpckhwd(xmm3, xmm0, xmm1);
         vperm2f128(ymm2, ymm2, ymm3, 0x20);
         vmovdqu(yword[B + 0x20], ymm2);
         lea(A1, ptr[A1 + LDA * 2]);
         sub(B, -192);
+        dec(I);
+        jg(l38, T_NEAR);
         align(4);
 
-        L(l2c8);
-        test(M, 0x1);
-        jle(l2fc, T_NEAR);
-        vmovdqu(ymm0, yword[A1 - 0x80]);
-        vmovdqu(ymm1, yword[A1 - 0x60]);
-        vmovdqu(ymm2, yword[A1 - 0x40]);
-        vmovdqu(yword[B - 0x80], ymm0);
-        vmovdqu(yword[B - 0x60], ymm1);
+        L(l118);
+        test(M, 0x2);
+        jle(l18c, T_NEAR);
+        vmovdqu(xmm0, xword[A1 - 0x80]);
+        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
+        vpunpcklwd(xmm2, xmm0, xmm1);
+        vpunpckhwd(xmm3, xmm0, xmm1);
+        vperm2f128(ymm2, ymm2, ymm3, 0x20);
+        vmovdqu(yword[B - 0x80], ymm2);
+        vmovdqu(xmm0, xword[A1 - 0x70]);
+        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x70]);
+        vpunpcklwd(xmm2, xmm0, xmm1);
+        vpunpckhwd(xmm3, xmm0, xmm1);
+        vperm2f128(ymm2, ymm2, ymm3, 0x20);
+        vmovdqu(yword[B - 0x60], ymm2);
+        vmovdqu(xmm0, xword[A1 - 0x60]);
+        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x60]);
+        vpunpcklwd(xmm2, xmm0, xmm1);
+        vpunpckhwd(xmm3, xmm0, xmm1);
+        vperm2f128(ymm2, ymm2, ymm3, 0x20);
         vmovdqu(yword[B - 0x40], ymm2);
+        lea(A1, ptr[A1 + LDA * 2]);
         sub(B, -96);
         align(4);
 
-        L(l2fc);
-        sub(N, 0x30);
-        cmp(N, 0x30);
+        L(l18c);
+        test(M, 0x1);
+        jle(l1c0, T_NEAR);
+        vmovdqu(xmm0, xword[A1 - 0x80]);
+        vmovdqu(xmm1, xword[A1 - 0x70]);
+        vmovdqu(xmm2, xword[A1 - 0x60]);
+        add(A1, LDA);
+        vmovdqu(xword[B - 0x80], xmm0);
+        vmovdqu(xword[B - 0x70], xmm1);
+        vmovdqu(xword[B - 0x60], xmm2);
+        sub(B, -48);
+        align(4);
+
+        L(l1c0);
+        sub(N, 0x18);
+        cmp(N, 0x18);
         jge(l24, T_NEAR);
         align(4);
 
-        L(l30c);
-        cmp(N, 0x20);
-        jl(l520, T_NEAR);
-        align(4);
-
-        L(l318);
-        mov(A1, A);
-        add(A, 0x40);
-        mov(I, M);
-        sar(I, 0x2);
-        jle(l44c, T_NEAR);
-        align(4);
-
-        L(l32c);
-        vmovdqu(xmm0, xword[A1 - 0x80]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x80], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x70]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x70]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x60], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x60]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x60]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x40], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x50]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x50]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x20], ymm2);
-        lea(A1, ptr[A1 + LDA * 2]);
-        vmovdqu(xmm0, xword[A1 - 0x80]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x70]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x70]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B + 0x20], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x60]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x60]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B + 0x40], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x50]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x50]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B + 0x60], ymm2);
-        lea(A1, ptr[A1 + LDA * 2]);
-        sub(B, -256);
-        dec(I);
-        jg(l32c, T_NEAR);
-        align(4);
-
-        L(l44c);
-        test(M, 0x2);
-        jle(l4e8, T_NEAR);
-        vmovdqu(xmm0, xword[A1 - 0x80]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x80], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x70]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x70]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x60], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x60]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x60]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x40], ymm2);
-        vmovdqu(xmm0, xword[A1 - 0x50]);
-        vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x50]);
-        vpunpcklwd(xmm2, xmm0, xmm1);
-        vpunpckhwd(xmm3, xmm0, xmm1);
-        vperm2f128(ymm2, ymm2, ymm3, 0x20);
-        vmovdqu(yword[B - 0x20], ymm2);
-        lea(A1, ptr[A1 + LDA * 2]);
-        sub(B, -128);
-        align(4);
-
-        L(l4e8);
-        test(M, 0x1);
-        jle(l510, T_NEAR);
-        vmovdqu(ymm0, yword[A1 - 0x80]);
-        vmovdqu(ymm1, yword[A1 - 0x60]);
-        add(A1, LDA);
-        vmovdqu(yword[B - 0x80], ymm0);
-        vmovdqu(yword[B - 0x60], ymm1);
-        sub(B, -64);
-        align(4);
-
-        L(l510);
-        sub(N, 0x20);
-        cmp(N, 0x20);
-        jge(l318, T_NEAR);
-        align(4);
-
-        L(l520);
+        L(l1d0);
         cmp(N, 0x10);
-        jl(l668, T_NEAR);
+        jl(l318, T_NEAR);
         align(4);
 
-        L(l52c);
+        L(l1dc);
         mov(A1, A);
         add(A, 0x20);
         mov(I, M);
         sar(I, 0x2);
-        jle(l5dc, T_NEAR);
+        jle(l28c, T_NEAR);
         align(4);
 
-        L(l540);
+        L(l1f0);
         vmovdqu(xmm0, xword[A1 - 0x80]);
         vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         vpunpcklwd(xmm2, xmm0, xmm1);
@@ -436,12 +260,12 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         lea(A1, ptr[A1 + LDA * 2]);
         sub(B, -128);
         dec(I);
-        jg(l540, T_NEAR);
+        jg(l1f0, T_NEAR);
         align(4);
 
-        L(l5dc);
+        L(l28c);
         test(M, 0x2);
-        jle(l630, T_NEAR);
+        jle(l2e0, T_NEAR);
         vmovdqu(xmm0, xword[A1 - 0x80]);
         vmovdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         vpunpcklwd(xmm2, xmm0, xmm1);
@@ -458,9 +282,9 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l630);
+        L(l2e0);
         test(M, 0x1);
-        jle(l658, T_NEAR);
+        jle(l308, T_NEAR);
         vmovdqu(xmm0, xword[A1 - 0x80]);
         vmovdqu(xmm1, xword[A1 - 0x70]);
         vmovdqu(xword[B - 0x80], xmm0);
@@ -468,26 +292,26 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l658);
+        L(l308);
         sub(N, 0x10);
         cmp(N, 0x10);
-        jge(l52c, T_NEAR);
+        jge(l1dc, T_NEAR);
         align(4);
 
-        L(l668);
+        L(l318);
         cmp(N, 0x8);
-        jl(l7ec, T_NEAR);
+        jl(l49c, T_NEAR);
         align(4);
 
-        L(l674);
+        L(l324);
         mov(A1, A);
         add(A, 0x10);
         mov(I, M);
         sar(I, 0x3);
-        jle(l730, T_NEAR);
+        jle(l3e0, T_NEAR);
         align(4);
 
-        L(l688);
+        L(l338);
         vmovdqu(xmm0, xword[A1 - 0x80]);
         add(A1, LDA);
         vmovdqu(xmm1, xword[A1 - 0x80]);
@@ -522,12 +346,12 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         vmovdqu(yword[B - 0x20], ymm2);
         sub(B, -128);
         dec(I);
-        jg(l688, T_NEAR);
+        jg(l338, T_NEAR);
         align(4);
 
-        L(l730);
+        L(l3e0);
         test(M, 0x4);
-        jle(l78c, T_NEAR);
+        jle(l43c, T_NEAR);
         vmovdqu(xmm0, xword[A1 - 0x80]);
         add(A1, LDA);
         vmovdqu(xmm1, xword[A1 - 0x80]);
@@ -547,9 +371,9 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l78c);
+        L(l43c);
         test(M, 0x2);
-        jle(l7c0, T_NEAR);
+        jle(l470, T_NEAR);
         vmovdqu(xmm0, xword[A1 - 0x80]);
         add(A1, LDA);
         vmovdqu(xmm1, xword[A1 - 0x80]);
@@ -561,34 +385,34 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l7c0);
+        L(l470);
         test(M, 0x1);
-        jle(l7dc, T_NEAR);
+        jle(l48c, T_NEAR);
         vmovdqu(xmm0, xword[A1 - 0x80]);
         vmovdqu(xword[B - 0x80], xmm0);
         sub(B, -16);
         align(4);
 
-        L(l7dc);
+        L(l48c);
         sub(N, 0x8);
         cmp(N, 0x8);
-        jge(l674, T_NEAR);
+        jge(l324, T_NEAR);
         align(4);
 
-        L(l7ec);
+        L(l49c);
         cmp(N, 0x4);
-        jl(l924, T_NEAR);
+        jl(l5d4, T_NEAR);
         align(4);
 
-        L(l7f8);
+        L(l4a8);
         mov(A1, A);
         add(A, 0x8);
         mov(I, M);
         sar(I, 0x3);
-        jle(l884, T_NEAR);
+        jle(l534, T_NEAR);
         align(4);
 
-        L(l808);
+        L(l4b8);
         vmovq(xmm0, qword[A1 - 0x80]);
         add(A1, LDA);
         vmovq(xmm1, qword[A1 - 0x80]);
@@ -615,12 +439,12 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         vmovdqu(yword[B - 0x60], ymm0);
         sub(B, -64);
         dec(I);
-        jg(l808, T_NEAR);
+        jg(l4b8, T_NEAR);
         align(4);
 
-        L(l884);
+        L(l534);
         test(M, 0x4);
-        jle(l8cc, T_NEAR);
+        jle(l57c, T_NEAR);
         vmovq(xmm0, qword[A1 - 0x80]);
         add(A1, LDA);
         vmovq(xmm1, qword[A1 - 0x80]);
@@ -636,9 +460,9 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l8cc);
+        L(l57c);
         test(M, 0x2);
-        jle(l8f8, T_NEAR);
+        jle(l5a8, T_NEAR);
         vmovq(xmm0, qword[A1 - 0x80]);
         add(A1, LDA);
         vmovq(xmm1, qword[A1 - 0x80]);
@@ -648,34 +472,34 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l8f8);
+        L(l5a8);
         test(M, 0x1);
-        jle(l914, T_NEAR);
+        jle(l5c4, T_NEAR);
         vmovq(xmm0, qword[A1 - 0x80]);
         vmovq(qword[B - 0x80], xmm0);
         sub(B, -8);
         align(4);
 
-        L(l914);
+        L(l5c4);
         sub(N, 0x4);
         cmp(N, 0x4);
-        jge(l7f8, T_NEAR);
+        jge(l4a8, T_NEAR);
         align(4);
 
-        L(l924);
+        L(l5d4);
         cmp(N, 0x2);
-        jl(la52, T_NEAR);
+        jl(l702, T_NEAR);
         align(4);
 
-        L(l930);
+        L(l5e0);
         mov(A1, A);
         add(A, 0x4);
         mov(I, M);
         sar(I, 0x3);
-        jle(l9b8, T_NEAR);
+        jle(l668, T_NEAR);
         align(4);
 
-        L(l940);
+        L(l5f0);
         vmovd(xmm0, dword[A1 - 0x80]);
         add(A1, LDA);
         vmovd(xmm1, dword[A1 - 0x80]);
@@ -702,12 +526,12 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         vmovdqu(xword[B - 0x70], xmm0);
         sub(B, -32);
         dec(I);
-        jg(l940, T_NEAR);
+        jg(l5f0, T_NEAR);
         align(4);
 
-        L(l9b8);
+        L(l668);
         test(M, 0x4);
-        jle(l9fc, T_NEAR);
+        jle(l6ac, T_NEAR);
         vmovd(xmm0, dword[A1 - 0x80]);
         add(A1, LDA);
         vmovd(xmm1, dword[A1 - 0x80]);
@@ -723,9 +547,9 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l9fc);
+        L(l6ac);
         test(M, 0x2);
-        jle(la28, T_NEAR);
+        jle(l6d8, T_NEAR);
         vmovd(xmm0, dword[A1 - 0x80]);
         add(A1, LDA);
         vmovd(xmm1, dword[A1 - 0x80]);
@@ -735,34 +559,34 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(la28);
+        L(l6d8);
         test(M, 0x1);
-        jle(la44, T_NEAR);
+        jle(l6f4, T_NEAR);
         vmovd(xmm0, dword[A1 - 0x80]);
         vmovd(dword[B - 0x80], xmm0);
         sub(B, -4);
         align(4);
 
-        L(la44);
+        L(l6f4);
         sub(N, 0x2);
         cmp(N, 0x2);
-        jge(l930, T_NEAR);
+        jge(l5e0, T_NEAR);
         align(4);
 
-        L(la52);
+        L(l702);
         cmp(N, 0x1);
-        jl(lb84, T_NEAR);
+        jl(l834, T_NEAR);
         align(4);
 
-        L(la5c);
+        L(l70c);
         mov(A1, A);
         add(A, 0x2);
         mov(LDA3, M);
         sar(LDA3, 0x3);
-        jle(lae4, T_NEAR);
+        jle(l794, T_NEAR);
         align(4);
 
-        L(la6c);
+        L(l71c);
         mov(ax, word[A1 - 0x80]);
         add(A1, LDA);
         vpinsrw(xmm0, xmm0, eax, 0x0);
@@ -790,12 +614,12 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         vmovdqu(xword[B - 0x80], xmm0);
         sub(B, -16);
         dec(LDA3);
-        jg(la6c, T_NEAR);
+        jg(l71c, T_NEAR);
         align(4);
 
-        L(lae4);
+        L(l794);
         test(M, 0x4);
-        jle(lb2c, T_NEAR);
+        jle(l7dc, T_NEAR);
         mov(ax, word[A1 - 0x80]);
         add(A1, LDA);
         vpinsrw(xmm0, xmm0, eax, 0x0);
@@ -812,9 +636,9 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(lb2c);
+        L(l7dc);
         test(M, 0x2);
-        jle(lb5c, T_NEAR);
+        jle(l80c, T_NEAR);
         mov(ax, word[A1 - 0x80]);
         add(A1, LDA);
         vpinsrw(xmm0, xmm0, eax, 0x0);
@@ -825,22 +649,22 @@ void jit_avx512_core_s16_48x8_copy_an_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(lb5c);
+        L(l80c);
         test(M, 0x1);
-        jle(lb74, T_NEAR);
+        jle(l824, T_NEAR);
         mov(ax, word[A1 - 0x80]);
         mov(word[B - 0x80], ax);
         sub(B, -2);
         align(4);
 
-        L(lb74);
+        L(l824);
         sub(N, 0x1);
         cmp(N, 0x1);
-        jge(la5c, T_NEAR);
+        jge(l70c, T_NEAR);
         align(4);
 
-        L(lb84);
-
+        L(l834);
+        vzeroupper();
         postamble();
     }
     outLocalLabel();
