@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -15,12 +15,16 @@
 *******************************************************************************/
 
 #include <atomic>
+#include <memory>
 #include <mutex>
 
 #include "common/dnnl_thread.hpp"
 #include "common/utils.hpp"
 
 #include "cpu/platform.hpp"
+
+#include "cpu/gemm/gemm.hpp"
+
 #include "cpu/x64/gemm/f32/jit_avx512_core_gemm_smalln_tn_f32_kern.hpp"
 #include "cpu/x64/gemm/gemm_info.hpp"
 #include "cpu/x64/jit_generator.hpp"
@@ -42,12 +46,10 @@ namespace avx512_core_gemm_smalln_tn_f32 {
 struct xbyak_gemm_smalln_tn_t : public jit_generator {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_avx512_core_gemm_smalln_tn_xbyak_gemm)
 
-    xbyak_gemm_smalln_tn_t(int N, float beta, float alpha,
-            void *code_ptr = nullptr,
-            size_t code_size = 80 * Xbyak::DEFAULT_MAX_CODE_SIZE)
-        : jit_generator(code_ptr, code_size), N(N), beta(beta), alpha(alpha) {}
+    xbyak_gemm_smalln_tn_t(int N, float beta, float alpha)
+        : jit_generator(jit_name()), N(N), beta(beta), alpha(alpha) {}
 
-    void generate() override ATTRIBUTE_OPTIMIZE {
+    void generate() override {
         using namespace Xbyak;
         /**
          * numN = 1 : 16 rows of A, 1x16 accumulators
@@ -139,8 +141,8 @@ struct xbyak_gemm_smalln_tn_t : public jit_generator {
                 ptr[rsp + tempzmm_offset + 64 * 7]};
 
         int numMREM = (numN == 1) ? 16 : 8;
-        Label label_mremT[17], label_kremT[16], label_k_loopT[16],
-                label_no_k_remT[16];
+        std::vector<Label> label_mremT(17), label_kremT(16), label_k_loopT(16),
+                label_no_k_remT(16);
 
         zmm_reg = zmmreg;
         TEMPZMM = TEMPZMM_;
@@ -689,7 +691,8 @@ dnnl_status_t jump_to_gemm_smalln_tn(
         const gemm_info_t<float, float, float> *arg) {
     if ((arg->n < 16 && arg->n > 1 && arg->transa == do_trans
                 && arg->transb != do_trans)
-            && mayiuse(avx512_core) && arg->co == nullptr) {
+            && mayiuse(avx512_core) && __BUILD_GEMM_AVX512
+            && arg->co == nullptr) {
         auto transa_char = (arg->transa != do_trans) ? "N" : "T";
         auto transb_char = (arg->transb != do_trans) ? "N" : "T";
         return jit_avx512_core_gemm_smalln_tn_f32(transa_char, transb_char,
@@ -723,16 +726,16 @@ dnnl_status_t sgemm_smalln_tn(const dim_t m, const dim_t n, const dim_t k,
         const dim_t ldb, const float beta, float *C, const dim_t ldc) {
     using namespace avx512_core_gemm_smalln_tn_f32;
 
-    static xbyak_gemm_smalln_tn_t *kernels[4][3][3];
+    static maybe_unique_ptr<xbyak_gemm_smalln_tn_t> kernels[4][3][3];
     static std::once_flag initialized;
 
-    dnnl_status_t st = dnnl_success;
+    static dnnl_status_t st = dnnl_success;
     std::call_once(initialized, [&] {
         for (dim_t N : {1, 2, 3, 4}) {
             for (float al : {0.0f, 1.0f, 2.0f}) {
                 for (float be : {0.0f, 1.0f, 2.0f}) {
                     auto &kern = kernels[N - 1][(dim_t)al][(dim_t)be];
-                    kern = new xbyak_gemm_smalln_tn_t(N, be, al);
+                    kern.reset(new xbyak_gemm_smalln_tn_t(N, be, al));
                     st = kern->create_kernel();
                     if (st != dnnl_success) return;
                 }
