@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ namespace cpu {
 namespace x64 {
 
 jit_avx_f32_copy_an_kern::jit_avx_f32_copy_an_kern()
-    : jit_generator(nullptr, F32_COPY_KERNEL_CODE_SIZE) {}
+    : jit_generator(jit_name()) {}
 
 void jit_avx_f32_copy_an_kern::generate() {
 
@@ -60,103 +60,7 @@ void jit_avx_f32_copy_an_kern::generate() {
 
     inLocalLabel();
     {
-
-        Xbyak::Label l10cc;
-        Xbyak::Label l116c;
-        Xbyak::Label l11c0;
-        Xbyak::Label l11f0;
-        Xbyak::Label l1200;
-        Xbyak::Label l1224;
-        Xbyak::Label l12c4;
-        Xbyak::Label l1318;
-        Xbyak::Label l134c;
-        Xbyak::Label l136c;
-        Xbyak::Label l1370;
-        Xbyak::Label l1394;
-        Xbyak::Label l1430;
-        Xbyak::Label l1484;
-        Xbyak::Label l14b8;
-        Xbyak::Label l14d8;
-        Xbyak::Label l14dc;
-        Xbyak::Label l1500;
-        Xbyak::Label l159c;
-        Xbyak::Label l15f0;
-        Xbyak::Label l1624;
-        Xbyak::Label l1644;
-        Xbyak::Label l1648;
-        Xbyak::Label l166c;
-        Xbyak::Label l168;
-        Xbyak::Label l1708;
-        Xbyak::Label l175c;
-        Xbyak::Label l1790;
-        Xbyak::Label l17b0;
-        Xbyak::Label l17b4;
-        Xbyak::Label l1e4;
-        Xbyak::Label l228;
-        Xbyak::Label l250;
-        Xbyak::Label l260;
-        Xbyak::Label l280;
-        Xbyak::Label l2fc;
-        Xbyak::Label l340;
-        Xbyak::Label l36c;
-        Xbyak::Label l388;
-        Xbyak::Label l38c;
-        Xbyak::Label l3ac;
-        Xbyak::Label l424;
-        Xbyak::Label l468;
-        Xbyak::Label l494;
-        Xbyak::Label l4b0;
-        Xbyak::Label l4b4;
-        Xbyak::Label l4d4;
-        Xbyak::Label l54;
-        Xbyak::Label l54c;
-        Xbyak::Label l590;
-        Xbyak::Label l5bc;
-        Xbyak::Label l5d8;
-        Xbyak::Label l5dc;
-        Xbyak::Label l5fc;
-        Xbyak::Label l674;
-        Xbyak::Label l6b8;
-        Xbyak::Label l6c;
-        Xbyak::Label l6e4;
-        Xbyak::Label l700;
-        Xbyak::Label l704;
-        Xbyak::Label l70c;
-        Xbyak::Label l728;
-        Xbyak::Label l740;
-        Xbyak::Label l87c;
-        Xbyak::Label l91c;
-        Xbyak::Label l970;
-        Xbyak::Label l9a0;
-        Xbyak::Label l9b0;
-        Xbyak::Label l9d4;
-        Xbyak::Label la74;
-        Xbyak::Label lac8;
-        Xbyak::Label lafc;
-        Xbyak::Label lb1c;
-        Xbyak::Label lb20;
-        Xbyak::Label lb44;
-        Xbyak::Label lbe0;
-        Xbyak::Label lc34;
-        Xbyak::Label lc68;
-        Xbyak::Label lc88;
-        Xbyak::Label lc8c;
-        Xbyak::Label lcb0;
-        Xbyak::Label ld4c;
-        Xbyak::Label lda0;
-        Xbyak::Label ldd4;
-        Xbyak::Label ldf4;
-        Xbyak::Label ldf8;
-        Xbyak::Label le1c;
-        Xbyak::Label leb8;
-        Xbyak::Label lf0c;
-        Xbyak::Label lf40;
-        Xbyak::Label lf60;
-        Xbyak::Label lf64;
-        Xbyak::Label lf6c;
-        Xbyak::Label lf78;
-        Xbyak::Label lf90;
-
+        std::vector<Xbyak::Label> labels(95);
         preamble();
 #ifdef _WIN32
         auto stacksize = get_size_of_abi_save_regs();
@@ -180,21 +84,21 @@ void jit_avx_f32_copy_an_kern::generate() {
         vpslld(xmm4, xmm4, 0x1f);
         vperm2f128(ymm4, ymm4, ymm4, 0x20);
         vucomiss(xmm6, xmm3);
-        jne(l70c, T_NEAR);
+        jne(labels[58], T_NEAR);
         cmp(N, 0x10);
-        jl(l260, T_NEAR);
+        jl(labels[33], T_NEAR);
         align(4);
 
-        L(l54);
+        L(labels[93]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x40);
         mov(I, M);
         sar(I, 0x3);
-        jle(l168, T_NEAR);
+        jle(labels[24], T_NEAR);
         align(4);
 
-        L(l6c);
+        L(labels[94]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -231,12 +135,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -512);
         dec(I);
-        jg(l6c, T_NEAR);
+        jg(labels[94], T_NEAR);
         align(4);
 
-        L(l168);
+        L(labels[24]);
         test(M, 0x4);
-        jle(l1e4, T_NEAR);
+        jle(labels[30], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -257,9 +161,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -256);
         align(4);
 
-        L(l1e4);
+        L(labels[30]);
         test(M, 0x2);
-        jle(l228, T_NEAR);
+        jle(labels[31], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -272,9 +176,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l228);
+        L(labels[31]);
         test(M, 0x1);
-        jle(l250, T_NEAR);
+        jle(labels[32], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -282,24 +186,24 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l250);
+        L(labels[32]);
         sub(N, 0x10);
         cmp(N, 0x10);
-        jge(l54, T_NEAR);
+        jge(labels[93], T_NEAR);
         align(4);
 
-        L(l260);
+        L(labels[33]);
         cmp(N, 0x8);
-        jl(l38c, T_NEAR);
+        jl(labels[39], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x20);
         mov(I, M);
         sar(I, 0x3);
-        jle(l2fc, T_NEAR);
+        jle(labels[35], T_NEAR);
         align(4);
 
-        L(l280);
+        L(labels[34]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 + LDA * 1 - 0x80]);
@@ -320,12 +224,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -256);
         dec(I);
-        jg(l280, T_NEAR);
+        jg(labels[34], T_NEAR);
         align(4);
 
-        L(l2fc);
+        L(labels[35]);
         test(M, 0x4);
-        jle(l340, T_NEAR);
+        jle(labels[36], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 + LDA * 1 - 0x80]);
@@ -338,9 +242,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l340);
+        L(labels[36]);
         test(M, 0x2);
-        jle(l36c, T_NEAR);
+        jle(labels[37], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 + LDA * 1 - 0x80]);
@@ -349,30 +253,30 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l36c);
+        L(labels[37]);
         test(M, 0x1);
-        jle(l388, T_NEAR);
+        jle(labels[38], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         sub(B, -32);
         align(4);
 
-        L(l388);
+        L(labels[38]);
         sub(N, 0x8);
         align(4);
 
-        L(l38c);
+        L(labels[39]);
         cmp(N, 0x4);
-        jl(l4b4, T_NEAR);
+        jl(labels[45], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x10);
         mov(I, M);
         sar(I, 0x3);
-        jle(l424, T_NEAR);
+        jle(labels[41], T_NEAR);
         align(4);
 
-        L(l3ac);
+        L(labels[40]);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmovups(xword[B - 0x80], xmm0);
         vmovups(xmm0, xword[A1 + LDA * 1 - 0x80]);
@@ -393,12 +297,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -128);
         dec(I);
-        jg(l3ac, T_NEAR);
+        jg(labels[40], T_NEAR);
         align(4);
 
-        L(l424);
+        L(labels[41]);
         test(M, 0x4);
-        jle(l468, T_NEAR);
+        jle(labels[42], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmovups(xword[B - 0x80], xmm0);
         vmovups(xmm0, xword[A1 + LDA * 1 - 0x80]);
@@ -411,9 +315,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l468);
+        L(labels[42]);
         test(M, 0x2);
-        jle(l494, T_NEAR);
+        jle(labels[43], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmovups(xword[B - 0x80], xmm0);
         vmovups(xmm0, xword[A1 + LDA * 1 - 0x80]);
@@ -422,30 +326,30 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l494);
+        L(labels[43]);
         test(M, 0x1);
-        jle(l4b0, T_NEAR);
+        jle(labels[44], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmovups(xword[B - 0x80], xmm0);
         sub(B, -16);
         align(4);
 
-        L(l4b0);
+        L(labels[44]);
         sub(N, 0x4);
         align(4);
 
-        L(l4b4);
+        L(labels[45]);
         cmp(N, 0x2);
-        jl(l5dc, T_NEAR);
+        jl(labels[51], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x8);
         mov(I, M);
         sar(I, 0x3);
-        jle(l54c, T_NEAR);
+        jle(labels[47], T_NEAR);
         align(4);
 
-        L(l4d4);
+        L(labels[46]);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmovlps(qword[B - 0x80], xmm0);
         vmovsd(xmm0, qword[A1 + LDA * 1 - 0x80]);
@@ -466,12 +370,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -64);
         dec(I);
-        jg(l4d4, T_NEAR);
+        jg(labels[46], T_NEAR);
         align(4);
 
-        L(l54c);
+        L(labels[47]);
         test(M, 0x4);
-        jle(l590, T_NEAR);
+        jle(labels[48], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmovlps(qword[B - 0x80], xmm0);
         vmovsd(xmm0, qword[A1 + LDA * 1 - 0x80]);
@@ -484,9 +388,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l590);
+        L(labels[48]);
         test(M, 0x2);
-        jle(l5bc, T_NEAR);
+        jle(labels[49], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmovlps(qword[B - 0x80], xmm0);
         vmovsd(xmm0, qword[A1 + LDA * 1 - 0x80]);
@@ -495,30 +399,30 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l5bc);
+        L(labels[49]);
         test(M, 0x1);
-        jle(l5d8, T_NEAR);
+        jle(labels[50], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmovlps(qword[B - 0x80], xmm0);
         sub(B, -8);
         align(4);
 
-        L(l5d8);
+        L(labels[50]);
         sub(N, 0x2);
         align(4);
 
-        L(l5dc);
+        L(labels[51]);
         cmp(N, 0x1);
-        jl(l704, T_NEAR);
+        jl(labels[57], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x4);
         mov(I, M);
         sar(I, 0x3);
-        jle(l674, T_NEAR);
+        jle(labels[53], T_NEAR);
         align(4);
 
-        L(l5fc);
+        L(labels[52]);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmovss(dword[B - 0x80], xmm0);
         vmovss(xmm0, dword[A1 + LDA * 1 - 0x80]);
@@ -539,12 +443,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -32);
         dec(I);
-        jg(l5fc, T_NEAR);
+        jg(labels[52], T_NEAR);
         align(4);
 
-        L(l674);
+        L(labels[53]);
         test(M, 0x4);
-        jle(l6b8, T_NEAR);
+        jle(labels[54], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmovss(dword[B - 0x80], xmm0);
         vmovss(xmm0, dword[A1 + LDA * 1 - 0x80]);
@@ -557,9 +461,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l6b8);
+        L(labels[54]);
         test(M, 0x2);
-        jle(l6e4, T_NEAR);
+        jle(labels[55], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmovss(dword[B - 0x80], xmm0);
         vmovss(xmm0, dword[A1 + LDA * 1 - 0x80]);
@@ -568,41 +472,41 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l6e4);
+        L(labels[55]);
         test(M, 0x1);
-        jle(l700, T_NEAR);
+        jle(labels[56], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmovss(dword[B - 0x80], xmm0);
         sub(B, -4);
         align(4);
 
-        L(l700);
+        L(labels[56]);
         sub(N, 0x1);
         align(4);
 
-        L(l704);
-        jmp(l17b4, T_NEAR);
+        L(labels[57]);
+        jmp(labels[29], T_NEAR);
         align(4);
 
-        L(l70c);
+        L(labels[58]);
         vxorps(xmm3, xmm3, xmm4);
         vucomiss(xmm6, xmm3);
-        jne(lf6c, T_NEAR);
+        jne(labels[90], T_NEAR);
         vmovaps(ymm6, ymm4);
         cmp(N, 0x10);
-        jl(l9b0, T_NEAR);
+        jl(labels[65], T_NEAR);
         align(4);
 
-        L(l728);
+        L(labels[59]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x40);
         mov(I, M);
         sar(I, 0x3);
-        jle(l87c, T_NEAR);
+        jle(labels[61], T_NEAR);
         align(4);
 
-        L(l740);
+        L(labels[60]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -655,12 +559,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -512);
         dec(I);
-        jg(l740, T_NEAR);
+        jg(labels[60], T_NEAR);
         align(4);
 
-        L(l87c);
+        L(labels[61]);
         test(M, 0x4);
-        jle(l91c, T_NEAR);
+        jle(labels[62], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -689,9 +593,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -256);
         align(4);
 
-        L(l91c);
+        L(labels[62]);
         test(M, 0x2);
-        jle(l970, T_NEAR);
+        jle(labels[63], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -708,9 +612,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l970);
+        L(labels[63]);
         test(M, 0x1);
-        jle(l9a0, T_NEAR);
+        jle(labels[64], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -720,24 +624,24 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l9a0);
+        L(labels[64]);
         sub(N, 0x10);
         cmp(N, 0x10);
-        jge(l728, T_NEAR);
+        jge(labels[59], T_NEAR);
         align(4);
 
-        L(l9b0);
+        L(labels[65]);
         cmp(N, 0x8);
-        jl(lb20, T_NEAR);
+        jl(labels[71], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x20);
         mov(I, M);
         sar(I, 0x3);
-        jle(la74, T_NEAR);
+        jle(labels[67], T_NEAR);
         align(4);
 
-        L(l9d4);
+        L(labels[66]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -766,12 +670,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -256);
         dec(I);
-        jg(l9d4, T_NEAR);
+        jg(labels[66], T_NEAR);
         align(4);
 
-        L(la74);
+        L(labels[67]);
         test(M, 0x4);
-        jle(lac8, T_NEAR);
+        jle(labels[68], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -788,9 +692,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(lac8);
+        L(labels[68]);
         test(M, 0x2);
-        jle(lafc, T_NEAR);
+        jle(labels[69], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -801,31 +705,31 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(lafc);
+        L(labels[69]);
         test(M, 0x1);
-        jle(lb1c, T_NEAR);
+        jle(labels[70], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
         sub(B, -32);
         align(4);
 
-        L(lb1c);
+        L(labels[70]);
         sub(N, 0x8);
         align(4);
 
-        L(lb20);
+        L(labels[71]);
         cmp(N, 0x4);
-        jl(lc8c, T_NEAR);
+        jl(labels[77], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x10);
         mov(I, M);
         sar(I, 0x3);
-        jle(lbe0, T_NEAR);
+        jle(labels[73], T_NEAR);
         align(4);
 
-        L(lb44);
+        L(labels[72]);
         vmovups(xmm0, xword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -854,12 +758,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -128);
         dec(I);
-        jg(lb44, T_NEAR);
+        jg(labels[72], T_NEAR);
         align(4);
 
-        L(lbe0);
+        L(labels[73]);
         test(M, 0x4);
-        jle(lc34, T_NEAR);
+        jle(labels[74], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -876,9 +780,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(lc34);
+        L(labels[74]);
         test(M, 0x2);
-        jle(lc68, T_NEAR);
+        jle(labels[75], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -889,31 +793,31 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(lc68);
+        L(labels[75]);
         test(M, 0x1);
-        jle(lc88, T_NEAR);
+        jle(labels[76], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
         sub(B, -16);
         align(4);
 
-        L(lc88);
+        L(labels[76]);
         sub(N, 0x4);
         align(4);
 
-        L(lc8c);
+        L(labels[77]);
         cmp(N, 0x2);
-        jl(ldf8, T_NEAR);
+        jl(labels[83], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x8);
         mov(I, M);
         sar(I, 0x3);
-        jle(ld4c, T_NEAR);
+        jle(labels[79], T_NEAR);
         align(4);
 
-        L(lcb0);
+        L(labels[78]);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -942,12 +846,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -64);
         dec(I);
-        jg(lcb0, T_NEAR);
+        jg(labels[78], T_NEAR);
         align(4);
 
-        L(ld4c);
+        L(labels[79]);
         test(M, 0x4);
-        jle(lda0, T_NEAR);
+        jle(labels[80], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -964,9 +868,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(lda0);
+        L(labels[80]);
         test(M, 0x2);
-        jle(ldd4, T_NEAR);
+        jle(labels[81], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -977,31 +881,31 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(ldd4);
+        L(labels[81]);
         test(M, 0x1);
-        jle(ldf4, T_NEAR);
+        jle(labels[82], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
         sub(B, -8);
         align(4);
 
-        L(ldf4);
+        L(labels[82]);
         sub(N, 0x2);
         align(4);
 
-        L(ldf8);
+        L(labels[83]);
         cmp(N, 0x1);
-        jl(lf64, T_NEAR);
+        jl(labels[89], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x4);
         mov(I, M);
         sar(I, 0x3);
-        jle(leb8, T_NEAR);
+        jle(labels[85], T_NEAR);
         align(4);
 
-        L(le1c);
+        L(labels[84]);
         vmovss(xmm0, dword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -1030,12 +934,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -32);
         dec(I);
-        jg(le1c, T_NEAR);
+        jg(labels[84], T_NEAR);
         align(4);
 
-        L(leb8);
+        L(labels[85]);
         test(M, 0x4);
-        jle(lf0c, T_NEAR);
+        jle(labels[86], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -1052,9 +956,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(lf0c);
+        L(labels[86]);
         test(M, 0x2);
-        jle(lf40, T_NEAR);
+        jle(labels[87], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -1065,38 +969,38 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(lf40);
+        L(labels[87]);
         test(M, 0x1);
-        jle(lf60, T_NEAR);
+        jle(labels[88], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
         sub(B, -4);
         align(4);
 
-        L(lf60);
+        L(labels[88]);
         sub(N, 0x1);
         align(4);
 
-        L(lf64);
-        jmp(l17b4, T_NEAR);
+        L(labels[89]);
+        jmp(labels[29], T_NEAR);
         align(4);
 
-        L(lf6c);
+        L(labels[90]);
         cmp(N, 0x10);
-        jl(l1200, T_NEAR);
+        jl(labels[4], T_NEAR);
         align(4);
 
-        L(lf78);
+        L(labels[91]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x40);
         mov(I, M);
         sar(I, 0x3);
-        jle(l10cc, T_NEAR);
+        jle(labels[0], T_NEAR);
         align(4);
 
-        L(lf90);
+        L(labels[92]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1149,12 +1053,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -512);
         dec(I);
-        jg(lf90, T_NEAR);
+        jg(labels[92], T_NEAR);
         align(4);
 
-        L(l10cc);
+        L(labels[0]);
         test(M, 0x4);
-        jle(l116c, T_NEAR);
+        jle(labels[1], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1183,9 +1087,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -256);
         align(4);
 
-        L(l116c);
+        L(labels[1]);
         test(M, 0x2);
-        jle(l11c0, T_NEAR);
+        jle(labels[2], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1202,9 +1106,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l11c0);
+        L(labels[2]);
         test(M, 0x1);
-        jle(l11f0, T_NEAR);
+        jle(labels[3], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1214,24 +1118,24 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l11f0);
+        L(labels[3]);
         sub(N, 0x10);
         cmp(N, 0x10);
-        jge(lf78, T_NEAR);
+        jge(labels[91], T_NEAR);
         align(4);
 
-        L(l1200);
+        L(labels[4]);
         cmp(N, 0x8);
-        jl(l1370, T_NEAR);
+        jl(labels[10], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x20);
         mov(I, M);
         sar(I, 0x3);
-        jle(l12c4, T_NEAR);
+        jle(labels[6], T_NEAR);
         align(4);
 
-        L(l1224);
+        L(labels[5]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1260,12 +1164,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -256);
         dec(I);
-        jg(l1224, T_NEAR);
+        jg(labels[5], T_NEAR);
         align(4);
 
-        L(l12c4);
+        L(labels[6]);
         test(M, 0x4);
-        jle(l1318, T_NEAR);
+        jle(labels[7], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1282,9 +1186,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l1318);
+        L(labels[7]);
         test(M, 0x2);
-        jle(l134c, T_NEAR);
+        jle(labels[8], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1295,31 +1199,31 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l134c);
+        L(labels[8]);
         test(M, 0x1);
-        jle(l136c, T_NEAR);
+        jle(labels[9], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
         sub(B, -32);
         align(4);
 
-        L(l136c);
+        L(labels[9]);
         sub(N, 0x8);
         align(4);
 
-        L(l1370);
+        L(labels[10]);
         cmp(N, 0x4);
-        jl(l14dc, T_NEAR);
+        jl(labels[16], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x10);
         mov(I, M);
         sar(I, 0x3);
-        jle(l1430, T_NEAR);
+        jle(labels[12], T_NEAR);
         align(4);
 
-        L(l1394);
+        L(labels[11]);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -1348,12 +1252,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -128);
         dec(I);
-        jg(l1394, T_NEAR);
+        jg(labels[11], T_NEAR);
         align(4);
 
-        L(l1430);
+        L(labels[12]);
         test(M, 0x4);
-        jle(l1484, T_NEAR);
+        jle(labels[13], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -1370,9 +1274,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l1484);
+        L(labels[13]);
         test(M, 0x2);
-        jle(l14b8, T_NEAR);
+        jle(labels[14], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -1383,31 +1287,31 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l14b8);
+        L(labels[14]);
         test(M, 0x1);
-        jle(l14d8, T_NEAR);
+        jle(labels[15], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
         sub(B, -16);
         align(4);
 
-        L(l14d8);
+        L(labels[15]);
         sub(N, 0x4);
         align(4);
 
-        L(l14dc);
+        L(labels[16]);
         cmp(N, 0x2);
-        jl(l1648, T_NEAR);
+        jl(labels[22], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x8);
         mov(I, M);
         sar(I, 0x3);
-        jle(l159c, T_NEAR);
+        jle(labels[18], T_NEAR);
         align(4);
 
-        L(l1500);
+        L(labels[17]);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -1436,12 +1340,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -64);
         dec(I);
-        jg(l1500, T_NEAR);
+        jg(labels[17], T_NEAR);
         align(4);
 
-        L(l159c);
+        L(labels[18]);
         test(M, 0x4);
-        jle(l15f0, T_NEAR);
+        jle(labels[19], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -1458,9 +1362,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l15f0);
+        L(labels[19]);
         test(M, 0x2);
-        jle(l1624, T_NEAR);
+        jle(labels[20], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -1471,31 +1375,31 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l1624);
+        L(labels[20]);
         test(M, 0x1);
-        jle(l1644, T_NEAR);
+        jle(labels[21], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
         sub(B, -8);
         align(4);
 
-        L(l1644);
+        L(labels[21]);
         sub(N, 0x2);
         align(4);
 
-        L(l1648);
+        L(labels[22]);
         cmp(N, 0x1);
-        jl(l17b4, T_NEAR);
+        jl(labels[29], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x4);
         mov(I, M);
         sar(I, 0x3);
-        jle(l1708, T_NEAR);
+        jle(labels[25], T_NEAR);
         align(4);
 
-        L(l166c);
+        L(labels[23]);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -1524,12 +1428,12 @@ void jit_avx_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -32);
         dec(I);
-        jg(l166c, T_NEAR);
+        jg(labels[23], T_NEAR);
         align(4);
 
-        L(l1708);
+        L(labels[25]);
         test(M, 0x4);
-        jle(l175c, T_NEAR);
+        jle(labels[26], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -1546,9 +1450,9 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l175c);
+        L(labels[26]);
         test(M, 0x2);
-        jle(l1790, T_NEAR);
+        jle(labels[27], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -1559,20 +1463,20 @@ void jit_avx_f32_copy_an_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l1790);
+        L(labels[27]);
         test(M, 0x1);
-        jle(l17b0, T_NEAR);
+        jle(labels[28], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
         sub(B, -4);
         align(4);
 
-        L(l17b0);
+        L(labels[28]);
         sub(N, 0x1);
         align(4);
 
-        L(l17b4);
+        L(labels[29]);
 
         postamble();
     }
