@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -19,6 +19,8 @@
 #include <malloc.h>
 #endif
 
+#include <vector>
+
 #include "oneapi/dnnl/dnnl_types.h"
 
 #include "common/bfloat16.hpp"
@@ -29,6 +31,7 @@
 #include "cpu/platform.hpp"
 
 #include "cpu/gemm/f32/gemm_utils_f32.hpp"
+#include "cpu/gemm/gemm.hpp"
 #include "cpu/gemm/gemm_msan_unpoison.hpp"
 
 #include "cpu/x64/jit_generator.hpp"
@@ -50,6 +53,11 @@ namespace dnnl {
 namespace impl {
 namespace cpu {
 namespace x64 {
+#ifndef DNNL_WITH_SYCL
+#define MAX_STACK_SZ (4 * PAGE_4K)
+#else
+#define MAX_STACK_SZ 0
+#endif
 
 template <typename c_type>
 struct alignas(64) gemm_per_thread_t {
@@ -66,14 +74,25 @@ struct alignas(64) gemm_per_thread_t {
 
 template <typename T>
 int get_vector_length() {
-    int v_bytes;
+    int v_bytes = 0;
 
-    if (mayiuse(avx512_core))
+    if (false) {
+        //dummy if
+#if __BUILD_GEMM_AVX512
+    } else if (mayiuse(avx512_core)) {
         v_bytes = cpu_isa_traits<avx512_core>::vlen;
-    else if (mayiuse(avx))
+#endif
+#if __BUILD_GEMM_AVX2
+    } else if (mayiuse(avx)) {
         v_bytes = cpu_isa_traits<avx>::vlen;
-    else
+#endif
+#if __BUILD_GEMM_SSE41
+    } else if (mayiuse(sse41)) {
         v_bytes = cpu_isa_traits<sse41>::vlen;
+#endif
+    } else {
+        assert(!"not supposed to be reached.");
+    }
 
     return v_bytes / sizeof(T);
 }
@@ -208,7 +227,7 @@ static inline dim_t get_m_padd_parallel_a(int ithr, dim_t m,
     if (!arg->a_packed) {
         constexpr auto multiplier = 10;
 
-        m_padd *= nstl::max(nthrs, multiplier);
+        m_padd *= nstl::min(nthrs, multiplier);
         if (m_padd > m) m_padd = utils::rnd_up(m, arg->um);
     }
 
@@ -374,36 +393,38 @@ template <typename a_type, typename b_type, typename c_type>
 void gemm_kernel(dim_t m, dim_t n, const dim_t k, const float alpha,
         const a_type *a, const b_type *b, float beta, c_type *c,
         const dim_t ldc, const c_type *a_row_sum, const c_type *b_col_sum,
-        const c_type *co, offset_type offsetc,
-        const gemm_info_t<a_type, b_type, c_type> *arg) {
-
-#if DNNL_WITH_SYCL
-    std::vector<c_type> col_offset_vec(m);
-    std::vector<c_type> row_offset_vec(n);
-    c_type *col_offset = col_offset_vec.data();
-    c_type *row_offset = row_offset_vec.data();
-#else
-    // Since m and n are limited by blocking, stack overflow may not happen;
-    // it's up to 32kB
-#if !defined(_MSC_VER)
-    c_type col_offset[m];
-    c_type row_offset[n];
-#else
-    c_type *col_offset = (c_type *)_alloca(sizeof(*col_offset) * m);
-    c_type *row_offset = (c_type *)_alloca(sizeof(*row_offset) * n);
-#endif
-#endif
+        c_type *row_offset_ws, c_type *col_offset_ws, const c_type *co,
+        offset_type offsetc, const gemm_info_t<a_type, b_type, c_type> *arg) {
 
     bool col_req = false;
     bool row_req = false;
 
     constexpr bool is_int8 = utils::one_of(
             data_traits<a_type>::data_type, data_type::s8, data_type::u8);
-    constexpr bool is_bf16 = data_traits<a_type>::data_type == data_type::bf16;
     constexpr bool is_f32 = data_traits<a_type>::data_type == data_type::f32;
-    bool is_int8_amx = is_int8 && mayiuse(avx512_core_bf16_amx_int8);
-    bool is_bf16_amx = is_bf16 && mayiuse(avx512_core_bf16_amx_bf16);
-    bool is_amx = is_int8_amx || is_bf16_amx;
+    bool is_int8_amx = is_int8 && mayiuse(avx512_core_amx) && __BUILD_GEMM_AMX;
+
+    dim_t m_stk = col_offset_ws ? 1 : m;
+    dim_t n_stk = row_offset_ws ? 1 : n;
+#if !defined(_MSC_VER)
+    std::vector<c_type> col_offset_stk_vec(m_stk);
+    std::vector<c_type> row_offset_stk_vec(n_stk);
+    c_type *col_offset_stk = col_offset_stk_vec.data();
+    c_type *row_offset_stk = row_offset_stk_vec.data();
+
+#else
+    c_type *col_offset_stk = nullptr;
+    if (!col_offset_ws)
+        col_offset_stk = (c_type *)_alloca(sizeof *col_offset_stk * m_stk);
+
+    c_type *row_offset_stk = nullptr;
+    if (!row_offset_ws)
+        row_offset_stk = (c_type *)_alloca(sizeof *row_offset_stk * n_stk);
+#endif
+
+    // Use the heap if already allocated and stack otherwise.
+    c_type *col_offset = col_offset_ws ? col_offset_ws : col_offset_stk;
+    c_type *row_offset = row_offset_ws ? row_offset_ws : row_offset_stk;
 
     if (is_int8) {
         c_type ao = arg->ao;
@@ -478,46 +499,13 @@ void gemm_kernel(dim_t m, dim_t n, const dim_t k, const float alpha,
 
     bool isBeta0 = beta == 0.0f;
 
-    dim_t align_m = 0;
-    dim_t align_n = 0;
-    dim_t align_k = k;
-    if (is_amx) {
-        align_m = m - utils::rnd_dn(m, arg->um);
-        align_n = n - utils::rnd_dn(n, arg->un);
-        align_k = utils::rnd_up(k, arg->uk);
-    }
-    m -= align_m;
-    n -= align_n;
-
     /* Column and row offsets are ignored by non-integer compute kernels.
      * Scaling is done only for bfloat16 kernels.
      */
-    if (m > 0) {
-        if (n > 0) {
-            arg->kernel[isBeta0][col_req][row_req](&m, &n, &align_k, &alpha, a,
-                    b, c, ldc, col_offset, row_offset);
-        }
-        if (align_n > 0) {
-            arg->kernel[isBeta0][col_req][row_req](&m, &align_n, &align_k,
-                    &alpha, a, b + n * align_k, c + n * ldc, ldc, col_offset,
-                    row_offset + n);
-        }
-    }
-    if (align_m > 0) {
-        if (n > 0) {
-            arg->kernel[isBeta0][col_req][row_req](&align_m, &n, &align_k,
-                    &alpha, a + m * align_k, b, c + m, ldc, col_offset + m,
-                    row_offset);
-        }
-        if (align_n > 0) {
-            arg->kernel[isBeta0][col_req][row_req](&align_m, &align_n, &align_k,
-                    &alpha, a + m * align_k, b + n * align_k, c + m + n * ldc,
-                    ldc, col_offset + m, row_offset + n);
-        }
-    }
+    if (m > 0 && n > 0)
+        arg->kernel[isBeta0][col_req][row_req](
+                &m, &n, &k, &alpha, a, b, c, ldc, col_offset, row_offset);
 
-    m += align_m;
-    n += align_n;
     msan_unpoison_matrix(c, m, n, ldc, sizeof(*c));
 
     // sgemm kernels don't support bias yet.
@@ -561,8 +549,9 @@ static dnnl_status_t gemm_kernel_driver(int ithr, dim_t m, dim_t n, dim_t k,
     constexpr bool is_int8 = utils::one_of(
             data_traits<a_type>::data_type, data_type::s8, data_type::u8);
     constexpr bool is_bf16 = data_traits<a_type>::data_type == data_type::bf16;
-    bool is_int8_amx = is_int8 && mayiuse(avx512_core_bf16_amx_int8);
-    bool is_bf16_amx = is_bf16 && mayiuse(avx512_core_bf16_amx_bf16);
+
+    bool is_int8_amx = is_int8 && mayiuse(avx512_core_amx) && __BUILD_GEMM_AMX;
+    bool is_bf16_amx = is_bf16 && mayiuse(avx512_core_amx) && __BUILD_GEMM_AMX;
     bool is_amx = is_int8_amx || is_bf16_amx;
 
     const std::shared_ptr<const gemm_pack_storage_t> &a_packed = arg->a_packed;
@@ -617,6 +606,15 @@ static dnnl_status_t gemm_kernel_driver(int ithr, dim_t m, dim_t n, dim_t k,
                 + b_col_sum_nelems * sizeof(*c) + PAGE_4K;
     }
 
+    size_t col_offset_ws_nelems = arg->um;
+    size_t row_offset_ws_nelems = n_padd;
+    size_t stk_sz = (col_offset_ws_nelems + row_offset_ws_nelems) * sizeof(*c);
+    const bool use_stack = is_int8 && stk_sz <= MAX_STACK_SZ;
+    if (!use_stack) {
+        mem_size += col_offset_ws_nelems * sizeof(*c) + PAGE_4K;
+        mem_size += row_offset_ws_nelems * sizeof(*c) + PAGE_4K;
+    }
+
     bool need_c_buffer
             = (is_int8 && (alpha != 1.0f || (beta != 1.0f && beta != 0.0f)))
             // AMX bfloat16 kernels don't support alpha scaling yet,
@@ -636,22 +634,33 @@ static dnnl_status_t gemm_kernel_driver(int ithr, dim_t m, dim_t n, dim_t k,
     }
 
     a_type *bufferA = (a_type *)align(mem, PAGE_4K);
-    b_type *bufferB = (b_type *)align(bufferA + a_buf_nelems, PAGE_4K);
+    void *p_next_buf = bufferA + a_buf_nelems;
+
+    b_type *bufferB = (b_type *)align(p_next_buf, PAGE_4K);
+    p_next_buf = bufferB + b_buf_nelems;
 
     c_type *a_row_sum = nullptr;
     c_type *b_col_sum = nullptr;
     if (is_int8) {
-        a_row_sum = (c_type *)align(bufferB + b_buf_nelems, PAGE_4K);
-        b_col_sum = (c_type *)align(a_row_sum + a_row_sum_nelems, PAGE_4K);
+        a_row_sum = (c_type *)align(p_next_buf, PAGE_4K);
+        p_next_buf = a_row_sum + a_row_sum_nelems;
+
+        b_col_sum = (c_type *)align(p_next_buf, PAGE_4K);
+        p_next_buf = b_col_sum + b_col_sum_nelems;
+    }
+
+    c_type *col_offset_ws = nullptr;
+    c_type *row_offset_ws = nullptr;
+    if (!use_stack) {
+        col_offset_ws = (c_type *)align(p_next_buf, PAGE_4K);
+        p_next_buf = col_offset_ws + col_offset_ws_nelems;
+
+        row_offset_ws = (c_type *)align(p_next_buf, PAGE_4K);
+        p_next_buf = row_offset_ws + row_offset_ws_nelems;
     }
 
     c_type *bufferC = nullptr;
-    if (need_c_buffer) {
-        if (is_int8)
-            bufferC = (c_type *)align(b_col_sum + b_col_sum_nelems, PAGE_4K);
-        else
-            bufferC = (c_type *)align(bufferB + b_buf_nelems, PAGE_4K);
-    }
+    if (need_c_buffer) bufferC = (c_type *)align(p_next_buf, PAGE_4K);
 
     int a_block_copied = 0;
     dim_t sizeM = 0;
@@ -762,7 +771,8 @@ static dnnl_status_t gemm_kernel_driver(int ithr, dim_t m, dim_t n, dim_t k,
                     if (need_c_buffer) {
                         gemm_kernel(sizeUM, sizeN, sizeK, 1.0f, bufferA_eff,
                                 bufferB, 0.0f, bufferC + Um, ldc_buf,
-                                a_row_sum_eff, b_col_sum, (c_type *)nullptr,
+                                a_row_sum_eff, b_col_sum, row_offset_ws,
+                                col_offset_ws, (c_type *)nullptr,
                                 offset_type::none, arg);
 
                         /* Finish the block adding the necessary alpha, beta
@@ -774,7 +784,8 @@ static dnnl_status_t gemm_kernel_driver(int ithr, dim_t m, dim_t n, dim_t k,
                     } else {
                         gemm_kernel(sizeUM, sizeN, sizeK, alpha, bufferA_eff,
                                 bufferB, beta_eff, c_block, ldc, a_row_sum_eff,
-                                b_col_sum, co + co_stride, offsetc_eff, arg);
+                                b_col_sum, row_offset_ws, col_offset_ws,
+                                co + co_stride, offsetc_eff, arg);
                     }
                 }
                 a_block_copied = 1;
@@ -814,15 +825,14 @@ static dnnl_status_t kernel_driver_parallel_acopiedbcopy(int ithr, dim_t m,
 
     size_t b_buf_nelems = k * n_padd;
     size_t b_col_sum_nelems = n_padd;
-
     constexpr bool is_int8 = utils::one_of(
             data_traits<a_type>::data_type, data_type::s8, data_type::u8);
     constexpr bool is_bf16 = data_traits<a_type>::data_type == data_type::bf16;
-    bool is_int8_amx = is_int8 && mayiuse(avx512_core_bf16_amx_int8);
-    bool is_bf16_amx = is_bf16 && mayiuse(avx512_core_bf16_amx_bf16);
-    bool is_amx = is_int8_amx || is_bf16_amx;
 
-    // B buffer needs to large due to zero-padding.
+    bool is_int8_amx = is_int8 && mayiuse(avx512_core_amx) && __BUILD_GEMM_AMX;
+    bool is_bf16_amx = is_bf16 && mayiuse(avx512_core_amx) && __BUILD_GEMM_AMX;
+    bool is_amx = is_int8_amx || is_bf16_amx;
+    // B buffer needs to be large due to zero-padding.
     if (is_amx)
         b_buf_nelems
                 = utils::rnd_up(k, arg->uk) * utils::rnd_up(n_padd, arg->un);
@@ -833,6 +843,15 @@ static dnnl_status_t kernel_driver_parallel_acopiedbcopy(int ithr, dim_t m,
 
     if (is_int8) { mem_size += b_col_sum_nelems * sizeof(*c) + PAGE_4K; }
 
+    size_t col_offset_ws_nelems = m;
+    size_t row_offset_ws_nelems = n_padd;
+    size_t stk_sz = (col_offset_ws_nelems + row_offset_ws_nelems) * sizeof(*c);
+    const bool use_stack = is_int8 && stk_sz <= MAX_STACK_SZ;
+    if (!use_stack) {
+        mem_size += col_offset_ws_nelems * sizeof(*c) + PAGE_4K;
+        mem_size += row_offset_ws_nelems * sizeof(*c) + PAGE_4K;
+    }
+
     bool need_c_buffer
             = (is_int8 && (alpha != 1.0f || (beta != 1.0f && beta != 0.0f)))
             // AMX bfloat16 kernels don't support alpha scaling yet,
@@ -852,19 +871,26 @@ static dnnl_status_t kernel_driver_parallel_acopiedbcopy(int ithr, dim_t m,
     }
 
     b_type *bufferB = (b_type *)align(mem, PAGE_4K);
+    void *p_next_buf = bufferB + b_buf_nelems;
 
     c_type *b_col_sum = nullptr;
     if (is_int8) {
-        b_col_sum = (c_type *)align(bufferB + b_buf_nelems, PAGE_4K);
+        b_col_sum = (c_type *)align(p_next_buf, PAGE_4K);
+        p_next_buf = b_col_sum + b_col_sum_nelems;
+    }
+
+    c_type *col_offset_ws = nullptr;
+    c_type *row_offset_ws = nullptr;
+    if (!use_stack) {
+        col_offset_ws = (c_type *)align(p_next_buf, PAGE_4K);
+        p_next_buf = col_offset_ws + col_offset_ws_nelems;
+
+        row_offset_ws = (c_type *)align(p_next_buf, PAGE_4K);
+        p_next_buf = row_offset_ws + row_offset_ws_nelems;
     }
 
     c_type *bufferC = nullptr;
-    if (need_c_buffer) {
-        if (is_int8)
-            bufferC = (c_type *)align(b_col_sum + b_col_sum_nelems, PAGE_4K);
-        else
-            bufferC = (c_type *)align(bufferB + b_buf_nelems, PAGE_4K);
-    }
+    if (need_c_buffer) bufferC = (c_type *)align(p_next_buf, PAGE_4K);
 
     dim_t sizeN = 0;
     for (dim_t Bn = 0; Bn < n; Bn += sizeN) {
@@ -898,15 +924,16 @@ static dnnl_status_t kernel_driver_parallel_acopiedbcopy(int ithr, dim_t m,
         c_type *c_block = c + Bn * ldc;
         if (need_c_buffer) {
             gemm_kernel(m, sizeN, k, 1.0f, bufferA, bufferB, 0.0f, bufferC,
-                    ldc_buf, a_row_sum, b_col_sum, (c_type *)nullptr,
-                    offset_type::none, arg);
+                    ldc_buf, a_row_sum, b_col_sum, row_offset_ws, col_offset_ws,
+                    (c_type *)nullptr, offset_type::none, arg);
 
             // Finish the block adding the necessary alpha, beta and offsets.
             add_results(m, sizeN, alpha, beta, bufferC, ldc_buf, c_block, ldc,
                     co + co_stride, offsetc);
         } else {
             gemm_kernel(m, sizeN, k, alpha, bufferA, bufferB, beta, c_block,
-                    ldc, a_row_sum, b_col_sum, co + co_stride, offsetc, arg);
+                    ldc, a_row_sum, b_col_sum, row_offset_ws, col_offset_ws,
+                    co + co_stride, offsetc, arg);
         }
     }
 
@@ -929,9 +956,9 @@ static inline bool nocopy_checker_avx2(const int nthr, const int transa,
     // Crude threshold to nocopy kernels if copy overhead is significant.
     if (1.0 / m + 1.0 / n >= FORCE_NOCOPY_THRESH) { return true; }
 
-    if (m <= 378 && n <= 378 && k >= nthr * 378) return false;
-
-    if (m >= nthr * 378 && k >= nthr * 378) return false;
+    const dim_t LIMIT = 378;
+    if (m <= LIMIT && n <= LIMIT && k >= nthr * LIMIT) return false;
+    if (m >= nthr * LIMIT && k >= nthr * LIMIT) return false;
 
     if (transb == no_trans) {
         if (m <= MN_NOCOPY_AVX2 && n <= MN_NOCOPY_AVX2) return true;
@@ -960,8 +987,9 @@ static inline bool nocopy_checker_avx512(int nthr, const int transa,
     static const dim_t K_NOTRANSB_PER_THR = 1;
     static const double FORCE_NOCOPY_THRESH = 0.00196;
 
-    bool is_NT_case = transa == no_trans && transb == do_trans;
-    bool is_TN_case = transa == do_trans && transb == no_trans;
+    bool is_NN = transa == no_trans && transb == no_trans;
+    bool is_NT = transa == no_trans && transb == do_trans;
+    bool is_TN = transa == do_trans && transb == no_trans;
 
     bool is_lda_bad = lda % BAD_LD_MULT == 0;
     bool is_ldb_bad = ldb % BAD_LD_MULT == 0;
@@ -971,14 +999,19 @@ static inline bool nocopy_checker_avx512(int nthr, const int transa,
     bool is_lda_verybad = lda % VERYBAD_LD_MULT == 0;
 
     // Copy-based performs better for TN case with small N in sequential case.
-    if (nthr == 1 && is_TN_case && m > 100
+    if (nthr == 1 && is_TN && m > 100
             && ((m < 1200 && n < 200 && k < 1200)
                     || (is_lda_bad && is_ldb_bad)))
         return false;
 
+    // Copy-based performs better for NN case on very bad leading dimension if
+    // each thread has enough work.
+    if (nthr <= 8 && is_NN && is_lda_verybad && k > 500 && n > 100)
+        return false;
+
     // Crude threshold for nocopy kernels if copy overhead is significant.
     if (1.0 / m + 1.0 / n >= FORCE_NOCOPY_THRESH
-            && !(is_lda_verybad && is_NT_case)) {
+            && !(is_lda_verybad && is_NT)) {
         return true;
     }
 
@@ -997,9 +1030,9 @@ static inline bool nocopy_checker_avx512(int nthr, const int transa,
         if (use_copy_based) return false;
     }
 
-    if (m <= 378 && n <= 378 && k >= nthr * 378) return false;
-
-    if (m >= nthr * 378 && k >= nthr * 378) return false;
+    const dim_t LIMIT = 378;
+    if (m <= LIMIT && n <= LIMIT && k >= nthr * LIMIT) return false;
+    if (m >= nthr * LIMIT && k >= nthr * LIMIT) return false;
 
     if (transb == no_trans) {
         if (m <= nthr * MN_NOTRANSB_PER_THR) return true;
@@ -1019,25 +1052,32 @@ static inline bool nocopy_checker(
 
     if (data_traits<a_type>::data_type != data_type::f32) return false;
 
-    if (!mayiuse(avx)) return false;
+    if (!(mayiuse(avx) && __BUILD_GEMM_AVX2)) return false;
 
     if (arg->force_nocopy) return true;
 
     auto m = arg->m, n = arg->n, k = arg->k;
+    UNUSED(m), UNUSED(n), UNUSED(k);
     auto lda = arg->lda, ldb = arg->ldb, ldc = arg->ldc;
+    UNUSED(lda), UNUSED(ldb), UNUSED(ldc);
     auto transa = arg->transa, transb = arg->transb;
+    UNUSED(transa), UNUSED(transb);
     auto packing = arg->packing;
 
     if (packing != pack_type::none) ldc = 64;
 
-    if (arg->a_packed || arg->b_packed)
-        return false;
+    if (arg->a_packed || arg->b_packed) return false;
+#if __BUILD_GEMM_AVX512
     else if (mayiuse(avx512_core))
         return nocopy_checker_avx512(
                 nthr, transa, transb, m, n, k, lda, ldb, ldc);
+#endif
+#if __BUILD_GEMM_AVX2
     else
         return nocopy_checker_avx2(
                 nthr, transa, transb, m, n, k, lda, ldb, ldc);
+#endif
+    return false;
 }
 
 template <typename a_type, typename b_type, typename c_type>
@@ -1070,26 +1110,32 @@ static inline void set_thread_opts_nopack(int nthrs, int nthrs_spawn,
     bool condition_2D_bsrc = false;
     if (isSgemm) {
         // If m is large and n is small then do 1D partitioning for AVX2.
-        if (!mayiuse(avx512_core) && n <= N2D_MAX && (m >= nthrs * M2D_MIN))
+        if (!(mayiuse(avx512_core) && __BUILD_GEMM_AVX512) && n <= N2D_MAX
+                && (m >= nthrs * M2D_MIN)) {
             condition_2D_bsrc = false;
-        else
+        } else {
             condition_2D_bsrc
                     = ((n > nthrs * N2D_MAX) || (n <= nthrs * N2D_MAX / 2))
                     && (m >= 2 * M2D_MIN);
+        }
     } else {
-        int scale = mayiuse(avx512_core) ? nthrs : 20;
+        int scale = (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) ? nthrs : 20;
         condition_2D_bsrc = (256 * m > scale * n) && (scale * m < 256 * n);
     }
 
     // TODO Check if we should use k-partitioning.
 
     int condition_1D_copya = false;
-    if (mayiuse(avx512_core)) {
+    if (false) {
+        // dummy if
+#if __BUILD_GEMM_AVX512
+    } else if (mayiuse(avx512_core)) {
         const dim_t thresh = isSgemm ? N2D_MAX / 4 : 68;
         if (m >= 1000 && (n >= nthrs * thresh)) {
             condition_2D_bsrc = false;
             condition_1D_copya = true;
         }
+#endif
     } else {
         if (m >= 1000 && n >= 4000) {
             condition_2D_bsrc = false;
@@ -1102,7 +1148,9 @@ static inline void set_thread_opts_nopack(int nthrs, int nthrs_spawn,
     // TODO: the reasons seems to be in copy_sum_bx routines. At least,
     //       after simple optimization of copy_sum_ax for avx512, similar
     //       restriction on offset B became unnecessary. Revisit.
-    if (is_int8 && arg->ao != 0 && (arg->bo != 0 || mayiuse(avx512_core))) {
+    if (is_int8 && arg->ao != 0
+            && (arg->bo != 0
+                    || (mayiuse(avx512_core) && __BUILD_GEMM_AVX512))) {
         condition_2D_bsrc = false;
         condition_1D_copya = true;
     }
@@ -1145,7 +1193,7 @@ static inline void set_thread_opts_nopack(int nthrs, int nthrs_spawn,
             } else if ((n <= 64 || n >= 256)) {
                 while (((nthrs_n > 1) && (n / nthrs_n < arg->un)
                                && (m / nthrs_m >= 2 * arg->um)
-                               && mayiuse(avx512_core))
+                               && mayiuse(avx512_core) && __BUILD_GEMM_AVX512)
                         || ((nthrs_n % 2 == 0)
                                 && (n / nthrs > N2D_MAX
                                         || n / nthrs_n <= N2D_MAX / 2)
@@ -1273,7 +1321,8 @@ static inline void set_thread_opts_pack(int nthrs,
     choose_k_blocking();
 
     // Choose m/n blocking.
-    auto min_mblk = mayiuse(avx512_core) ? (MBLK / 2) : arg->um;
+    auto min_mblk = (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) ? (MBLK / 2)
+                                                                  : arg->um;
     min_mblk = do_m_blocking ? min_mblk : m;
     min_mblk = do_m_blocking_only ? arg->um : min_mblk;
     auto min_nblk = do_n_blocking ? NBLK / 2 : n;
@@ -1327,9 +1376,13 @@ static inline int set_thread_opts(int nthrs, int nthrs_spawn,
         dim_t BK = 0;
         auto m = arg->m, n = arg->n, k = arg->k;
 
-        if (mayiuse(avx512_core)) {
+        if (false) {
+            // dummy if
+#if __BUILD_GEMM_AVX512
+        } else if (mayiuse(avx512_core)) {
             cpu::gemm_utils::calc_nthr_nocopy_avx512_common(m, n, k, nthrs,
                     &nthrs_m, &nthrs_n, &nthrs_k, &BM, &BN, &BK);
+#endif
         } else {
             cpu::gemm_utils::calc_nthr_nocopy_avx(m, n, k, nthrs, &nthrs_m,
                     &nthrs_n, &nthrs_k, &BM, &BN, &BK);
@@ -1361,9 +1414,12 @@ decompose_matrices(const gemm_slice_t &slice,
     dim_t stride_bn = (arg->transb != no_trans) ? 1 : arg->ldb;
     dim_t stride_bk = (arg->transb == no_trans) ? 1 : arg->ldb;
 
-    auto a = arg->a + slice.off_m * stride_am + slice.off_k * stride_ak;
-    auto b = arg->b + slice.off_n * stride_bn + slice.off_k * stride_bk;
-    auto c = arg->c + slice.off_m + slice.off_n * arg->ldc;
+    auto a = arg->a;
+    auto b = arg->b;
+    auto c = arg->c;
+    if (a) a += slice.off_m * stride_am + slice.off_k * stride_ak;
+    if (b) b += slice.off_n * stride_bn + slice.off_k * stride_bk;
+    if (c) c += slice.off_m + slice.off_n * arg->ldc;
 
     dim_t co_stride;
     switch (arg->offsetc) {
@@ -1371,7 +1427,8 @@ decompose_matrices(const gemm_slice_t &slice,
         case offset_type::column: co_stride = slice.off_m; break;
         default: co_stride = 0; break;
     }
-    auto co = arg->co + co_stride;
+    auto co = arg->co;
+    if (co) co += co_stride;
 
     return std::make_tuple(a, b, c, co);
 }
@@ -1397,10 +1454,9 @@ static dnnl_status_t parallel_a_copy(const int ithr, const int nthrs,
     constexpr bool is_int8 = utils::one_of(
             data_traits<a_type>::data_type, data_type::s8, data_type::u8);
     constexpr bool is_bf16 = data_traits<a_type>::data_type == data_type::bf16;
-    bool is_int8_amx = is_int8 && mayiuse(avx512_core_bf16_amx_int8);
-    bool is_bf16_amx = is_bf16 && mayiuse(avx512_core_bf16_amx_bf16);
+    bool is_int8_amx = is_int8 && mayiuse(avx512_core_amx) && __BUILD_GEMM_AMX;
+    bool is_bf16_amx = is_bf16 && mayiuse(avx512_core_amx) && __BUILD_GEMM_AMX;
     bool is_amx = is_int8_amx || is_bf16_amx;
-
     const std::shared_ptr<const gemm_pack_storage_t> &a_packed = arg->a_packed;
 
     // Scaling C matrix.
@@ -1552,17 +1608,47 @@ static inline void adjust_thread_count(dim_t m, dim_t n, dim_t k, int *nthrs) {
     auto veclen = get_vector_length<T>();
     const double fp_per_cycle = 2.0 * 2.0 * veclen;
 
-    if (mayiuse(avx2) && !mayiuse(avx512_core))
+    const bool is_f32 = data_traits<T>::data_type == data_type::f32;
+
+    const bool is_avx512 = mayiuse(avx512_core) && __BUILD_GEMM_AVX512;
+    const bool is_avx = mayiuse(avx) && __BUILD_GEMM_AVX2;
+    const bool is_only_avx2 = mayiuse(avx2) && __BUILD_GEMM_AVX2 && !is_avx512;
+
+    // Some sgemm cases still benefit from using all threads.
+    const bool use_all_threads = is_f32 && n > 50
+            && ((is_avx && m <= 3) || (is_avx512 && m <= 10));
+    if (use_all_threads) return;
+
+    if (is_only_avx2)
         if (m > 10 * n && n < *nthrs)
             if (m / *nthrs < veclen * 3)
                 *nthrs = nstl::max(m / veclen / 3, dim_t(1));
 
     double gemm_cycles = m * n * k / fp_per_cycle;
-    if (data_traits<T>::data_type == data_type::f32) {
-        gemm_cycles *= 2.0;
-    } else {
-        gemm_cycles *= 8.0;
+    gemm_cycles *= is_f32 ? 2.0 : 8.0;
+
+#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_THREADPOOL
+    if (is_f32) {
+        static auto l2_cache_per_thread = platform::get_per_core_cache_size(2);
+        static int n_cores_per_socket
+                = static_cast<int>(platform::get_num_cores());
+        auto l2_cache_socket = l2_cache_per_thread * n_cores_per_socket;
+        auto problem_memory_footprint = (m * n + m * k + n * k) * sizeof(float);
+
+        if (is_only_avx2) {
+            // Somehow it seems beneficial to split the job into bigger pieces.
+            // Use L2 per-core cache size as a deal-breaker.
+            int use_n_threads = utils::div_up(
+                    problem_memory_footprint, l2_cache_per_thread);
+            *nthrs = nstl::min(*nthrs, use_n_threads);
+            return;
+        }
+        if (l2_cache_socket > problem_memory_footprint) {
+            *nthrs = nstl::min(*nthrs, n_cores_per_socket);
+            return;
+        }
     }
+#endif
 
     int i = *nthrs;
 
@@ -1605,24 +1691,34 @@ static inline void adjust_thread_count(dim_t m, dim_t n, dim_t k, int *nthrs) {
 
 template <typename a_type, typename b_type, typename c_type>
 static dnnl_status_t call_no_copy_sgemm(
-        gemm_info_t<a_type, b_type, c_type> *arg) {
+        int nthrs, gemm_info_t<a_type, b_type, c_type> *arg) {
 
     if (arg->packing == pack_type::none) {
+#if __BUILD_GEMM_AVX2
         auto transa_char = (arg->transa != do_trans) ? "N" : "T";
         auto transb_char = (arg->transb != do_trans) ? "N" : "T";
+#endif
 
-        if (mayiuse(avx512_core))
-            return jit_avx512_common_gemm_f32(transa_char, transb_char, &arg->m,
+        if (false) {
+            // dummy if
+#if __BUILD_GEMM_AVX512
+        } else if (mayiuse(avx512_core)) {
+            return jit_avx512_common_gemm_f32(nthrs, transa_char, transb_char,
+                    &arg->m, &arg->n, &arg->k, &arg->alpha, (float *)arg->a,
+                    &arg->lda, (float *)arg->b, &arg->ldb, &arg->beta,
+                    (float *)arg->c, &arg->ldc, (float *)arg->co);
+#endif
+#if __BUILD_GEMM_AVX2
+        } else if (mayiuse(avx)) {
+            return jit_avx_gemm_f32(nthrs, transa_char, transb_char, &arg->m,
                     &arg->n, &arg->k, &arg->alpha, (float *)arg->a, &arg->lda,
                     (float *)arg->b, &arg->ldb, &arg->beta, (float *)arg->c,
                     &arg->ldc, (float *)arg->co);
-        else
-            return jit_avx_gemm_f32(transa_char, transb_char, &arg->m, &arg->n,
-                    &arg->k, &arg->alpha, (float *)arg->a, &arg->lda,
-                    (float *)arg->b, &arg->ldb, &arg->beta, (float *)arg->c,
-                    &arg->ldc, (float *)arg->co);
+#endif
+        }
     } else
         return pack_no_copy(arg);
+    return status::unimplemented;
 }
 
 template <typename a_type, typename b_type, typename c_type>
@@ -1638,12 +1734,14 @@ static dnnl_status_t gemm_threading_driver(
 
     if ((arg->m <= 0) || (arg->n <= 0)) return dnnl_success;
 
+#if __BUILD_GEMM_AVX512
     if (!is_a_packed && !is_b_packed && jump_to_gemv_s8x8s32(arg))
         return dnnl_success;
 
     if (!is_a_packed && !is_b_packed
             && jump_to_gemm_smalln_tn(arg) == dnnl_success)
         return dnnl_success;
+#endif
 
     if (!is_a_packed && !is_b_packed && jump_to_gemv(arg) == dnnl_success)
         return dnnl_success;
@@ -1654,7 +1752,7 @@ static dnnl_status_t gemm_threading_driver(
     if (is_b_packed && arg->ao != 0)
         if (!arg->b_packed->has_col_sums()) return dnnl_invalid_arguments;
 
-    auto nthr_max = (dnnl_in_parallel()) ? 1 : dnnl_get_max_threads();
+    auto nthr_max = dnnl_get_current_num_threads();
     int nthr_goal = nthr_max;
 
     adjust_thread_count<c_type>(arg->m, arg->n, arg->k, &nthr_goal);
@@ -1742,7 +1840,8 @@ static dnnl_status_t gemm_threading_driver(
         if (arg->measure_only) return dnnl_success;
     }
 
-    if (nocopy_checker(nthr_goal, arg)) return call_no_copy_sgemm(arg);
+    if (nocopy_checker(nthr_goal, arg))
+        return call_no_copy_sgemm(nthr_goal, arg);
 
     if (nthr_goal == 1)
         return gemm_kernel_driver(0, arg->m, arg->n, arg->k, arg->a, arg->b,
@@ -1760,7 +1859,8 @@ static dnnl_status_t gemm_threading_driver(
     for (int ithr = 0; ithr < nthr_max; ithr++) {
         thread_arg[ithr].result = dnnl_success;
         thread_arg[ithr].compute_done = false;
-        thread_arg[ithr].c_local = thread_arg[ithr].c_global = nullptr;
+        thread_arg[ithr].c_local = nullptr;
+        thread_arg[ithr].c_global = nullptr;
         thread_arg[ithr].ldc_global = arg->ldc;
         thread_arg[ithr].ldc_local = 0;
 
@@ -1875,22 +1975,31 @@ static dnnl_status_t gemm_threading_driver(
                                 == data_type::f32);
                         assert(arg->packing == pack_type::none);
 
-                        if (mayiuse(avx512_core)) {
-                            avx512_common_gemm_f32::sgemm_nocopy_driver(
-                                    arg->transa == no_trans ? "N" : "T",
-                                    arg->transb == no_trans ? "N" : "T", m, n,
-                                    k, &arg->alpha, (float *)a, arg->lda,
-                                    (float *)b, arg->ldb, &beta_eff,
-                                    (float *)c_eff, ldc_eff, nullptr, nullptr);
-                        } else {
-                            avx_gemm_f32::sgemm_nocopy_driver(
-                                    arg->transa == no_trans ? "N" : "T",
-                                    arg->transb == no_trans ? "N" : "T", m, n,
-                                    k, &arg->alpha, (float *)a, arg->lda,
-                                    (float *)b, arg->ldb, &beta_eff,
-                                    (float *)c_eff, ldc_eff, nullptr, nullptr);
+                        if (false) {
+                            // dummy if
+#if __BUILD_GEMM_AVX512
+                        } else if (mayiuse(avx512_core)) {
+                            thread_arg[ithr].result = avx512_common_gemm_f32::
+                                    sgemm_nocopy_driver(
+                                            arg->transa == no_trans ? "N" : "T",
+                                            arg->transb == no_trans ? "N" : "T",
+                                            m, n, k, &arg->alpha, (float *)a,
+                                            arg->lda, (float *)b, arg->ldb,
+                                            &beta_eff, (float *)c_eff, ldc_eff,
+                                            nullptr);
+#endif
+#if __BUILD_GEMM_AVX2
+                        } else if (mayiuse(avx)) {
+                            thread_arg[ithr].result
+                                    = avx_gemm_f32::sgemm_nocopy_driver(
+                                            arg->transa == no_trans ? "N" : "T",
+                                            arg->transb == no_trans ? "N" : "T",
+                                            m, n, k, &arg->alpha, (float *)a,
+                                            arg->lda, (float *)b, arg->ldb,
+                                            &beta_eff, (float *)c_eff, ldc_eff,
+                                            nullptr);
+#endif
                         }
-                        thread_arg[ithr].result = dnnl_success;
                         break;
                 }
 
@@ -1942,25 +2051,33 @@ dnnl_status_t gemm_driver(const char *transA, const char *transB,
             data_traits<a_type>::data_type, data_type::s8, data_type::u8);
     MAYBE_UNUSED(is_int8);
 
+#if __BUILD_GEMM_AVX512
     // gemm_driver supports bfloat16 gemm for Intel AVX512 and
     // Intel AVX512 BF16.
     assert(IMPLICATION(data_traits<a_type>::data_type == data_type::bf16,
             mayiuse(avx512_core) && !force_nocopy));
+#endif
 
+#if __BUILD_GEMM_SSE41
     // gemm_driver supports 8-bit integer Intel AVX512, Intel AVX2, Intel AVX,
     // Intel SSE4.1 and Intel DL Boost.
-    assert(IMPLICATION(is_int8, mayiuse(sse41) && !mayiuse(avx512_mic)));
+    assert(IMPLICATION(is_int8, mayiuse(sse41)));
+#endif
 
+#if __BUILD_GEMM_SSE41
     // gemm_driver supports sgemm for Intel AVX512, Intel AVX2, Intel AVX,
     // and Intel SSE4.1
     assert(IMPLICATION(
             data_traits<a_type>::data_type == data_type::f32, mayiuse(sse41)));
+#endif
 
     // 8-bit integer gemm doesn't support nocopy kernels.
     assert(IMPLICATION(is_int8, !force_nocopy));
 
+#if __BUILD_GEMM_AVX2
     // gemm_driver can only dispatch nocopy for avx and above.
     assert(IMPLICATION(force_nocopy, mayiuse(avx)));
+#endif
 
     gemm_info_t<a_type, b_type, c_type> args(transA, transB, offsetC, m, n, k,
             alpha, a, lda, oa, b, ldb, ob, beta, c, ldc, oc, force_nocopy,
@@ -2016,6 +2133,7 @@ template // Instantiate sgemm
                 pack_type packing, gemm_pack_storage_t *pack_dst,
                 bool measure_only);
 
+#undef MAX_STACK_SZ
 } // namespace x64
 } // namespace cpu
 } // namespace impl
