@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ namespace cpu {
 namespace x64 {
 
 jit_avx2_vnni_u8_copy_at_kern::jit_avx2_vnni_u8_copy_at_kern()
-    : jit_generator(nullptr, U8_COPY_KERNEL_CODE_SIZE) {}
+    : jit_generator(jit_name()) {}
 
 void jit_avx2_vnni_u8_copy_at_kern::generate() {
 
@@ -62,55 +62,7 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
 
     inLocalLabel();
     {
-
-        Xbyak::Label l1024;
-        Xbyak::Label l1090;
-        Xbyak::Label l10a0;
-        Xbyak::Label l10ac;
-        Xbyak::Label l10c4;
-        Xbyak::Label l1138;
-        Xbyak::Label l1188;
-        Xbyak::Label l11cc;
-        Xbyak::Label l1214;
-        Xbyak::Label l1254;
-        Xbyak::Label l1264;
-        Xbyak::Label l1270;
-        Xbyak::Label l1288;
-        Xbyak::Label l12c0;
-        Xbyak::Label l12ec;
-        Xbyak::Label l1318;
-        Xbyak::Label l1348;
-        Xbyak::Label l1368;
-        Xbyak::Label l1376;
-        Xbyak::Label l1380;
-        Xbyak::Label l1390;
-        Xbyak::Label l13ac;
-        Xbyak::Label l13cc;
-        Xbyak::Label l13ec;
-        Xbyak::Label l1408;
-        Xbyak::Label l1420;
-        Xbyak::Label l1430;
-        Xbyak::Label l20;
-        Xbyak::Label l2d8;
-        Xbyak::Label l40;
-        Xbyak::Label l468;
-        Xbyak::Label l5a4;
-        Xbyak::Label l6e8;
-        Xbyak::Label l81c;
-        Xbyak::Label l82c;
-        Xbyak::Label l838;
-        Xbyak::Label l854;
-        Xbyak::Label la04;
-        Xbyak::Label lb14;
-        Xbyak::Label lbf0;
-        Xbyak::Label lccc;
-        Xbyak::Label ld9c;
-        Xbyak::Label ldac;
-        Xbyak::Label ldb8;
-        Xbyak::Label ldd4;
-        Xbyak::Label leb0;
-        Xbyak::Label lf40;
-        Xbyak::Label lfb0;
+        std::vector<Xbyak::Label> labels(48);
 
         preamble();
 #ifdef _WIN32
@@ -126,10 +78,10 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -128);
         lea(LDA3, ptr[LDA + LDA * 2]);
         cmp(N, 0x18);
-        jl(l82c, T_NEAR);
+        jl(labels[34], T_NEAR);
         align(4);
 
-        L(l20);
+        L(labels[27]);
         mov(A1, A);
         mov(I, LDA);
         shl(I, 0x4);
@@ -137,10 +89,10 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         add(A, I);
         mov(I, M);
         sar(I, 0x4);
-        jle(l2d8, T_NEAR);
+        jle(labels[28], T_NEAR);
         align(4);
 
-        L(l40);
+        L(labels[29]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         movdqu(xmm2, xword[A1 + LDA * 2 - 0x80]);
@@ -270,12 +222,12 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(A1, -16);
         sub(B, -384);
         dec(I);
-        jg(l40, T_NEAR);
+        jg(labels[29], T_NEAR);
         align(4);
 
-        L(l2d8);
+        L(labels[28]);
         test(M, 0x8);
-        jle(l468, T_NEAR);
+        jle(labels[30], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         movq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         movq(xmm2, qword[A1 + LDA * 2 - 0x80]);
@@ -351,9 +303,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -192);
         align(4);
 
-        L(l468);
+        L(labels[30]);
         test(M, 0x4);
-        jle(l5a4, T_NEAR);
+        jle(labels[31], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         movd(xmm2, dword[A1 + LDA * 2 - 0x80]);
@@ -412,9 +364,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -96);
         align(4);
 
-        L(l5a4);
+        L(labels[31]);
         test(M, 0x2);
-        jle(l6e8, T_NEAR);
+        jle(labels[32], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -476,9 +428,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -48);
         align(4);
 
-        L(l6e8);
+        L(labels[32]);
         test(M, 0x1);
-        jle(l81c, T_NEAR);
+        jle(labels[33], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -537,28 +489,28 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -24);
         align(4);
 
-        L(l81c);
+        L(labels[33]);
         sub(N, 0x18);
         cmp(N, 0x18);
-        jge(l20, T_NEAR);
+        jge(labels[27], T_NEAR);
         align(4);
 
-        L(l82c);
+        L(labels[34]);
         cmp(N, 0x10);
-        jl(ldac, T_NEAR);
+        jl(labels[42], T_NEAR);
         align(4);
 
-        L(l838);
+        L(labels[35]);
         mov(A1, A);
         mov(I, LDA);
         shl(I, 0x4);
         add(A, I);
         mov(I, M);
         sar(I, 0x4);
-        jle(la04, T_NEAR);
+        jle(labels[37], T_NEAR);
         align(4);
 
-        L(l854);
+        L(labels[36]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         movdqu(xmm2, xword[A1 + LDA * 2 - 0x80]);
@@ -646,12 +598,12 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(A1, -16);
         sub(B, -256);
         dec(I);
-        jg(l854, T_NEAR);
+        jg(labels[36], T_NEAR);
         align(4);
 
-        L(la04);
+        L(labels[37]);
         test(M, 0x8);
-        jle(lb14, T_NEAR);
+        jle(labels[38], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         movq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         movq(xmm2, qword[A1 + LDA * 2 - 0x80]);
@@ -703,9 +655,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(lb14);
+        L(labels[38]);
         test(M, 0x4);
-        jle(lbf0, T_NEAR);
+        jle(labels[39], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         movd(xmm2, dword[A1 + LDA * 2 - 0x80]);
@@ -746,9 +698,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(lbf0);
+        L(labels[39]);
         test(M, 0x2);
-        jle(lccc, T_NEAR);
+        jle(labels[40], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -790,9 +742,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(lccc);
+        L(labels[40]);
         test(M, 0x1);
-        jle(ld9c, T_NEAR);
+        jle(labels[41], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -832,28 +784,28 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(ld9c);
+        L(labels[41]);
         sub(N, 0x10);
         cmp(N, 0x10);
-        jge(l838, T_NEAR);
+        jge(labels[35], T_NEAR);
         align(4);
 
-        L(ldac);
+        L(labels[42]);
         cmp(N, 0x8);
-        jl(l10a0, T_NEAR);
+        jl(labels[2], T_NEAR);
         align(4);
 
-        L(ldb8);
+        L(labels[43]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         lea(I, ptr[A1 + LDA * 8]);
         mov(A, I);
         mov(I, M);
         sar(I, 0x4);
-        jle(leb0, T_NEAR);
+        jle(labels[45], T_NEAR);
         align(4);
 
-        L(ldd4);
+        L(labels[44]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         movdqu(xmm2, xword[A1 + LDA * 2 - 0x80]);
@@ -898,12 +850,12 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         movdqu(xword[B - 0x10], xmm3);
         sub(B, -128);
         dec(I);
-        jg(ldd4, T_NEAR);
+        jg(labels[44], T_NEAR);
         align(4);
 
-        L(leb0);
+        L(labels[45]);
         test(M, 0x8);
-        jle(lf40, T_NEAR);
+        jle(labels[46], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         movq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         movq(xmm2, qword[A1 + LDA * 2 - 0x80]);
@@ -931,9 +883,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(lf40);
+        L(labels[46]);
         test(M, 0x4);
-        jle(lfb0, T_NEAR);
+        jle(labels[47], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         movd(xmm2, dword[A1 + LDA * 2 - 0x80]);
@@ -955,9 +907,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(lfb0);
+        L(labels[47]);
         test(M, 0x2);
-        jle(l1024, T_NEAR);
+        jle(labels[0], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -980,9 +932,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l1024);
+        L(labels[0]);
         test(M, 0x1);
-        jle(l1090, T_NEAR);
+        jle(labels[1], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -1003,28 +955,28 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l1090);
+        L(labels[1]);
         sub(N, 0x8);
         cmp(N, 0x8);
-        jge(ldb8, T_NEAR);
+        jge(labels[43], T_NEAR);
         align(4);
 
-        L(l10a0);
+        L(labels[2]);
         cmp(N, 0x4);
-        jl(l1264, T_NEAR);
+        jl(labels[10], T_NEAR);
         align(4);
 
-        L(l10ac);
+        L(labels[3]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 2]);
         lea(I, ptr[A1 + LDA * 4]);
         mov(A, I);
         mov(I, M);
         sar(I, 0x4);
-        jle(l1138, T_NEAR);
+        jle(labels[5], T_NEAR);
         align(4);
 
-        L(l10c4);
+        L(labels[4]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         sub(A1, -16);
@@ -1049,12 +1001,12 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         movdqu(xword[B - 0x50], xmm3);
         sub(B, -64);
         dec(I);
-        jg(l10c4, T_NEAR);
+        jg(labels[4], T_NEAR);
         align(4);
 
-        L(l1138);
+        L(labels[5]);
         test(M, 0x8);
-        jle(l1188, T_NEAR);
+        jle(labels[6], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         movq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         sub(A1, -8);
@@ -1071,9 +1023,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l1188);
+        L(labels[6]);
         test(M, 0x4);
-        jle(l11cc, T_NEAR);
+        jle(labels[7], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         sub(A1, -4);
@@ -1087,9 +1039,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l11cc);
+        L(labels[7]);
         test(M, 0x2);
-        jle(l1214, T_NEAR);
+        jle(labels[8], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -1104,9 +1056,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l1214);
+        L(labels[8]);
         test(M, 0x1);
-        jle(l1254, T_NEAR);
+        jle(labels[9], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -1119,28 +1071,28 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(l1254);
+        L(labels[9]);
         sub(N, 0x4);
         cmp(N, 0x4);
-        jge(l10ac, T_NEAR);
+        jge(labels[3], T_NEAR);
         align(4);
 
-        L(l1264);
+        L(labels[10]);
         cmp(N, 0x2);
-        jl(l1376, T_NEAR);
+        jl(labels[18], T_NEAR);
         align(4);
 
-        L(l1270);
+        L(labels[11]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 1]);
         lea(I, ptr[A1 + LDA * 2]);
         mov(A, I);
         mov(I, M);
         sar(I, 0x4);
-        jle(l12c0, T_NEAR);
+        jle(labels[13], T_NEAR);
         align(4);
 
-        L(l1288);
+        L(labels[12]);
         movdqu(xmm0, xword[A1 - 0x80]);
         sub(A1, -16);
         movdqu(xmm1, xword[A2 - 0x80]);
@@ -1152,12 +1104,12 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         movdqu(xword[B - 0x70], xmm2);
         sub(B, -32);
         dec(I);
-        jg(l1288, T_NEAR);
+        jg(labels[12], T_NEAR);
         align(4);
 
-        L(l12c0);
+        L(labels[13]);
         test(M, 0x8);
-        jle(l12ec, T_NEAR);
+        jle(labels[14], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         sub(A1, -8);
         movq(xmm1, qword[A2 - 0x80]);
@@ -1167,9 +1119,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l12ec);
+        L(labels[14]);
         test(M, 0x4);
-        jle(l1318, T_NEAR);
+        jle(labels[15], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         sub(A1, -4);
         movd(xmm1, dword[A2 - 0x80]);
@@ -1179,9 +1131,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l1318);
+        L(labels[15]);
         test(M, 0x2);
-        jle(l1348, T_NEAR);
+        jle(labels[16], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         sub(A1, -2);
         pinsrw(xmm0, eax, 0x0);
@@ -1192,9 +1144,9 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(l1348);
+        L(labels[16]);
         test(M, 0x1);
-        jle(l1368, T_NEAR);
+        jle(labels[17], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         mov(byte[B - 0x80], al);
         mov(al, byte[A2 - 0x80]);
@@ -1202,76 +1154,76 @@ void jit_avx2_vnni_u8_copy_at_kern::generate() {
         sub(B, -2);
         align(4);
 
-        L(l1368);
+        L(labels[17]);
         sub(N, 0x2);
         cmp(N, 0x2);
-        jge(l1270, T_NEAR);
+        jge(labels[11], T_NEAR);
         align(4);
 
-        L(l1376);
+        L(labels[18]);
         cmp(N, 0x1);
-        jl(l1430, T_NEAR);
+        jl(labels[26], T_NEAR);
         align(4);
 
-        L(l1380);
+        L(labels[19]);
         mov(A1, A);
         add(A, LDA);
         mov(I, M);
         sar(I, 0x4);
-        jle(l13ac, T_NEAR);
+        jle(labels[21], T_NEAR);
         align(4);
 
-        L(l1390);
+        L(labels[20]);
         movdqu(xmm0, xword[A1 - 0x80]);
         sub(A1, -16);
         movdqu(xword[B - 0x80], xmm0);
         sub(B, -16);
         dec(I);
-        jg(l1390, T_NEAR);
+        jg(labels[20], T_NEAR);
         align(4);
 
-        L(l13ac);
+        L(labels[21]);
         test(M, 0x8);
-        jle(l13cc, T_NEAR);
+        jle(labels[22], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         sub(A1, -8);
         movq(qword[B - 0x80], xmm0);
         sub(B, -8);
         align(4);
 
-        L(l13cc);
+        L(labels[22]);
         test(M, 0x4);
-        jle(l13ec, T_NEAR);
+        jle(labels[23], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         sub(A1, -4);
         movd(dword[B - 0x80], xmm0);
         sub(B, -4);
         align(4);
 
-        L(l13ec);
+        L(labels[23]);
         test(M, 0x2);
-        jle(l1408, T_NEAR);
+        jle(labels[24], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         mov(word[B - 0x80], ax);
         sub(A1, -2);
         sub(B, -2);
         align(4);
 
-        L(l1408);
+        L(labels[24]);
         test(M, 0x1);
-        jle(l1420, T_NEAR);
+        jle(labels[25], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         mov(byte[B - 0x80], al);
         sub(B, -1);
         align(4);
 
-        L(l1420);
+        L(labels[25]);
         sub(N, 0x1);
         cmp(N, 0x1);
-        jge(l1380, T_NEAR);
+        jge(labels[19], T_NEAR);
         align(4);
 
-        L(l1430);
+        L(labels[26]);
         postamble();
     }
     outLocalLabel();
