@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -25,7 +25,7 @@ namespace x64 {
 
 jit_avx512_core_u8_copy_sum_bn_kern::jit_avx512_core_u8_copy_sum_bn_kern(
         bool s8_case)
-    : jit_generator(nullptr, U8_COPY_KERNEL_CODE_SIZE), s8_case(s8_case) {}
+    : jit_generator(jit_name()), s8_case(s8_case) {}
 
 void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
 
@@ -66,39 +66,7 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
 
     inLocalLabel();
     {
-
-        Xbyak::Label l20;
-        Xbyak::Label l22c;
-        Xbyak::Label l340;
-        Xbyak::Label l3f8;
-        Xbyak::Label l48;
-        Xbyak::Label l498;
-        Xbyak::Label l51c;
-        Xbyak::Label l540;
-        Xbyak::Label l54c;
-        Xbyak::Label l56c;
-        Xbyak::Label l664;
-        Xbyak::Label l6f8;
-        Xbyak::Label l75c;
-        Xbyak::Label l7b4;
-        Xbyak::Label l7fc;
-        Xbyak::Label l81c;
-        Xbyak::Label l828;
-        Xbyak::Label l848;
-        Xbyak::Label l8d8;
-        Xbyak::Label l930;
-        Xbyak::Label l974;
-        Xbyak::Label l9b8;
-        Xbyak::Label l9ec;
-        Xbyak::Label la0a;
-        Xbyak::Label la14;
-        Xbyak::Label la28;
-        Xbyak::Label la6c;
-        Xbyak::Label laa8;
-        Xbyak::Label lae0;
-        Xbyak::Label lb14;
-        Xbyak::Label lb38;
-        Xbyak::Label lb58;
+        std::vector<Xbyak::Label> labels(32);
 
         preamble();
         auto stacksize = get_size_of_abi_save_regs();
@@ -112,13 +80,13 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         mov(A1, (size_t)&hbit);
         movdqu(xmm15, xword[A1]);
 
-        auto maybe_perform_s8_shift_xmm = [=](Xbyak::Xmm x) {
+        auto maybe_perform_s8_shift_xmm = [this](Xbyak::Xmm x) {
             if (s8_case) xorps(x, xmm15);
         };
-        auto maybe_perform_s8_shift_r8 = [=](const Xbyak::Reg8 &r) {
+        auto maybe_perform_s8_shift_r8 = [this](const Xbyak::Reg8 &r) {
             if (s8_case) xor_(r, (int8_t)0x80);
         };
-        auto maybe_perform_s8_shift_r16 = [=](const Xbyak::Reg16 &r) {
+        auto maybe_perform_s8_shift_r16 = [this](const Xbyak::Reg16 &r) {
             if (s8_case) xor_(r, (int16_t)0x8080);
         };
 
@@ -129,10 +97,10 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -128);
         lea(LDA3, ptr[LDA + LDA * 2]);
         cmp(N, 0x8);
-        jl(l540, T_NEAR);
+        jl(labels[7], T_NEAR);
         align(4);
 
-        L(l20);
+        L(labels[0]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         lea(I, ptr[A1 + LDA * 8]);
@@ -141,10 +109,10 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         pxor(xmm9, xmm9);
         mov(I, M);
         sar(I, 0x4);
-        jle(l22c, T_NEAR);
+        jle(labels[1], T_NEAR);
         align(4);
 
-        L(l48);
+        L(labels[4]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         movdqu(xmm2, xword[A1 + LDA * 2 - 0x80]);
@@ -253,12 +221,12 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         movdqu(xword[B - 0x10], xmm3);
         sub(B, -128);
         dec(I);
-        jg(l48, T_NEAR);
+        jg(labels[4], T_NEAR);
         align(4);
 
-        L(l22c);
+        L(labels[1]);
         test(M, 0x8);
-        jle(l340, T_NEAR);
+        jle(labels[2], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         movq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         movq(xmm2, qword[A1 + LDA * 2 - 0x80]);
@@ -318,9 +286,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l340);
+        L(labels[2]);
         test(M, 0x4);
-        jle(l3f8, T_NEAR);
+        jle(labels[3], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         movd(xmm2, dword[A1 + LDA * 2 - 0x80]);
@@ -358,9 +326,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l3f8);
+        L(labels[3]);
         test(M, 0x2);
-        jle(l498, T_NEAR);
+        jle(labels[5], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -393,9 +361,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l498);
+        L(labels[5]);
         test(M, 0x1);
-        jle(l51c, T_NEAR);
+        jle(labels[6], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -422,22 +390,22 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l51c);
+        L(labels[6]);
         mov(A1, qword[ARG_BIAS]);
         movdqu(xword[A1], xmm8);
         movdqu(xword[A1 + 0x10], xmm9);
         add(qword[ARG_BIAS], 0x20);
         sub(N, 0x8);
         cmp(N, 0x8);
-        jge(l20, T_NEAR);
+        jge(labels[0], T_NEAR);
         align(4);
 
-        L(l540);
+        L(labels[7]);
         cmp(N, 0x4);
-        jl(l81c, T_NEAR);
+        jl(labels[15], T_NEAR);
         align(4);
 
-        L(l54c);
+        L(labels[8]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 2]);
         lea(I, ptr[A1 + LDA * 4]);
@@ -445,10 +413,10 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         pxor(xmm7, xmm7);
         mov(I, M);
         sar(I, 0x4);
-        jle(l664, T_NEAR);
+        jle(labels[10], T_NEAR);
         align(4);
 
-        L(l56c);
+        L(labels[9]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         sub(A1, -16);
@@ -505,12 +473,12 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         movdqu(xword[B - 0x50], xmm3);
         sub(B, -64);
         dec(I);
-        jg(l56c, T_NEAR);
+        jg(labels[9], T_NEAR);
         align(4);
 
-        L(l664);
+        L(labels[10]);
         test(M, 0x8);
-        jle(l6f8, T_NEAR);
+        jle(labels[11], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         movq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         sub(A1, -8);
@@ -543,9 +511,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l6f8);
+        L(labels[11]);
         test(M, 0x4);
-        jle(l75c, T_NEAR);
+        jle(labels[12], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         sub(A1, -4);
@@ -567,9 +535,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l75c);
+        L(labels[12]);
         test(M, 0x2);
-        jle(l7b4, T_NEAR);
+        jle(labels[13], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -589,9 +557,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l7b4);
+        L(labels[13]);
         test(M, 0x1);
-        jle(l7fc, T_NEAR);
+        jle(labels[14], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -607,21 +575,21 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(l7fc);
+        L(labels[14]);
         mov(A1, qword[ARG_BIAS]);
         movdqu(xword[A1], xmm7);
         add(qword[ARG_BIAS], 0x10);
         sub(N, 0x4);
         cmp(N, 0x4);
-        jge(l54c, T_NEAR);
+        jge(labels[8], T_NEAR);
         align(4);
 
-        L(l81c);
+        L(labels[15]);
         cmp(N, 0x2);
-        jl(la0a, T_NEAR);
+        jl(labels[23], T_NEAR);
         align(4);
 
-        L(l828);
+        L(labels[16]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 1]);
         lea(I, ptr[A1 + LDA * 2]);
@@ -629,10 +597,10 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         pxor(xmm7, xmm7);
         mov(I, M);
         sar(I, 0x4);
-        jle(l8d8, T_NEAR);
+        jle(labels[18], T_NEAR);
         align(4);
 
-        L(l848);
+        L(labels[17]);
         movdqu(xmm0, xword[A1 - 0x80]);
         sub(A1, -16);
         movdqu(xmm1, xword[A2 - 0x80]);
@@ -664,12 +632,12 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         movdqu(xword[B - 0x70], xmm2);
         sub(B, -32);
         dec(I);
-        jg(l848, T_NEAR);
+        jg(labels[17], T_NEAR);
         align(4);
 
-        L(l8d8);
+        L(labels[18]);
         test(M, 0x8);
-        jle(l930, T_NEAR);
+        jle(labels[19], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         sub(A1, -8);
         movq(xmm1, qword[A2 - 0x80]);
@@ -689,9 +657,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l930);
+        L(labels[19]);
         test(M, 0x4);
-        jle(l974, T_NEAR);
+        jle(labels[20], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         sub(A1, -4);
         movd(xmm1, dword[A2 - 0x80]);
@@ -707,9 +675,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l974);
+        L(labels[20]);
         test(M, 0x2);
-        jle(l9b8, T_NEAR);
+        jle(labels[21], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         sub(A1, -2);
         pinsrw(xmm0, eax, 0x0);
@@ -725,9 +693,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(l9b8);
+        L(labels[21]);
         test(M, 0x1);
-        jle(l9ec, T_NEAR);
+        jle(labels[22], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         maybe_perform_s8_shift_r8(al);
         pinsrb(xmm0, eax, 0x0);
@@ -741,30 +709,30 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         paddd(xmm7, xmm5);
         align(4);
 
-        L(l9ec);
+        L(labels[22]);
         mov(A1, qword[ARG_BIAS]);
         movq(qword[A1], xmm7);
         add(qword[ARG_BIAS], 0x8);
         sub(N, 0x2);
         cmp(N, 0x2);
-        jge(l828, T_NEAR);
+        jge(labels[16], T_NEAR);
         align(4);
 
-        L(la0a);
+        L(labels[23]);
         cmp(N, 0x1);
-        jl(lb58, T_NEAR);
+        jl(labels[31], T_NEAR);
         align(4);
 
-        L(la14);
+        L(labels[24]);
         mov(A1, A);
         add(A, LDA);
         pxor(xmm7, xmm7);
         mov(I, M);
         sar(I, 0x4);
-        jle(la6c, T_NEAR);
+        jle(labels[26], T_NEAR);
         align(4);
 
-        L(la28);
+        L(labels[25]);
         movdqu(xmm0, xword[A1 - 0x80]);
         sub(A1, -16);
         maybe_perform_s8_shift_xmm(xmm0);
@@ -780,12 +748,12 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         movdqu(xword[B - 0x80], xmm0);
         sub(B, -16);
         dec(I);
-        jg(la28, T_NEAR);
+        jg(labels[25], T_NEAR);
         align(4);
 
-        L(la6c);
+        L(labels[26]);
         test(M, 0x8);
-        jle(laa8, T_NEAR);
+        jle(labels[27], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         sub(A1, -8);
         maybe_perform_s8_shift_xmm(xmm0);
@@ -799,9 +767,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(laa8);
+        L(labels[27]);
         test(M, 0x4);
-        jle(lae0, T_NEAR);
+        jle(labels[28], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         sub(A1, -4);
         maybe_perform_s8_shift_xmm(xmm0);
@@ -814,9 +782,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(lae0);
+        L(labels[28]);
         test(M, 0x2);
-        jle(lb14, T_NEAR);
+        jle(labels[29], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         maybe_perform_s8_shift_r16(ax);
         pinsrw(xmm0, eax, 0x0);
@@ -829,9 +797,9 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -2);
         align(4);
 
-        L(lb14);
+        L(labels[29]);
         test(M, 0x1);
-        jle(lb38, T_NEAR);
+        jle(labels[30], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         maybe_perform_s8_shift_r8(al);
         pinsrb(xmm0, eax, 0x0);
@@ -841,16 +809,16 @@ void jit_avx512_core_u8_copy_sum_bn_kern::generate() {
         sub(B, -1);
         align(4);
 
-        L(lb38);
+        L(labels[30]);
         mov(A1, qword[ARG_BIAS]);
         movd(dword[A1], xmm7);
         add(qword[ARG_BIAS], 0x4);
         sub(N, 0x1);
         cmp(N, 0x1);
-        jge(la14, T_NEAR);
+        jge(labels[24], T_NEAR);
         align(4);
 
-        L(lb58);
+        L(labels[31]);
 
         postamble();
     }
