@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -69,7 +69,6 @@ private:
     reg64_t load_loop_iter = r13;
     reg64_t bcast_loop_iter = r14;
     reg64_t reduce_loop_iter = r15;
-    reg64_t imm_addr64 = reduce_loop_iter;
     reg64_t reg_reduce_pos_flag = r8;
     reg64_t reg_output_stride = r12;
     reg64_t reg_bias_data = r12;
@@ -77,12 +76,14 @@ private:
     reg64_t reg_tmp_output_stride = reg_bcast_data;
     reg64_t reg_tmp = aux_reg_bcast_data;
     reg64_t reg_output_stride_scale = load_loop_iter;
+    reg64_t reg_long_offt = reg_bcast_data;
 
     constexpr static int reg64_size_ = sizeof(int64_t);
     constexpr static int reg_diff_bias_data_stack_offt = 0;
-    constexpr static int reg_binary_post_op_acc_off = 1 * reg64_size_;
-    constexpr static int reg_abi_param1_backup = 2 * reg64_size_;
-    constexpr static int stack_space_needed = 3 * reg64_size_;
+    constexpr static int reg_abi_param1_backup = 1 * reg64_size_;
+    constexpr static int reg_bcast_data_off = 2 * reg64_size_;
+    constexpr static int reg_dw_binary_output_off = 3 * reg64_size_;
+    constexpr static int stack_space_needed = 4 * reg64_size_;
 
     ymm_t vreg_bcast = ymm_t(15);
     ymm_t vtmp = ymm_t(14);
