@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2021 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -88,8 +88,9 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
         const void *post_ops_binary_rhs_arg_vec,
         const void *post_ops_binary_rhs_arg_vec_dw) const {
     const memory_desc_wrapper src_d(pd()->src_md());
-    const memory_desc_wrapper dst_d(pd()->dst_md());
+    const memory_desc_wrapper dst_d(pd()->dst_1x1_md());
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
+    const memory_desc_wrapper dw_dst_d(pd()->dst_md());
     const memory_desc_wrapper dw_weights_d(
             pd()->arg_md(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS));
     const memory_desc_wrapper dw_bias_d(
@@ -112,7 +113,7 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
 
     auto p = jit_1x1_conv_call_s();
 
-    auto rp = rtus_driver_t<avx512_common>::call_params_t();
+    auto rp = rtus_driver_t<avx512_core>::call_params_t();
 
     const int nb_oc = jcp.nb_load;
     const int nb_ic = jcp.nb_reduce;
@@ -128,13 +129,16 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
     const int nb_load_blocking_max = jcp.with_dw_conv
             ? jcp.nb_load_blocking
             : jcp.nb_load_blocking_max;
+    const bool is_dst_layout_nxc = utils::one_of(
+            jcp.dst_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
+    const bool is_src_layout_nxc = utils::one_of(
+            jcp.src_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
 
     // Begin: declare Variables needed for dw conv.
     memory_tracking::grantor_t dw_scratchpad(
             scratchpad, memory_tracking::names::prefix_fusion);
     dst_data_t *pbuf;
     size_t row_offset;
-    const int jcp_dw_kh = 3;
     const int nb_buffer = jcp.nb_load_blocking;
     std::vector<dst_data_t *> addrs;
     // End
@@ -184,23 +188,21 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
 
     auto ker_1x1 = [&](int ocb, int ocb_start, int icb, int n, int g, int od,
                            int oh, int ow, int id, int ih, int iw) {
-        const bool is_dst_layout_nxc = utils::one_of(jcp.dst_tag,
-                format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
         const int oc_off_idx = is_dst_layout_nxc
                 ? g * jcp.oc + ocb * jcp.oc_block
                 : g * nb_oc + ocb;
         const size_t dst_off = data_blk_off(dst_d, n, oc_off_idx, od, oh, ow);
 
-        p.output_data = jcp.with_dw_conv ? pbuf + (oh % jcp_dw_kh) * row_offset
-                                         : &dst[dst_off];
-        p.bias_data
-                = &bias[oc_off_idx * (is_dst_layout_nxc ? 1 : jcp.oc_block)];
+        p.output_data = jcp.with_dw_conv
+                ? pbuf + (oh % pd()->dw_conv_pd_->jcp_.kh) * row_offset
+                : &dst[dst_off];
+        p.bias_data = bias
+                ? &bias[oc_off_idx * (is_dst_layout_nxc ? 1 : jcp.oc_block)]
+                : nullptr;
 
         p.load_data
                 = &weights[pd()->with_groups() ? weights_d.blk_off(g, ocb, icb)
                                                : weights_d.blk_off(ocb, icb)];
-        const bool is_src_layout_nxc = utils::one_of(jcp.src_tag,
-                format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
         const int ic_off_idx = is_src_layout_nxc
                 ? g * jcp.ic + icb * jcp.ic_block
                 : g * nb_ic + icb;
@@ -216,10 +218,8 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
         } else
             p.bcast_data = src + data_blk_off(src_d, n, ic_off_idx, id, ih, iw);
 
-        p.dst_l_off = dst_off;
-        p.oc_l_off = oc_off_idx * (is_dst_layout_nxc ? 1 : jcp.oc_block);
         p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
-        p.dst_orig = dst;
+        p.dst_orig = static_cast<const float *>(p.output_data) - dst_off;
 
         (*kernel_)(&p);
     };
@@ -321,8 +321,8 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
             addrs[i] = pbuf + ((oh_1x1++) % jcp_dw.kh) * row_offset;
 
         const auto ocb_end = ocb_start + load_step;
-        const auto wch_stride
-                = jcp_dw.iw * jcp_dw.nb_ch_blocking * jcp_dw.ch_block;
+        const auto wch_stride = (is_src_layout_nxc ? 1 : jcp_dw.iw)
+                * jcp_dw.nb_ch_blocking * jcp_dw.ch_block;
         const int dil_h = jcp_dw.dilate_h + 1;
         const int str_h = jcp_dw.stride_h;
         const int ch_num = jcp_dw.nb_ch_blocking;
@@ -346,7 +346,12 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
             jit_conv_call_s par_conv_dw;
 
             par_conv_dw.src = addrs.data();
-            par_conv_dw.dst = &dst[dst_d.blk_off(n, ch, dw_oh, ow)];
+
+            const size_t ch_step = is_dst_layout_nxc
+                    ? jcp_dw.ch_block
+                    : dw_dst_d.blk_off(0, 1, 0, 0);
+            par_conv_dw.dst
+                    = &dst[dw_dst_d.blk_off(n, 0, dw_oh, ow) + ch * ch_step];
 
             par_conv_dw.filt
                     = &weights_dw[dw_weights_d.blk_off(ch, 0, 0, kh, kw)];
@@ -359,7 +364,6 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
             par_conv_dw.load_work = (nstl::min(ch + ch_num, jcp_dw.nb_ch) - ch)
                     * jcp_dw.ch_block;
 
-            par_conv_dw.oc_l_off = ch * jcp_dw.ch_block;
             par_conv_dw.post_ops_binary_rhs_arg_vec
                     = post_ops_binary_rhs_arg_vec_dw;
             par_conv_dw.dst_orig = dst;
@@ -434,7 +438,8 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
     }
 }
 
-template struct jit_avx512_common_1x1_convolution_fwd_t<data_type::f32>;
+REG_AVX512_ISA(template struct jit_avx512_common_1x1_convolution_fwd_t<
+        data_type::f32>);
 /* convolution backward wtr data */
 
 template <data_type_t diff_dst_type, data_type_t wei_type,
@@ -477,7 +482,7 @@ void jit_avx512_common_1x1_convolution_bwd_data_t<diff_dst_type, wei_type,
 
     parallel(jcp.nthr, [&](const int ithr, const int nthr) {
         auto p = jit_1x1_conv_call_s();
-        auto rp = rtus_driver_t<avx512_common>::call_params_t();
+        auto rp = rtus_driver_t<avx512_core>::call_params_t();
 
         int bcast_start {0}, bcast_end {0}, icb_start {0}, icb_end {0};
         balance2D(nthr, ithr, work_amount, bcast_start, bcast_end, jcp.nb_load,
@@ -585,7 +590,8 @@ void jit_avx512_common_1x1_convolution_bwd_data_t<diff_dst_type, wei_type,
     });
 }
 
-template struct jit_avx512_common_1x1_convolution_bwd_data_t<data_type::f32>;
+REG_AVX512_ISA(template struct jit_avx512_common_1x1_convolution_bwd_data_t<
+        data_type::f32>);
 
 /* convolution backward wtr weights */
 
@@ -606,20 +612,7 @@ status_t jit_avx512_common_1x1_convolution_bwd_weights_t ::init(
     CHECK(acc_ker_->create_kernel());
     CHECK(reducer_bias_->create_kernel());
 
-    const auto &jcp = kernel_->jcp;
-
-    if (jcp.transpose_src) {
-        auto tp = jit_transpose4x16_src_t();
-        tp.src_pf0_distance = 4;
-        tp.tr_src_pf0_distance = 0;
-        tp.src_pf1 = true;
-        tp.tr_src_pf1 = false;
-        CHECK(safe_ptr_assign(
-                trans_kernel_, new jit_transpose4x16_src(&jcp, &tp)));
-        CHECK(trans_kernel_->create_kernel());
-    }
-
-    CHECK(init_rtus_driver<avx512_common>(this));
+    CHECK(init_rtus_driver<avx512_core>(this));
     return status::success;
 }
 
@@ -649,21 +642,15 @@ void jit_avx512_common_1x1_convolution_bwd_weights_t::execute_backward_weights(
             : diff_bias_in;
     auto wei_reduction = scratchpad.get<data_t>(key_conv_wei_reduction);
 
-    /* prepare src transposition barriers */
-    auto tr_src = scratchpad.get<data_t>(key_conv_tr_src);
-    auto tr_src_bctx
-            = scratchpad.get<simple_barrier::ctx_t>(key_conv_tr_src_bctx);
-    if (jcp.transpose_src) {
-        for (int i = 0; i < jcp.nthr; ++i)
-            simple_barrier::ctx_init(&tr_src_bctx[i]);
-    }
-
     const int ndims = src_d.ndims();
     const int wei_size = jcp.ngroups * rnd_up(jcp.oc, jcp.oc_block)
             * rnd_up(jcp.ic, jcp.ic_block);
 
-    simple_barrier::ctx_t reduction_barrier;
-    simple_barrier::ctx_init(&reduction_barrier);
+    simple_barrier::ctx_t *reduction_barrier
+            = scratchpad.template get<simple_barrier::ctx_t>(
+                    key_conv_wei_reduction_bctx);
+    if (dnnl_thr_syncable() && jcp.nthr_mb > 1)
+        simple_barrier::ctx_init(reduction_barrier);
 
     const auto reducer_bia_scratchpad
             = memory_tracking::grantor_t(scratchpad, prefix_reducer_bia);
@@ -690,94 +677,45 @@ void jit_avx512_common_1x1_convolution_bwd_weights_t::execute_backward_weights(
         return remaining < tail_step ? remaining : default_step;
     };
 
-    // TODO: use memory descriptor with the same fmt as src
-    // (or use a macro :))
-    auto tr_src_off = [&](int img, int icb, int is) {
-        const size_t tr_chn_size = jcp.tr_is * jcp.ic_block;
-        const size_t tr_img_size = tr_chn_size * nb_ic * jcp.ngroups;
-        return img * tr_img_size + icb * tr_chn_size + is * jcp.ic_block;
-    };
-
     const bool is_src_layout_nxc = utils::one_of(
             jcp.src_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
-    auto uker_trans = [&](int ithr_mb, int img, int sp_b_start, int sp_size,
-                              int g_start, int g_work, int ic_b_start,
-                              int ic_b_work, int ithr, int nthr,
-                              int first_ic_b) {
-        assert(!is_src_layout_nxc);
-        const int work_amount = g_work * ic_b_work;
+    const bool is_ddst_layout_nxc = utils::one_of(
+            jcp.dst_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
 
-        int start {0}, end {0};
-        balance211(work_amount, nthr, ithr, start, end);
-
-        int g {0}, ic_b {0};
-        nd_iterator_init(start, g, g_work, ic_b, ic_b_work);
-        g += g_start;
-        const int ic_b_tr = g * nb_ic + first_ic_b + ic_b;
-        ic_b += ic_b_start;
-
-        const int _ic = g * nb_ic + ic_b;
-
-        const int is = sp_b_start * jcp.reduce_block;
-        const int id = is / (jcp.ih * jcp.iw);
-        const int is_2d = is % (jcp.ih * jcp.iw);
-        const int ih = is_2d / jcp.iw;
-        const int iw = is_2d % jcp.iw;
-
-        const int src1_off = data_blk_off(src_d, img, _ic, id, ih, iw);
-        data_t *src1 = (data_t *)&src[src1_off];
-        data_t *tr_src1 = &tr_src[tr_src_off(ithr_mb, ic_b_tr, is)];
-
-        assert(jcp.ic_block == 16);
-        const int src_stride = jcp.is * jcp.ic_block;
-        const int tr_src_stride = jcp.tr_is * jcp.ic_block;
-
-        const int my_work = end - start;
-        for (int iwork = 0; iwork < my_work; iwork++) {
-            auto par_trans = jit_src_transpose_s();
-            assert(sp_size % 4 == 0 || sp_size % 4 == jcp.is % 4);
-            par_trans.size = sp_size;
-            par_trans.src = src1;
-            par_trans.tr_src = tr_src1;
-            par_trans.src_prf = src1 + 64 * 16;
-            par_trans.tr_src_prf = tr_src1 + 80 * 16;
-            (*trans_kernel_)(&par_trans);
-
-            src1 += src_stride;
-            tr_src1 += tr_src_stride;
+    auto maybe_zero_icpad = [&](const int g_start, const int g_end,
+                                    const int ocb_start, const int ocb_end) {
+        // write zeros to IC padded region.
+        const int ic_tail = jcp.ic_without_padding % jcp.ic_block;
+        if (is_ddst_layout_nxc && ic_tail != 0) {
+            for_(int g = g_start; g < g_end; ++g)
+            for (int z_ocb = ocb_start; z_ocb < ocb_end; ++z_ocb) {
+                const int z_icb = nb_ic - 1;
+                const size_t off = wht_blk_off(diff_weights_d, g, z_ocb, z_icb)
+                        + ic_tail * jcp.oc_block;
+                data_t *z_wei = diff_weights + off;
+                const int zero_work
+                        = (nb_ic * jcp.ic_block - jcp.ic_without_padding)
+                        * jcp.oc_block;
+                PRAGMA_OMP_SIMD()
+                for (int o = 0; o < zero_work; ++o) {
+                    z_wei[o] = 0;
+                }
+            }
         }
     };
 
-    const bool is_ddst_layout_nxc = utils::one_of(
-            jcp.dst_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
     auto ker = [&](const int ithr, const int nthr) {
         assert(nthr == jcp.nthr);
-        const bool ready_for_async
-                = utils::one_of(jcp.ver, ver_fma, ver_avx512_core);
-        MAYBE_UNUSED(ready_for_async);
-        assert(IMPLICATION(
-                !ready_for_async && !dnnl_thr_syncable(), jcp.nthr_mb == 1));
 
         const int ithr_ic_b = ithr % jcp.nthr_ic_b;
         const int ithr_oc_b = ithr / jcp.nthr_ic_b % jcp.nthr_oc_b;
         const int ithr_g = ithr / jcp.nthr_ic_b / jcp.nthr_oc_b % jcp.nthr_g;
         const int ithr_mb = ithr / jcp.nthr_ic_b / jcp.nthr_oc_b / jcp.nthr_g;
 
-        const int ithr_but_oc
-                = (ithr_mb * jcp.nthr_g + ithr_g) * jcp.nthr_ic_b + ithr_ic_b;
-
         /* reduction dimension */
         int mb_sp_b_start {0}, mb_sp_b_end {0};
-        if (jcp.transpose_src && jcp.nthr_mb < jcp.mb / 2) {
-            // it's preferable to parallelize by mb if possible
-            int img_start {0}, img_end {0};
-            balance211(jcp.mb, jcp.nthr_mb, ithr_mb, img_start, img_end);
-            mb_sp_b_start = img_start * sp_nb;
-            mb_sp_b_end = img_end * sp_nb;
-        } else {
-            balance211(mb_sp_work, jcp.nthr_mb, ithr_mb, mb_sp_b_start,
-                    mb_sp_b_end);
-        }
+        balance211(
+                mb_sp_work, jcp.nthr_mb, ithr_mb, mb_sp_b_start, mb_sp_b_end);
 
         /* independent dimensions */
         int g_start {0}, oc_b_start {0}, ic_b_start {0};
@@ -826,27 +764,12 @@ void jit_avx512_common_1x1_convolution_bwd_weights_t::execute_backward_weights(
                         bcast_step = step(nb_ic_blocking, ic_b_end - ic_b,
                                 jcp.nb_bcast_blocking_max);
                     }
-                    if (jcp.transpose_src) {
-                        if (jcp.nthr_oc_b > 1)
-                            simple_barrier::barrier(
-                                    &tr_src_bctx[ithr_but_oc], jcp.nthr_oc_b);
-                        const int sp_size
-                                = nstl::min(sp_b_step * jcp.reduce_block,
-                                        jcp.is - sp_b * jcp.reduce_block);
-                        uker_trans(ithr_mb, img, sp_b, sp_size, g, 1, ic_b,
-                                bcast_step, ithr_oc_b, jcp.nthr_oc_b,
-                                ic_b_start);
-                        if (jcp.nthr_oc_b > 1)
-                            simple_barrier::barrier(
-                                    &tr_src_bctx[ithr_but_oc], jcp.nthr_oc_b);
-                    }
 
                     for (int oc_b = oc_b_start; oc_b < oc_b_end;
                             oc_b += load_step) {
                         load_step = step(nb_oc_blocking, oc_b_end - oc_b,
                                 jcp.nb_load_blocking_max);
                         const int _ic_b = g * nb_ic + ic_b;
-                        const int _ic_b_tr = g * nb_ic + ic_b_start;
                         const int oc_off_idx = is_ddst_layout_nxc
                                 ? g * jcp.oc + oc_b * jcp.oc_block
                                 : g * nb_oc + oc_b;
@@ -860,9 +783,8 @@ void jit_avx512_common_1x1_convolution_bwd_weights_t::execute_backward_weights(
                         const int ic_off_idx
                                 = (is_src_layout_nxc ? jcp.ic_block : 1)
                                 * _ic_b;
-                        const data_t *diff_src = jcp.transpose_src
-                                ? &tr_src[tr_src_off(ithr_mb, _ic_b_tr, 0)]
-                                : &src[src_d.blk_off(img, ic_off_idx)];
+                        const data_t *diff_src
+                                = &src[src_d.blk_off(img, ic_off_idx)];
 
                         int sp_b_end = sp_b + sp_b_step;
                         const data_t *pdiff_dst = &diff_dst[diff_dst_d.blk_off(
@@ -870,7 +792,7 @@ void jit_avx512_common_1x1_convolution_bwd_weights_t::execute_backward_weights(
                         const data_t *local_src = diff_src;
 
                         auto p = jit_1x1_conv_call_s();
-                        auto rp = rtus_driver_t<avx512_common>::call_params_t();
+                        auto rp = rtus_driver_t<avx512_core>::call_params_t();
 
                         p.output_stride = utils::rnd_up(jcp.ic, jcp.ic_block)
                                 * jcp.oc_block * jcp.typesize_out;
@@ -930,9 +852,13 @@ void jit_avx512_common_1x1_convolution_bwd_weights_t::execute_backward_weights(
             }
         }
 
+        if (ithr_mb == 0 && ic_b_end >= jcp.nb_bcast) {
+            maybe_zero_icpad(g_start, g_end, oc_b_start, oc_b_end);
+        }
+
         /* diff_weights[:] += sum(wei_reduction[thr_mb][:]) */
         if (dnnl_thr_syncable() && jcp.nthr_mb > 1) {
-            simple_barrier::barrier(&reduction_barrier, jcp.nthr);
+            simple_barrier::barrier(reduction_barrier, jcp.nthr);
             const int work = g_work * oc_b_work * ic_b_work;
             int start {0}, end {0};
             balance211(work, jcp.nthr_mb, ithr_mb, start, end);
