@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -130,8 +130,9 @@ void jit_avx512_core_bf16_1x1_convolution_fwd_t<dst_type>::execute_forward_thr(
         const void *post_ops_binary_rhs_arg_vec,
         const void *post_ops_binary_rhs_arg_vec_dw) const {
     const memory_desc_wrapper src_d(pd()->src_md());
-    const memory_desc_wrapper dst_d(pd()->dst_md());
+    const memory_desc_wrapper dst_d(pd()->dst_1x1_md());
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
+    const memory_desc_wrapper dw_dst_d(pd()->dst_md());
     const memory_desc_wrapper dw_weights_d(
             pd()->arg_md(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS));
     const memory_desc_wrapper dw_bias_d(
@@ -150,7 +151,7 @@ void jit_avx512_core_bf16_1x1_convolution_fwd_t<dst_type>::execute_forward_thr(
 
     auto p = jit_1x1_conv_call_s();
 
-    auto rp = rtus_driver_t<avx512_common>::call_params_t();
+    auto rp = rtus_driver_t<avx512_core>::call_params_t();
 
     const int nb_oc = jcp.nb_load;
     const int nb_ic = jcp.nb_reduce;
@@ -173,6 +174,12 @@ void jit_avx512_core_bf16_1x1_convolution_fwd_t<dst_type>::execute_forward_thr(
     const auto jcp_dw = pd()->jcp_dw_;
     const int nb_buffer = jcp.nb_load_blocking;
     std::vector<decltype(pbuf)> addrs;
+    const bool is_dst_layout_nxc = utils::one_of(
+            jcp.dst_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
+    const bool is_src_layout_nxc = utils::one_of(
+            jcp.src_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
+
+    auto start_off = dst_d.off_l(0);
 
     auto step = [](int default_step, int remaining, int tail_step) {
         assert(default_step <= tail_step);
@@ -224,8 +231,6 @@ void jit_avx512_core_bf16_1x1_convolution_fwd_t<dst_type>::execute_forward_thr(
 
     auto ker_1x1 = [&](int ocb, int ocb_start, int icb, int n, int g, int od,
                            int oh, int ow, int id, int ih, int iw) {
-        const bool is_dst_layout_nxc = utils::one_of(jcp.dst_tag,
-                format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
         const int oc_off_idx = is_dst_layout_nxc
                 ? g * jcp.oc + ocb * jcp.oc_block
                 : g * nb_oc + ocb;
@@ -242,8 +247,6 @@ void jit_avx512_core_bf16_1x1_convolution_fwd_t<dst_type>::execute_forward_thr(
                 = &weights[pd()->with_groups() ? weights_d.blk_off(g, ocb, icb)
                                                : weights_d.blk_off(ocb, icb)];
 
-        const bool is_src_layout_nxc = utils::one_of(jcp.src_tag,
-                format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
         const int ic_off_idx = is_src_layout_nxc
                 ? g * jcp.ic + icb * jcp.ic_block
                 : g * nb_ic + icb;
@@ -266,12 +269,11 @@ void jit_avx512_core_bf16_1x1_convolution_fwd_t<dst_type>::execute_forward_thr(
                 : rnd_up((jcp.load_dim / grp_count), jcp.load_block);
         const size_t str_size = jcp.bcast_dim * max_load_per_thread;
         p.store_buffer = store_buffer + ithr * str_size
-                + data_blk_off(dst_d, 0, 0, od, oh, ow);
+                + data_blk_off(dst_d, 0, 0, od, oh, ow) - start_off;
 
-        p.dst_l_off = dst_off;
-        p.oc_l_off = oc_off_idx * (is_dst_layout_nxc ? 1 : jcp.oc_block);
         p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
-        p.dst_orig = dst;
+        p.dst_orig = static_cast<const char *>(p.output_data)
+                - dst_off * dst_d.data_type_size();
 
         (*kernel_)(&p);
     };
@@ -331,8 +333,8 @@ void jit_avx512_core_bf16_1x1_convolution_fwd_t<dst_type>::execute_forward_thr(
             addrs[i] = pbuf + ((oh_1x1++) % jcp_dw->kh) * row_offset;
 
         const auto ocb_end = ocb_start + load_step;
-        const auto wch_stride
-                = jcp_dw->iw * jcp_dw->nb_ch_blocking * jcp_dw->ch_block;
+        const auto wch_stride = (is_src_layout_nxc ? 1 : jcp_dw->iw)
+                * jcp_dw->nb_ch_blocking * jcp_dw->ch_block;
 
         const int dil_h = jcp_dw->dilate_h + 1;
         const int str_h = jcp_dw->stride_h;
@@ -357,8 +359,13 @@ void jit_avx512_core_bf16_1x1_convolution_fwd_t<dst_type>::execute_forward_thr(
             jit_conv_call_s par_conv_dw;
 
             par_conv_dw.src = addrs.data();
-            par_conv_dw.dst = &dst[dst_d.blk_off(n, ch, dw_oh, ow)
-                    * dst_d.data_type_size()];
+
+            const size_t ch_step = is_dst_layout_nxc
+                    ? jcp_dw->ch_block
+                    : dw_dst_d.blk_off(0, 1, 0, 0);
+            par_conv_dw.dst
+                    = &dst[(dw_dst_d.blk_off(n, 0, dw_oh, ow) + ch * ch_step)
+                            * dw_dst_d.data_type_size()];
 
             par_conv_dw.filt
                     = &weights_dw[dw_weights_d.blk_off(ch, 0, 0, kh, kw)];
@@ -371,7 +378,6 @@ void jit_avx512_core_bf16_1x1_convolution_fwd_t<dst_type>::execute_forward_thr(
             par_conv_dw.load_work = (nstl::min(ch + ch_num, jcp_dw->nb_ch) - ch)
                     * jcp_dw->ch_block;
 
-            par_conv_dw.oc_l_off = ch * jcp_dw->ch_block;
             par_conv_dw.post_ops_binary_rhs_arg_vec
                     = post_ops_binary_rhs_arg_vec_dw;
             par_conv_dw.dst_orig = dst;
@@ -447,8 +453,10 @@ void jit_avx512_core_bf16_1x1_convolution_fwd_t<dst_type>::execute_forward_thr(
     }
 }
 
-template struct jit_avx512_core_bf16_1x1_convolution_fwd_t<data_type::f32>;
-template struct jit_avx512_core_bf16_1x1_convolution_fwd_t<data_type::bf16>;
+REG_AVX512_ISA(template struct jit_avx512_core_bf16_1x1_convolution_fwd_t<
+        data_type::f32>);
+REG_AVX512_ISA(template struct jit_avx512_core_bf16_1x1_convolution_fwd_t<
+        data_type::bf16>);
 
 template <data_type_t diff_src_type>
 void jit_avx512_core_bf16_1x1_convolution_bwd_data_t<
@@ -496,7 +504,7 @@ void jit_avx512_core_bf16_1x1_convolution_bwd_data_t<
 
     auto p = jit_1x1_conv_call_s();
 
-    auto rp = rtus_driver_t<avx512_common>::call_params_t();
+    auto rp = rtus_driver_t<avx512_core>::call_params_t();
     const int nb_ic = jcp.nb_load;
     const int nb_oc = jcp.nb_reduce;
     const int os_block = jcp.bcast_block;
@@ -609,9 +617,10 @@ void jit_avx512_core_bf16_1x1_convolution_bwd_data_t<
     }
 }
 
-template struct jit_avx512_core_bf16_1x1_convolution_bwd_data_t<data_type::f32>;
-template struct jit_avx512_core_bf16_1x1_convolution_bwd_data_t<
-        data_type::bf16>;
+REG_AVX512_ISA(template struct jit_avx512_core_bf16_1x1_convolution_bwd_data_t<
+        data_type::f32>);
+REG_AVX512_ISA(template struct jit_avx512_core_bf16_1x1_convolution_bwd_data_t<
+        data_type::bf16>);
 
 /* convolution backward wtr weights */
 
@@ -656,7 +665,7 @@ jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<diff_weights_type>::init(
         }
     }
 
-    CHECK(init_rtus_driver<avx512_common>(this));
+    CHECK(init_rtus_driver<avx512_core>(this));
     return status::success;
 }
 
@@ -697,8 +706,11 @@ void jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<diff_weights_type>::
             = jcp.dst_dt == data_type::bf16 ? jcp.nthr_mb : jcp.nthr_mb - 1;
     auto bia_reduction = wei_reduction + n_wei_buffers * wei_size;
 
-    simple_barrier::ctx_t reduction_barrier;
-    if (dnnl_thr_syncable()) simple_barrier::ctx_init(&reduction_barrier);
+    simple_barrier::ctx_t *reduction_barrier
+            = scratchpad.template get<simple_barrier::ctx_t>(
+                    key_conv_wei_reduction_bctx);
+    if (dnnl_thr_syncable() && jcp.nthr_mb > 1)
+        simple_barrier::ctx_init(reduction_barrier);
 
     // TODO (Roma): remove this restriction
     assert(jcp.stride_w == 1 && jcp.stride_h == 1);
@@ -721,6 +733,29 @@ void jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<diff_weights_type>::
 
     const bool is_ddst_layout_nxc = utils::one_of(
             jcp.dst_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
+
+    auto maybe_zero_icpad = [&](const int g_start, const int g_end,
+                                    const int ocb_start, const int ocb_end) {
+        // write zeros to IC padded region.
+        const int ic_tail = jcp.ic_without_padding % jcp.ic_block;
+        if (ic_tail != 0) {
+            for_(int g = g_start; g < g_end; ++g)
+            for (int z_ocb = ocb_start; z_ocb < ocb_end; ++z_ocb) {
+                const int z_icb = jcp.nb_bcast - 1;
+                const size_t off = wht_blk_off(diff_weights_d, g, z_ocb, z_icb)
+                        + ic_tail * jcp.oc_block;
+                diff_wei_data_t *z_wei = diff_weights + off;
+                const int zero_work
+                        = (jcp.nb_bcast * jcp.ic_block - jcp.ic_without_padding)
+                        * jcp.oc_block;
+                PRAGMA_OMP_SIMD()
+                for (int o = 0; o < zero_work; ++o) {
+                    z_wei[o] = 0;
+                }
+            }
+        }
+    };
+
     auto ker = [&](const int ithr, const int nthr) {
         assert(nthr == jcp.nthr);
 
@@ -808,7 +843,7 @@ void jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<diff_weights_type>::
                         const src_data_t *local_src = diff_src;
 
                         auto p = jit_1x1_conv_call_s();
-                        auto rp = rtus_driver_t<avx512_common>::call_params_t();
+                        auto rp = rtus_driver_t<avx512_core>::call_params_t();
 
                         p.output_stride = utils::rnd_up(jcp.ic, jcp.oc_block)
                                 * jcp.oc_block * jcp.typesize_out;
@@ -966,7 +1001,7 @@ void jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<diff_weights_type>::
         /* diff_weights[:] += sum(ws_reduction_[thr_mb][:]) */
         if (jcp.nthr_mb > _start_nthr_mb) {
             if (dnnl_thr_syncable())
-                simple_barrier::barrier(&reduction_barrier, jcp.nthr);
+                simple_barrier::barrier(reduction_barrier, jcp.nthr);
             const int work = g_work * oc_b_work * ic_b_work;
             int start {0}, end {0};
             balance211(work, jcp.nthr_mb, ithr_mb, start, end);
@@ -1081,6 +1116,9 @@ void jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<diff_weights_type>::
                 }
             }
         }
+        if (ic_b_end >= jcp.nb_bcast) {
+            maybe_zero_icpad(g_start, g_end, oc_b_start, oc_b_end);
+        }
     };
 
     parallel(jcp.nthr, [&](const int ithr, const int nthr) {
@@ -1104,10 +1142,12 @@ void jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<diff_weights_type>::
     }
 }
 
-template struct jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<
-        data_type::f32>;
-template struct jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<
-        data_type::bf16>;
+REG_AVX512_ISA(
+        template struct jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<
+                data_type::f32>);
+REG_AVX512_ISA(
+        template struct jit_avx512_core_bf16_1x1_convolution_bwd_weights_t<
+                data_type::bf16>);
 
 } // namespace x64
 } // namespace cpu
