@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -19,12 +19,12 @@
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
 
+#include "cpu/cpu_primitive.hpp"
+
 #include "cpu/x64/jit_generator.hpp"
 
 #include "cpu/x64/jit_avx512_core_x8s8s32x_1x1_convolution.hpp"
 
-#include "cpu/cpu_primitive.hpp"
-
 namespace dnnl {
 namespace impl {
 namespace cpu {
@@ -35,15 +35,14 @@ using namespace dnnl::impl::memory_tracking::names;
 using namespace dnnl::impl::utils;
 
 /* convolution forward */
-template <data_type_t src_type, data_type_t dst_type>
-status_t jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
-        dst_type>::execute_forward(const exec_ctx_t &ctx) const {
-    auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
-    auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
-    auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
-    auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+status_t jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward(
+        const exec_ctx_t &ctx) const {
+    const auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
+    const auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
+    const auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
+    auto dst = CTX_OUT_MEM(char *, DNNL_ARG_DST);
     auto weights_dw = CTX_IN_MEM(
-            const wei_data_t *, DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS);
+            const char *, DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS);
     auto bias_dw = CTX_IN_MEM(
             const char *, DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS);
     const auto post_ops_binary_rhs_arg_vec
@@ -53,83 +52,97 @@ status_t jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
                     pd()->jcp_.post_ops.entry_.size() + 1)
             : std::vector<const void *> {};
 
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    DEFINE_ARG_SCALES_BUFFER(
+            dw_wei_scales, DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(
+            dw_dst_scales, DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_DST);
+
     DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
 
     auto scratchpad = ctx.get_scratchpad_grantor();
 
-    if (pd()->jcp_.signed_input && pd()->jcp_.ver != ver_vnni) {
-        auto local_scales
-                = scratchpad.template get<float>(key_conv_adjusted_scales);
-        auto scales = pd()->attr()->output_scales_.scales_;
-        size_t count = pd()->attr()->output_scales_.count_;
-        float factor = 1.f / pd()->jcp_.wei_adj_scale;
-        if (count == 1) {
-            utils::array_set(
-                    local_scales, scales[0] * factor, pd()->jcp_.ic_block);
-        } else {
-            for (size_t c = 0; c < count; c++)
-                local_scales[c] = scales[c] * factor;
-        }
+    auto local_scales
+            = scratchpad.template get<float>(key_conv_adjusted_scales);
+    // Src scale is always a single value
+    float src_scale = src_scales[0];
+    int wei_mask = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    float factor = (pd()->jcp_.signed_input && (!pd()->jcp_.has_vnni))
+            ? 1.f / pd()->jcp_.wei_adj_scale
+            : 1.f;
+    switch (wei_mask) {
+        case 0:
+            utils::array_set(local_scales, src_scale * wei_scales[0] * factor,
+                    pd()->jcp_.ic_block);
+            break;
+        default:
+            for (dim_t c = 0; c < pd()->OC(); c++)
+                local_scales[c] = src_scale * wei_scales[c] * factor;
     }
 
+    const float *dw_oscales = nullptr;
     if (pd()->jcp_.with_dw_conv) {
         auto jcp_dw = pd()->jcp_dw_;
-        if (jcp_dw->signed_input && jcp_dw->ver != ver_vnni) {
-            memory_tracking::grantor_t dw_scratchpad(
-                    scratchpad, memory_tracking::names::prefix_fusion);
-            auto attr_dw = pd()->dw_conv_pd_->attr();
-
-            auto local_scales = dw_scratchpad.template get<float>(
-                    key_conv_adjusted_scales);
-            auto scales = attr_dw->output_scales_.scales_;
-            size_t count = attr_dw->output_scales_.count_;
-            float factor = 1.f / jcp_dw->wei_adj_scale;
-            if (count == 1) {
-                utils::array_set(
-                        local_scales, scales[0] * factor, pd()->jcp_.ic_block);
-            } else {
-                for (size_t c = 0; c < count; c++)
-                    local_scales[c] = scales[c] * factor;
-            }
+        memory_tracking::grantor_t dw_scratchpad(
+                scratchpad, memory_tracking::names::prefix_fusion);
+        auto dw_local_scales
+                = dw_scratchpad.template get<float>(key_conv_adjusted_scales);
+        auto attr_dw = pd()->dw_conv_pd_->attr();
+        int wei_mask = attr_dw->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+        dim_t count = wei_mask == 0 ? 1 : pd()->dw_conv_pd_->OC();
+        float factor = 1.f / jcp_dw->wei_adj_scale;
+        if (count == 1) {
+            utils::array_set(dw_local_scales,
+                    dw_wei_scales[0] / dst_scales[0] * factor,
+                    pd()->jcp_.ic_block);
+        } else {
+            for (dim_t c = 0; c < count; c++)
+                dw_local_scales[c] = dw_wei_scales[c] / dst_scales[0] * factor;
         }
+        dw_oscales = dw_local_scales;
     }
     parallel(pd()->jcp_.nthr, [&](const int ithr, const int nthr) {
         execute_forward_thr(ithr, nthr, src, weights, bias, weights_dw, bias_dw,
-                dst, src_zero_point, dst_zero_point, scratchpad,
+                dst, local_scales, dst_scales, dw_oscales, dw_dst_scales,
+                src_zero_point, dst_zero_point, scratchpad,
                 post_ops_binary_rhs_arg_vec.data(),
                 post_ops_binary_rhs_arg_vec_dw.data());
     });
     return status::success;
 }
 
-template <data_type_t src_type, data_type_t dst_type>
-void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
-        dst_type>::execute_forward_thr(const int ithr, const int nthr,
-        const src_data_t *src, const wei_data_t *weights, const char *bias,
-        const wei_data_t *weights_dw, const char *bias_dw, dst_data_t *dst,
+void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward_thr(
+        const int ithr, const int nthr, const char *src, const char *weights,
+        const char *bias, const char *weights_dw, const char *bias_dw,
+        char *dst, const float *oscales, const float *dst_scales,
+        const float *dw_oscales, const float *dw_dst_scales,
         const int32_t *src_zero_point, const int32_t *dst_zero_point,
         const memory_tracking::grantor_t &scratchpad,
         const void *post_ops_binary_rhs_arg_vec,
         const void *post_ops_binary_rhs_arg_vec_dw) const {
     const memory_desc_wrapper src_d(pd()->src_md());
-    const memory_desc_wrapper dst_d(pd()->dst_md());
+    const memory_desc_wrapper dst_d(pd()->dst_1x1_md());
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
+    const memory_desc_wrapper dw_dst_d(pd()->dst_md());
     const memory_desc_wrapper dw_weights_d(
             pd()->arg_md(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS));
 
     const auto &jcp = pd()->jcp_;
 
+    const size_t src_dt_size = types::data_type_size(src_d.data_type());
+    const size_t dst_dt_size = types::data_type_size(dst_d.data_type());
     const size_t bia_dt_size = pd()->with_bias()
             ? types::data_type_size(pd()->desc()->bias_desc.data_type)
             : 0;
 
     auto rtus_space = pd()->rtus_.reduce_src_
-            ? scratchpad.get<src_data_t>(key_conv_rtus_space)
+            ? scratchpad.get<char>(key_conv_rtus_space)
             : nullptr;
 
-    auto local_scales = scratchpad.get<float>(key_conv_adjusted_scales);
-
     const int work_amount = jcp.mb * jcp.ngroups * jcp.nb_bcast;
 
     const bool is_2d = pd()->ndims() == 4;
@@ -139,25 +152,19 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
     const int stride_h = pd()->KSH();
     const int stride_w = pd()->KSW();
 
-    float *oscales {nullptr};
-    if (jcp.signed_input && jcp.ver != ver_vnni)
-        oscales = scratchpad.get<float>(key_conv_adjusted_scales);
-    else
-        oscales = pd()->attr()->output_scales_.scales_;
-
     auto offset = weights_d.size() - weights_d.additional_buffer_size();
-    wei_data_t *w = const_cast<wei_data_t *>(weights);
+    char *w = const_cast<char *>(weights);
     const int32_t *compensation = (jcp.signed_input)
             ? reinterpret_cast<int32_t *>(w + offset)
             : nullptr;
     const int32_t *zp_compensation = jcp.src_zero_point
-            ? reinterpret_cast<int32_t *>(&w[offset])
+            ? reinterpret_cast<int32_t *>(w + offset)
                     + (jcp.signed_input ? jcp.ngroups * jcp.oc : 0)
             : nullptr;
 
     auto p = jit_1x1_conv_call_s();
 
-    auto rp = rtus_driver_t<avx512_common>::call_params_t();
+    auto rp = rtus_driver_t<avx512_core>::call_params_t();
     const int nb_oc = jcp.nb_load;
     const int nb_ic = jcp.nb_reduce;
     // override some constants for fused dw_conv
@@ -177,28 +184,25 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
     memory_tracking::grantor_t dw_scratchpad(
             scratchpad, memory_tracking::names::prefix_fusion);
 
-    const size_t dw_bia_dt_size = jcp_dw && jcp_dw->with_bias
-            ? types::data_type_size(dw_pd->desc()->bias_desc.data_type)
-            : 0;
-
-    float *dw_oscales {nullptr};
+    size_t dw_bia_dt_size = 0;
     int32_t *compensation_dw {nullptr};
-    if (jcp.with_dw_conv) {
+    if (jcp.with_dw_conv && jcp_dw) {
+        if (jcp_dw->with_bias)
+            dw_bia_dt_size
+                    = types::data_type_size(dw_pd->desc()->bias_desc.data_type);
+
         offset = dw_weights_d.size() - dw_weights_d.additional_buffer_size();
-        w = const_cast<wei_data_t *>(weights_dw);
+        w = const_cast<char *>(weights_dw);
         compensation_dw = (jcp_dw->signed_input)
                 ? reinterpret_cast<int32_t *>(w + offset)
                 : nullptr;
-        if (jcp_dw->signed_input && jcp_dw->ver != ver_vnni)
-            dw_oscales = dw_scratchpad.get<float>(key_conv_adjusted_scales);
-        else
-            dw_oscales = dw_pd->attr()->output_scales_.scales_;
+        dw_oscales = dw_scratchpad.get<float>(key_conv_adjusted_scales);
     }
 
-    dst_data_t *pbuf {nullptr};
+    char *pbuf {nullptr};
     size_t row_offset {};
     const int nb_buffer = jcp.nb_load_blocking;
-    std::vector<dst_data_t *> addrs;
+    std::vector<char *> addrs;
     // End
 
     auto step = [](int default_step, int remaining, int tail_step) {
@@ -259,10 +263,11 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
                         : dst_d.blk_off(n, _ocb * jcp.oc_block, ow);
 
         p.output_data = jcp.with_dw_conv ? pbuf + (oh % jcp_dw->kh) * row_offset
-                                         : &dst[dst_off];
-        p.load_data
-                = &weights[pd()->with_groups() ? weights_d.blk_off(g, ocb, icb)
-                                               : weights_d.blk_off(ocb, icb)];
+                                         : dst + dst_dt_size * dst_off;
+        const auto wei_offset = pd()->with_groups()
+                ? weights_d.blk_off(g, ocb, icb)
+                : weights_d.blk_off(ocb, icb);
+        p.load_data = weights + wei_offset;
         p.bias_data = &bias[_ocb * jcp.oc_block * bia_dt_size];
         p.compensation = (jcp.signed_input) ? &compensation[_ocb * jcp.oc_block]
                                             : nullptr;
@@ -271,28 +276,28 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
                 : nullptr;
         p.src_zero_point = jcp.src_zero_point ? src_zero_point : nullptr;
         p.dst_zero_point = jcp.dst_zero_point ? dst_zero_point : nullptr;
-        p.scales = (jcp.signed_input && jcp.ver != ver_vnni)
-                ? &local_scales[jcp.is_oc_scale * _ocb * jcp.oc_block]
-                : &oscales[jcp.is_oc_scale * _ocb * jcp.oc_block];
+        p.scales = &oscales[jcp.is_oc_scale * _ocb * jcp.oc_block];
+        p.dst_scale = dst_scales;
         const size_t src_off = is_3d
                 ? src_d.blk_off(n, _icb * jcp.ic_block, id, ih, iw)
                 : is_2d ? src_d.blk_off(n, _icb * jcp.ic_block, ih, iw)
                         : src_d.blk_off(n, _icb * jcp.ic_block, iw);
         if (pd()->rtus_.reduce_src_) {
-            rp.ws = rtus_space + ithr * pd()->rtus_.space_per_thread_
-                    + _icb * jcp.is * jcp.ic_block;
+            rp.ws = rtus_space
+                    + src_dt_size
+                            * (ithr * pd()->rtus_.space_per_thread_
+                                    + _icb * jcp.is * jcp.ic_block);
             if (ocb == ocb_start) {
-                rp.src = src + src_off;
+                rp.src = src + src_dt_size * src_off;
                 (*rtus_driver_)(&rp);
             }
             p.bcast_data = rp.ws;
         } else
-            p.bcast_data = src + src_off;
+            p.bcast_data = src + src_dt_size * src_off;
 
-        p.dst_l_off = dst_off;
-        p.oc_l_off = _ocb * jcp.oc_block;
         p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
-        p.dst_orig = dst;
+        p.dst_orig = static_cast<const char *>(p.output_data)
+                - dst_off * dst_dt_size;
 
         (*kernel_)(&p);
     };
@@ -396,8 +401,9 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
                 ocb += jcp_dw->nb_ch_blocking) {
 
             par_conv_dw.src = addrs.data();
-            par_conv_dw.dst = &dst[(dst_offset + jcp_dw->ch_block * ocb)
-                    * jcp_dw->typesize_out];
+            par_conv_dw.dst = dst
+                    + (dst_offset + jcp_dw->ch_block * ocb)
+                            * jcp_dw->typesize_out;
 
             par_conv_dw.filt
                     = weights_dw + dw_weights_d.blk_off(ocb, 0) + wei_stride;
@@ -412,8 +418,8 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
             par_conv_dw.scales = dw_oscales
                     ? &dw_oscales[jcp_dw->is_oc_scale * ocb * jcp_dw->ch_block]
                     : nullptr;
+            par_conv_dw.dst_scale = dw_dst_scales;
 
-            par_conv_dw.oc_l_off = ocb * jcp_dw->ch_block;
             par_conv_dw.post_ops_binary_rhs_arg_vec
                     = post_ops_binary_rhs_arg_vec_dw;
             par_conv_dw.dst_orig = dst;
@@ -427,8 +433,7 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
 
     auto conv_dw = [&]() {
         auto jcp_dw = pd()->jcp_dw_;
-        auto dw_conv_buffer
-                = dw_scratchpad.get<dst_data_t>(key_fusion_inout_buffer);
+        auto dw_conv_buffer = dw_scratchpad.get<char>(key_fusion_inout_buffer);
 
         const auto dw_conv_buffer_size_
                 = (size_t)jcp_dw->kh * jcp.ow * nb_buffer * jcp.oc_block;
@@ -490,16 +495,6 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<src_type,
     }
 }
 
-using namespace data_type;
-template struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<u8, u8>;
-template struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<s8, u8>;
-template struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<u8, s8>;
-template struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<s8, s8>;
-template struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<u8, s32>;
-template struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<s8, s32>;
-template struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<u8, f32>;
-template struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t<s8, f32>;
-
 } // namespace x64
 } // namespace cpu
 } // namespace impl
