@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -39,14 +39,16 @@ struct jit_pp_ker_t : pp_ker_t, public jit_generator {
 
     status_t create_kernel() override { return jit_generator::create_kernel(); }
     void operator()(void *void_dst, const acc_data_t *acc, const char *bias,
-            const float *scales, float sum_scale, float signed_scale, int g,
-            size_t start, size_t end, const zero_point_call_params_t &zp,
+            const float *scales, float dst_scale, float sum_scale,
+            float signed_scale, int g, size_t start, size_t end,
+            const zero_point_call_params_t &zp,
             const void *post_ops_binary_rhs_arg_vec, const void *dst_orig,
             const exec_ctx_t & /* ctx */, const memory_desc_t & /* dst_md */,
             const single_gemm_conv_chunk_desc_t &) const override;
 
 private:
-    void apply_postops(const Xbyak::Reg64 &reg_dst, const int idx);
+    void apply_postops(
+            const Xbyak::Reg64 &reg_dst, const int idx, const size_t offset);
     void generate() override;
     void append_zp_src_comp(size_t offset, int idx, bool apply_mask);
     void load_as_f32(const Xbyak::Zmm &dst, const Xbyak::Opmask &mask,
@@ -60,10 +62,6 @@ private:
     Xbyak::Zmm get_masked_vreg_dst(int idx, bool apply_mask) const;
     Xbyak::Zmm reserve_zmm();
 
-    template <typename T>
-    void advance_binary_postops_off(const T &offset);
-    void zero_binary_postops_off();
-    void set_binary_postops_off(const Xbyak::Reg64 &reg);
     const Xbyak::Opmask &opmask_binary = k2;
 
     struct ker_args_t {
@@ -71,6 +69,7 @@ private:
         const acc_data_t *acc;
         const char *bias;
         const float *scales;
+        float dst_scale;
         float sum_scale;
         float signed_scale;
         size_t len;
@@ -116,11 +115,11 @@ private:
 
     const Xbyak::Reg64 &reg_tmp_comp_
             = r12; // used to broadcast scalar values to vreg
-    const Xbyak::Reg64 &reg_g_oc_off_ = reg_tmp_comp_;
     const Xbyak::Reg64 &reg_zp_src_comp_ = r14;
 
     const Xbyak::Zmm vreg_zero_;
     const Xbyak::Zmm vreg_scale_;
+    const Xbyak::Zmm vreg_dst_scale_;
     const Xbyak::Zmm vreg_sum_scale_;
     const Xbyak::Zmm vreg_signed_scale_;
     const Xbyak::Zmm vreg_saturation_ubound_;
@@ -134,7 +133,6 @@ private:
     const size_t bias_step_factor_;
     const size_t sum_step_factor_;
     const size_t max_unroll_;
-    int dst_l_offset_ = 0;
 
     std::unique_ptr<jit_gemm_x8s8s32x_zp_pad_comp_helper> zp_pad_comp_helper_;
 };
@@ -142,6 +140,7 @@ private:
 jit_pp_ker_t::jit_pp_ker_t(
         const convolution_pd_t *pd, const conv_gemm_conf_t &jcp)
     : pp_ker_t(pd, jcp)
+    , jit_generator(jit_name())
     , number_of_reserved_zmm_regs_(0)
     , bias_data_type_size_(jcp.bias_data_type != data_type::undef
                       ? types::data_type_size(jcp.bias_data_type)
@@ -152,6 +151,7 @@ jit_pp_ker_t::jit_pp_ker_t(
     , vreg_zero_((jcp_.with_eltwise || saturation_needed_) ? reserve_zmm()
                                                            : Xbyak::Zmm(0))
     , vreg_scale_(reserve_zmm())
+    , vreg_dst_scale_(reserve_zmm())
     , vreg_sum_scale_(jcp_.with_sum ? reserve_zmm() : Xbyak::Zmm(0))
     , vreg_signed_scale_(jcp_.signed_input ? reserve_zmm() : Xbyak::Zmm(0))
     , vreg_saturation_ubound_(
@@ -186,8 +186,8 @@ jit_pp_ker_t::jit_pp_ker_t(
 
 #define PARAM_OFF(x) offsetof(ker_args_t, x)
         const rhs_arg_static_params_t rhs_arg_static_params {helper_vmm_idx,
-                r13, r14, preserve_gpr, preserve_vmm,
-                PARAM_OFF(post_ops_binary_rhs_arg_vec),
+                r13, r14, r15, preserve_gpr, preserve_vmm,
+                PARAM_OFF(post_ops_binary_rhs_arg_vec), PARAM_OFF(dst_orig),
                 memory_desc_wrapper(pd->dst_md()), tail_size, opmask_binary,
                 use_exact_tail_scalar_bcast};
 #undef PARAM_OFF
@@ -201,7 +201,7 @@ jit_pp_ker_t::jit_pp_ker_t(
 }
 
 void jit_pp_ker_t::operator()(void *void_dst, const acc_data_t *acc,
-        const char *bias, const float *scales, float sum_scale,
+        const char *bias, const float *scales, float dst_scale, float sum_scale,
         float signed_scale, int g, size_t start, size_t end,
         const zero_point_call_params_t &zp,
         const void *post_ops_binary_rhs_arg_vec, const void *dst_orig,
@@ -225,9 +225,11 @@ void jit_pp_ker_t::operator()(void *void_dst, const acc_data_t *acc,
     const ptrdiff_t g_oc_offset_prologue = g_oc_offset + oc_offset;
     args.bias = bias + g_oc_offset_prologue * bias_data_type_size_;
     args.zp_src = zp.src + (jcp_.zp.src_is_common ? 0 : g_oc_offset_prologue);
-    args.zp_src_comp = zp.src_comp + g_oc_offset_prologue;
+    args.zp_src_comp
+            = zp.src_comp ? zp.src_comp + g_oc_offset_prologue : nullptr;
     args.zp_dst = zp.dst;
     args.scales = scales + jcp_.scale_idx_mult * g_oc_offset_prologue;
+    args.dst_scale = dst_scale;
     args.sum_scale = sum_scale;
     args.signed_scale = signed_scale;
     args.len = end - start;
@@ -258,26 +260,6 @@ void jit_pp_ker_t::operator()(void *void_dst, const acc_data_t *acc,
     jit_generator::operator()(&args);
 }
 
-template <typename T>
-void jit_pp_ker_t::advance_binary_postops_off(const T &offset) {
-    add(reg_g_oc_off_, offset);
-
-    Xbyak::Label end;
-    cmp(reg_g_oc_off_, jcp_.oc);
-    jl(end, T_NEAR);
-    xor_(reg_g_oc_off_, reg_g_oc_off_);
-
-    L(end);
-}
-void jit_pp_ker_t::zero_binary_postops_off() {
-    xor_(reg_g_oc_off_, reg_g_oc_off_);
-    dst_l_offset_ = 0;
-}
-void jit_pp_ker_t::set_binary_postops_off(const Xbyak::Reg64 &reg) {
-    mov(reg_g_oc_off_, reg);
-    dst_l_offset_ = 0;
-}
-
 Xbyak::Zmm jit_pp_ker_t::reserve_zmm() {
     return Xbyak::Zmm(number_of_reserved_zmm_regs_++);
 }
@@ -323,28 +305,18 @@ void jit_pp_ker_t::append_zp_src_comp(size_t offset, int idx, bool apply_mask) {
                 });
 }
 
-void jit_pp_ker_t::apply_postops(const Xbyak::Reg64 &reg_dst, const int idx) {
+void jit_pp_ker_t::apply_postops(
+        const Xbyak::Reg64 &reg_dst, const int idx, const size_t offset) {
 #define PARAM_OFF(x) offsetof(ker_args_t, x)
     if (jcp_.with_eltwise || jcp_.with_binary) {
         if (jcp_.with_binary) {
             binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
-            const auto dst_offset_reg = reg_dst;
             const auto vmm_idx = vreg_dst_idx(idx);
-            rhs_arg_params.vmm_idx_to_oc_elem_off_addr.emplace(
-                    vmm_idx, ptr[reg_param_ + PARAM_OFF(g_oc_offset)]);
-            rhs_arg_params.vmm_idx_to_oc_off_oprnd.emplace(
-                    vmm_idx, reg_g_oc_off_);
-            rhs_arg_params.vmm_idx_to_out_off_oprnd.emplace(
-                    vmm_idx, dst_offset_reg);
-            rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(
-                    vmm_idx, dst_l_offset_);
-            rhs_arg_params.vmm_tail_idx_.emplace(vmm_idx);
 
-            const injector_utils::register_preserve_guard_t register_guard(
-                    this, {dst_offset_reg});
-            sub(dst_offset_reg, ptr[reg_param_ + PARAM_OFF(dst_orig)]);
-            const auto size = sizeof(jcp_.dst_data_type);
-            if (size) shr(dst_offset_reg, std::log2(size));
+            rhs_arg_params.vmm_idx_to_out_reg.emplace(vmm_idx, reg_dst);
+            rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(vmm_idx,
+                    offset * types::data_type_size(jcp_.dst_data_type));
+            rhs_arg_params.vmm_tail_idx_.emplace(vmm_idx);
 
             postops_injector_->compute_vector(
                     vreg_dst_idx(idx), rhs_arg_params);
@@ -408,6 +380,8 @@ void jit_pp_ker_t::generate() {
         vcvtdq2ps(vreg_zp_dst_common_, ptr_b[reg_tmp_]);
     }
 
+    if (jcp_.with_dst_scale)
+        vbroadcastss(vreg_dst_scale_, ptr[reg_param_ + PARAM_OFF(dst_scale)]);
     if (jcp_.with_sum)
         vbroadcastss(vreg_sum_scale_, ptr[reg_param_ + PARAM_OFF(sum_scale)]);
     if (jcp_.signed_input)
@@ -426,8 +400,6 @@ void jit_pp_ker_t::generate() {
         init_saturate_f32(vreg_zero_, vreg_saturation_ubound_, reg_tmp_comp_,
                 data_type::f32, jcp_.dst_data_type);
 
-    if (jcp_.with_binary) set_binary_postops_off(reg_oc_offset_);
-
     // Load accumulated value, convert to float, apply sum (if any),
     // bias (if any), scaling, and relu (if any);
     // then convert to destination type and store
@@ -445,13 +417,8 @@ void jit_pp_ker_t::generate() {
             vmovups(vreg_scale, scale_addr);
         }
 
-        if (jcp_.with_binary) {
-            if (offset) {
-                advance_binary_postops_off(vlen);
-                dst_l_offset_ += offset;
-            }
-            kmovq(opmask_binary, mask_reg);
-        }
+        if (jcp_.with_binary) kmovq(opmask_binary, mask_reg);
+
         const auto vreg_dst_masked = get_masked_vreg_dst(idx, apply_mask);
         const auto vreg_dst = get_vreg_dst(idx);
         if (jcp_.zp.src_exists) {
@@ -465,6 +432,8 @@ void jit_pp_ker_t::generate() {
         if (jcp_.signed_input)
             vmulps(vreg_dst_masked, vreg_dst, vreg_signed_scale_);
 
+        vmulps(vreg_dst_masked, vreg_dst, vreg_scale_);
+
         if (jcp_.with_bias) {
             const auto bias_addr
                     = ptr[reg_bias_ + offset * bias_data_type_size_];
@@ -473,26 +442,27 @@ void jit_pp_ker_t::generate() {
             vaddps(vreg_dst_masked, vreg_dst, vreg_bias);
         }
 
-        vmulps(vreg_dst_masked, vreg_dst, vreg_scale_);
-
         const auto dst_addr = ptr[reg_dst_ + offset * dst_data_type_size_];
 
         if (jcp_.with_sum) {
             const auto vreg_prev_dst = get_vreg_prev_dst(idx);
-            load_as_f32(vreg_prev_dst, mask_reg, dst_addr, jcp_.dst_data_type);
+            load_as_f32(vreg_prev_dst, mask_reg, dst_addr, jcp_.sum_data_type);
             vfmadd231ps(vreg_dst_masked, vreg_prev_dst, vreg_sum_scale_);
         }
 
-        apply_postops(reg_dst_, idx);
+        apply_postops(reg_dst_, idx, offset);
+
+        if (jcp_.with_dst_scale) {
+            vmulps(vreg_dst_masked, vreg_dst, vreg_dst_scale_);
+        }
 
         if (jcp_.zp.dst_exists) {
             vaddps(vreg_dst_masked, vreg_dst, vreg_zp_dst_common_);
         }
 
         if (saturation_needed_) {
-            saturate_f32(get_vreg_dst(idx), vreg_zero_, vreg_saturation_ubound_,
-                    jcp_.dst_data_type);
-            vcvtps2dq(vreg_dst_masked, vreg_dst);
+            saturate_cvt_f32(get_vreg_dst(idx), vreg_zero_,
+                    vreg_saturation_ubound_, jcp_.dst_data_type);
         }
 
         switch (jcp_.dst_data_type) {
@@ -509,7 +479,6 @@ void jit_pp_ker_t::generate() {
                                           const size_t binary_offset) {
         add(reg_dst_, offset * dst_data_type_size_);
         add(reg_acc_, offset * sizeof(acc_data_t));
-        if (jcp_.with_binary) { advance_binary_postops_off(binary_offset); }
         if (jcp_.scale_idx_mult) {
             assert(jcp_.scale_idx_mult == 1);
             add(reg_scales_, offset * sizeof(float));
@@ -532,7 +501,6 @@ void jit_pp_ker_t::generate() {
                                           const Reg64 binary_offset) {
         lea(reg_dst_, ptr[reg_dst_ + offset * dst_data_type_size_]);
         lea(reg_acc_, ptr[reg_acc_ + offset * sizeof(acc_data_t)]);
-        if (jcp_.with_binary) { advance_binary_postops_off(binary_offset); }
         if (jcp_.scale_idx_mult) {
             assert(jcp_.scale_idx_mult == 1);
             lea(reg_scales_, ptr[reg_scales_ + offset * sizeof(float)]);
@@ -558,10 +526,6 @@ void jit_pp_ker_t::generate() {
     // (bias or per-oc scaling factors)
     const auto rewind_ptrs = [&]() {
         if (jcp_.with_bias) sub(reg_bias_, jcp_.oc * bias_data_type_size_);
-        if (jcp_.with_binary) {
-            zero_binary_postops_off();
-            dst_l_offset_ = 0;
-        }
         if (jcp_.zp.src_exists) {
             const auto offset = jcp_.oc * sizeof(int32_t);
             sub(reg_zp_src_comp_, offset);
@@ -721,23 +685,27 @@ void jit_pp_ker_t::generate() {
 
     postamble();
 
-    if (jcp_.with_eltwise) postops_injector_->prepare_table();
+    if (jcp_.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 }
 
-bool mayiuse_jit_pp_kernel() noexcept {
-    return mayiuse(avx512_core);
+bool mayiuse_jit_pp_kernel(data_type_t dst_dt) noexcept {
+    const auto is_bf16_dst_dt = dst_dt == data_type::bf16;
+    return mayiuse(avx512_core) && !is_bf16_dst_dt;
 }
 
 pp_ker_t *jit_pp_ker_create(
         const convolution_pd_t *pd, const conv_gemm_conf_t &jcp) {
-    return mayiuse_jit_pp_kernel() ? new jit_pp_ker_t(pd, jcp) : nullptr;
+    return mayiuse_jit_pp_kernel(pd->dst_md()->data_type)
+            ? new jit_pp_ker_t(pd, jcp)
+            : nullptr;
 }
 
 bool post_ops_ok(const post_ops_t &post_ops, const memory_desc_wrapper *dst_d) {
     using namespace x64::injector;
     static constexpr bool sum_at_pos_0_only = true;
     static constexpr bool sum_requires_scale_one = false;
-    return mayiuse_jit_pp_kernel()
+    return mayiuse_jit_pp_kernel(dst_d->data_type())
             && dnnl::impl::cpu::x64::injector::post_ops_ok(
                     {avx512_core, {binary, eltwise, sum}, post_ops, dst_d,
                             sum_at_pos_0_only, sum_requires_scale_one});
