@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 #include <assert.h>
 
 #include "common/c_types_map.hpp"
+#include "common/convolution_pd.hpp"
 #include "common/memory.hpp"
 #include "common/memory_tracking.hpp"
 #include "common/nstl.hpp"
@@ -47,15 +48,18 @@ template <cpu_isa_t isa, typename Vmm>
 _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::_jit_uni_x8s8s32x_1x1_conv_kernel(
         const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, true, isa), jcp(ajcp), attr_(attr) {
+    : jit_generator(jit_name(), isa), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
         static constexpr bool preserve_vmm = true;
-        rhs_arg_static_params_t rhs_arg_static_params {15, r13, r14,
+        const size_t tail_size = get_tail_size();
+        static constexpr bool use_exact_tail_scalar_bcast = false;
+        rhs_arg_static_params_t rhs_arg_static_params {15, r13, r14, r15,
                 preserve_gpr, preserve_vmm,
-                GET_OFF(post_ops_binary_rhs_arg_vec),
-                memory_desc_wrapper(dst_md)};
+                GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
+                memory_desc_wrapper(dst_md), tail_size,
+                use_exact_tail_scalar_bcast};
         static_params_t static_params {this->param1, rhs_arg_static_params};
 
         postops_injector_
@@ -89,7 +93,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::bcast_loop(
     L(bcast_loop);
     {
         assert(jcp.bcast_block == jcp.ur);
-        reduce_loop(load_loop_blk, jcp.ur, 0, false);
+        reduce_loop(load_loop_blk, jcp.ur, false);
         add(aux1_reg_bcast_data, jcp.bcast_loop_bcast_step);
         add(aux_reg_output_data, jcp.bcast_loop_output_step);
 
@@ -103,7 +107,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::bcast_loop(
         Label bcast_loop_tail_out;
         cmp(reg_bcast_loop_iter, 0);
         jz(bcast_loop_tail_out, T_NEAR);
-        reduce_loop(load_loop_blk, jcp.ur_tail, 0, true);
+        reduce_loop(load_loop_blk, jcp.ur_tail, true);
         L(bcast_loop_tail_out);
     }
 }
@@ -142,22 +146,29 @@ void iterate(const int ur, const int load_loop_blk, const F &f) {
 template <cpu_isa_t isa, typename Vmm>
 void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::apply_sum(const int ur,
         const int load_loop_blk, const bool mask_flag_in,
-        const float *p_sum_scale) {
+        const float *p_sum_scale, const int32_t *p_sum_zp) {
 
     if (jcp.with_sum) {
-        assert(p_sum_scale != nullptr && "p_sum_scale = nullptr");
+        assert(!utils::any_null(p_sum_scale, p_sum_zp)
+                && "p_sum_scale or p_sum_zp = nullptr");
         const float sum_scale = *p_sum_scale;
+        const int32_t sum_zp = *p_sum_zp;
         const auto sum_injector_lam = [this, mask_flag_in, load_loop_blk,
-                                              sum_scale](const int i_ur,
+                                              sum_scale, sum_zp](const int i_ur,
                                               const int i_load) {
             const bool mask_flag = mask_flag_in && i_load == load_loop_blk - 1;
             const auto ymm_prev_dst = vmm_zero;
 
             const auto r = vreg_accum(load_loop_blk, i_load, i_ur);
-            cvt2ps(jcp.dst_dt, ymm_prev_dst, aux_reg_output_data,
+            cvt2ps(jcp.sum_dt, ymm_prev_dst, aux_reg_output_data,
                     output_ptr(i_load, i_ur),
                     mask_flag ? get_tail_size() : simd_w);
 
+            if (sum_zp != 0) {
+                uni_vbroadcastss(vmm_tmp, ptr[reg_ptr_sum_zp]);
+                uni_vcvtdq2ps(vmm_tmp, vmm_tmp);
+                uni_vsubps(vmm_prev_dst, vmm_prev_dst, vmm_tmp);
+            }
             if (sum_scale == 1.f)
                 uni_vaddps(r, r, ymm_prev_dst);
             else {
@@ -167,6 +178,8 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::apply_sum(const int ur,
         };
         const auto sum_injector
                 = [=]() { iterate(ur, load_loop_blk, sum_injector_lam); };
+        if (sum_zp != 0)
+            mov(reg_ptr_sum_zp, reinterpret_cast<size_t>(p_sum_zp));
         postops_injector_->set_lambda_injector(
                 primitive_kind::sum, sum_injector);
     }
@@ -175,52 +188,61 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::apply_sum(const int ur,
 template <cpu_isa_t isa, typename Vmm>
 void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::apply_postops(const int ur,
         const int load_loop_blk, const bool mask_flag_in,
-        const float *p_sum_scale) {
+        const float *p_sum_scale, const int32_t *p_sum_zp) {
 
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
-        apply_sum(ur, load_loop_blk, mask_flag_in, p_sum_scale);
+        if (jcp.with_sum && *p_sum_zp != 0)
+            mov(ptr[rsp + reg_bcast_loop_iter_off], reg_ptr_sum_zp);
+        apply_sum(ur, load_loop_blk, mask_flag_in, p_sum_scale, p_sum_zp);
 
-        binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
+        binary_injector::rhs_arg_dynamic_params_t rhs_arg_params,
+                rhs_arg_params_tail;
         vmm_index_set_t vmm_idxs;
         if (jcp.with_binary) {
-            const auto oc_off_oprnd = r12;
             iterate(ur, load_loop_blk, [&](const int i_ur, const int i_load) {
-                const int ur_stride = jcp.with_dw_conv
-                        ? jcp.nb_load_blocking * jcp.oc_block * i_ur
-                        : jcp.oc_without_padding * jcp.ngroups * i_ur;
-                const int aux_output_offset = jcp.typesize_out
+                const int ur_stride
+                        = jcp.oc_without_padding * jcp.ngroups * i_ur;
+                const size_t aux_output_offset = jcp.typesize_out
                         * (ur_stride + i_load * jcp.load_block);
                 const auto vmm_idx
                         = vreg_accum_idx(load_loop_blk, i_load, i_ur);
                 vmm_idxs.emplace(vmm_idx);
-                rhs_arg_params.vmm_idx_to_oc_elem_off_addr.emplace(
-                        vmm_idx, ptr[param1 + GET_OFF(oc_l_off)]);
-                rhs_arg_params.vmm_idx_to_oc_elem_off_val.emplace(
-                        vmm_idx, i_load * jcp.load_block);
-                rhs_arg_params.vmm_idx_to_oc_off_oprnd.emplace(
-                        vmm_idx, oc_off_oprnd);
-                rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(
-                        vmm_idx, aux_output_offset);
-            });
 
-            const injector_utils::register_preserve_guard_t register_guard(
-                    this, {oc_off_oprnd});
-            mov(oc_off_oprnd,
-                    ptr[rsp + reg_binary_post_op_acc_off
-                            + register_guard.stack_space_occupied()]);
+                rhs_arg_params_tail.vmm_idx_to_out_reg.emplace(
+                        vmm_idx, aux_reg_output_data);
+                rhs_arg_params_tail.vmm_idx_to_out_elem_off_val.emplace(
+                        vmm_idx, aux_output_offset);
+                if (mask_flag_in && (i_load + 1 == load_loop_blk))
+                    rhs_arg_params_tail.vmm_tail_idx_.emplace(vmm_idx);
+            });
+            rhs_arg_params = rhs_arg_params_tail;
+            rhs_arg_params.vmm_tail_idx_.clear();
+            Label postops_done;
+            if (mask_flag_in) {
+                Label postops_no_tail;
+                test(reg_reduce_pos_flag, FLAG_OC_LAST);
+                je(postops_no_tail, T_NEAR);
+                postops_injector_->compute_vector_range(
+                        vmm_idxs, rhs_arg_params_tail);
+                jmp(postops_done, T_NEAR);
+                L(postops_no_tail);
+            }
             postops_injector_->compute_vector_range(vmm_idxs, rhs_arg_params);
+            L(postops_done);
         } else {
             iterate(ur, load_loop_blk, [&](const int i_ur, const int i_load) {
                 vmm_idxs.emplace(vreg_accum_idx(load_loop_blk, i_load, i_ur));
             });
             postops_injector_->compute_vector_range(vmm_idxs, rhs_arg_params);
         }
+        if (jcp.with_sum && *p_sum_zp != 0)
+            mov(reg_ptr_sum_zp, ptr[rsp + reg_bcast_loop_iter_off]);
     }
 }
 
 template <cpu_isa_t isa, typename Vmm>
 void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
-        int load_loop_blk, int ur, int substep, bool wraparound) {
+        int load_loop_blk, int ur, bool wraparound) {
 
     // use 0x10001 to represent 2 words of 0x1
     // and avoid using uni_vpbroadcastb that is missing in jit generator
@@ -237,14 +259,6 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
         return Vmm(15 - vmm_idx);
     };
 
-    auto vmm_bias_alpha = [&]() {
-        const int vmm_idx = ur * load_loop_blk;
-        assert(vmm_idx < ker_max_reg_idx);
-        return Vmm(15 - vmm_idx);
-    };
-
-    auto xmm_bias_alpha = [&]() { return Xmm(vmm_bias_alpha().getIdx()); };
-
     auto bcast_ptr = [&](int i_reduce, int i_ur) {
         assert(i_ur < jcp.ur);
         assert(i_reduce <= jcp.reduce_loop_unroll);
@@ -287,16 +301,13 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
         const int sum_idx = p.find(primitive_kind::sum);
         const float *p_sum_scale
                 = (sum_idx != -1) ? &p.entry_[sum_idx].sum.scale : nullptr;
+        const int32_t *p_sum_zp
+                = (sum_idx != -1) ? &p.entry_[sum_idx].sum.zero_point : nullptr;
         mov(ptr[rsp + reg_bcast_data_off], reg_bcast_data);
         mov(reg_ptr_scales, ptr[rsp + reg_ptr_sum_scale_off]);
         if (p_sum_scale && *p_sum_scale != 1.f) {
             mov(ptr[rsp + reg_load_data_off], reg_load_data);
-            mov(reg_ptr_sum_scale, (size_t)p_sum_scale);
-        }
-        if (jcp.signed_input && jcp.ver != ver_vnni) {
-            mov(reg_store_bcast, float2int(jcp.wei_adj_scale));
-            uni_vmovq(xmm_bias_alpha(), reg_store_bcast);
-            uni_vbroadcastss(vmm_bias_alpha(), xmm_bias_alpha());
+            mov(reg_ptr_sum_scale, reinterpret_cast<size_t>(p_sum_scale));
         }
         if (jcp.src_zero_point) {
             mov(reg_zp_compensation, ptr[rsp + reg_zp_compensation_off]);
@@ -311,12 +322,10 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
             const auto ptr_scales_offset
                     = jcp.is_oc_scale * (sizeof(float) * jcp.oc_block * i_load);
             if (jcp.with_bias) {
-                if (jcp.signed_input)
+                if (jcp.signed_input || jcp.dst_scale)
                     mov(reg_bias_data, ptr[rsp + reg_bias_data_off]);
                 cvt2ps(jcp.bia_dt, vmm_bias, reg_bias_data,
                         jcp.typesize_bia * jcp.oc_block * i_load, load_size);
-                if (jcp.signed_input && jcp.ver != ver_vnni)
-                    uni_vmulps(vmm_bias, vmm_bias, vmm_bias_alpha());
             }
             if (jcp.signed_input) {
                 mov(reg_comp_data, ptr[rsp + reg_comp_data_off]);
@@ -346,13 +355,27 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
                 uni_vcvtdq2ps(r, r);
                 if (jcp.signed_input) uni_vaddps(r, r, vmm_comp);
                 if (jcp.src_zero_point) uni_vaddps(r, r, vmm_zp_comp);
-                if (jcp.with_bias) uni_vaddps(r, r, vmm_bias);
 
                 uni_vmulps(r, r, vmm_scale);
+
+                if (jcp.with_bias) uni_vaddps(r, r, vmm_bias);
             }
         }
 
-        apply_postops(ur, load_loop_blk, mask_flag_in, p_sum_scale);
+        apply_postops(ur, load_loop_blk, mask_flag_in, p_sum_scale, p_sum_zp);
+
+        if (jcp.dst_scale) {
+            mov(reg_ptr_dst_scale, ptr[rsp + reg_dst_scale_off]);
+            uni_vmovups(vmm_dst_scale, ptr[reg_ptr_dst_scale]);
+
+            /* Apply dst scale to accumulator */
+            for (int i_load = 0; i_load < load_loop_blk; ++i_load) {
+                for (int i_ur = 0; i_ur < ur; ++i_ur) {
+                    const auto r = vreg_accum(load_loop_blk, i_load, i_ur);
+                    uni_vmulps(r, r, vmm_dst_scale);
+                }
+            }
+        }
 
         if (jcp.dst_zero_point) {
             mov(reg_dst_zero_point, ptr[rsp + reg_dst_zero_point_off]);
@@ -376,8 +399,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
             for (int i_ur = 0; i_ur < ur; ++i_ur)
                 for (int i_load = 0; i_load < load_loop_blk; ++i_load) {
                     auto r = vreg_accum(load_loop_blk, i_load, i_ur);
-                    saturate_f32(r, vmm_zero, vmm_saturation, jcp.dst_dt);
-                    uni_vcvtps2dq(r, r);
+                    saturate_cvt_f32(r, vmm_zero, vmm_saturation, jcp.dst_dt);
                 }
         }
 
@@ -398,7 +420,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
     };
 
     auto compute = [&](Vmm vreg_acc, Vmm vreg_wei, Vmm vreg_src) {
-        if (jcp.ver == ver_vnni) {
+        if (jcp.has_vnni) {
             vpdpbusd(vreg_acc, vreg_src, vreg_wei, VexEncoding);
         } else {
             uni_vpmaddubsw(vmm_tmp, vreg_src, vreg_wei);
@@ -493,12 +515,6 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
     preamble();
 
     sub(rsp, stack_space_needed);
-    if (jcp.with_binary) {
-        // zero initialize binary post_ops offset accumulator (store on stack)
-        const auto binary_post_op_acc_off_reg = r15;
-        xor_(binary_post_op_acc_off_reg, binary_post_op_acc_off_reg);
-        mov(ptr[rsp + reg_binary_post_op_acc_off], binary_post_op_acc_off_reg);
-    }
 
     if (jcp.with_bias) mov(reg_bias_data, ptr[param1 + GET_OFF(bias_data)]);
     if (jcp.signed_input) {
@@ -512,6 +528,11 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
         mov(reg_src_zero_point, ptr[param1 + GET_OFF(src_zero_point)]);
         mov(ptr[rsp + reg_src_zero_point_off], reg_src_zero_point);
     }
+    if (jcp.dst_scale) {
+        if (!jcp.signed_input) mov(ptr[rsp + reg_bias_data_off], reg_bias_data);
+        mov(reg_ptr_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
+        mov(ptr[rsp + reg_dst_scale_off], reg_ptr_dst_scale);
+    }
     if (jcp.dst_zero_point) {
         mov(reg_dst_zero_point, ptr[param1 + GET_OFF(dst_zero_point)]);
         mov(ptr[rsp + reg_dst_zero_point_off], reg_dst_zero_point);
@@ -532,20 +553,13 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
         bcast_loop(load_loop_blk);
         add(reg_load_data, load_loop_blk * jcp.load_loop_load_step);
         if (jcp.with_bias) {
-            if (jcp.signed_input)
+            if (jcp.signed_input || jcp.dst_scale)
                 mov(reg_bias_data, ptr[rsp + reg_bias_data_off]);
             add(reg_bias_data,
                     load_loop_blk * jcp.load_block * jcp.typesize_bia);
-            if (jcp.signed_input)
+            if (jcp.signed_input || jcp.dst_scale)
                 mov(ptr[rsp + reg_bias_data_off], reg_bias_data);
         }
-        if (jcp.with_binary) {
-            mov(aux_reg_load_data,
-                    EVEX_compress_addr(rsp, reg_binary_post_op_acc_off));
-            add(aux_reg_load_data, jcp.load_block * load_loop_blk);
-            mov(EVEX_compress_addr(rsp, reg_binary_post_op_acc_off),
-                    aux_reg_load_data);
-        }
         if (jcp.signed_input) {
             mov(reg_comp_data, ptr[rsp + reg_comp_data_off]);
             add(reg_comp_data,
@@ -613,7 +627,8 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
     add(rsp, stack_space_needed);
     postamble();
 
-    if (jcp.with_eltwise) postops_injector_->prepare_table();
+    if (jcp.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 }
 
 template <cpu_isa_t isa>
@@ -621,15 +636,22 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
         jit_1x1_conv_conf_t &jcp, const convolution_desc_t &cd,
         const memory_desc_wrapper &src_d, const memory_desc_wrapper &weights_d,
         const memory_desc_wrapper &dst_d, const memory_desc_wrapper &bias_d,
-        const primitive_attr_t &attr, int nthreads, bool reduce_src) {
+        primitive_attr_t &attr, int nthreads, bool reduce_src) {
+    // disabling verbose dispatch messages for unsupported isa for better readability
     if (!mayiuse(isa)) return status::unimplemented;
 
+    // Big int (> INT_MAX) values are unsupported and jcp fields may overflow
+    // TODO: change data type of jcp fields to size_t
+    VDISPATCH_CONV_IC(!has_large_size(cd, src_d, weights_d, dst_d),
+            VERBOSE_BAD_PARAM, "Large size is not supported");
+
     const bool with_groups = weights_d.ndims() == src_d.ndims() + 1;
-    if (!one_of(src_d.data_type(), data_type::u8, data_type::s8)
+    const bool dt_not_ok
+            = !one_of(src_d.data_type(), data_type::u8, data_type::s8)
             || weights_d.data_type() != data_type::s8
             || !one_of(dst_d.data_type(), data_type::f32, data_type::s32,
-                    data_type::s8, data_type::u8))
-        return status::unimplemented;
+                    data_type::s8, data_type::u8);
+    VDISPATCH_CONV_IC(!dt_not_ok, VERBOSE_UNSUPPORTED_DT_CFG);
 
     const int ndims = src_d.ndims();
     jcp.nthr = nthreads;
@@ -658,8 +680,8 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
 
     jcp.signed_input = (src_d.data_type() == data_type::s8);
 
-    jcp.os = jcp.od * jcp.oh * jcp.ow;
-    jcp.is = jcp.id * jcp.ih * jcp.iw;
+    jcp.os = static_cast<dim_t>(jcp.od) * jcp.oh * jcp.ow;
+    jcp.is = static_cast<dim_t>(jcp.id) * jcp.ih * jcp.iw;
 
     const auto &post_ops = attr.post_ops_;
     const int dw_conv_ind = post_ops.find(primitive_kind::convolution);
@@ -672,7 +694,8 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
 
     const int binary_ind
             = post_ops.find(primitive_kind::binary, 0, dw_conv_ind);
-    jcp.with_binary = binary_ind != -1;
+    const int prelu_ind = post_ops.find(primitive_kind::prelu, 0, dw_conv_ind);
+    jcp.with_binary = !everyone_is(-1, binary_ind, prelu_ind);
 
     const int sum_ind = post_ops.find(primitive_kind::sum, 0, dw_conv_ind);
     jcp.with_sum = sum_ind != -1;
@@ -684,19 +707,20 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
             = zp.common(DNNL_ARG_SRC); // otherwise, it's per-channel
     assert(IMPLICATION(jcp.src_zero_point, jcp.zp_src_is_common));
 
-    if ((jcp.dst_zero_point || jcp.src_zero_point) && jcp.with_dw_conv)
-        return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            !((jcp.dst_zero_point || jcp.src_zero_point) && jcp.with_dw_conv),
+            VERBOSE_UNSUPPORTED_FEATURE, "does not support zero-point");
 
     format_tag_t dat_tag = utils::pick(
             ndims - 3, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
-    jcp.src_tag = src_d.matches_one_of_tag(dat_tag);
-    jcp.dst_tag = dst_d.matches_one_of_tag(dat_tag);
+    jcp.src_tag = src_d.mb_stride_relaxed_match(dat_tag);
+    jcp.dst_tag = dst_d.mb_stride_relaxed_match(dat_tag);
 
     bool args_ok = true && jcp.ngroups == 1 && jcp.src_tag == dat_tag
             && jcp.dst_tag == dat_tag;
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(args_ok, VERBOSE_UNSUPPORTED_TAG);
 
-    jcp.ver = mayiuse(avx2_vnni) ? ver_vnni : ver_unused;
+    jcp.has_vnni = mayiuse(avx2_vnni);
 
     jcp.oc = rnd_up(jcp.oc, simd_w);
     jcp.ic = rnd_up(jcp.ic, simd_w);
@@ -709,25 +733,21 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
         jcp.post_ops = post_ops;
     }
 
-    for (auto &post_op : jcp.post_ops.entry_)
-        if (post_op.is_binary() && post_op.binary.src1_desc.dims[1] != 1) {
-            post_op.binary.src1_desc.dims[1] = jcp.oc;
-        }
-
     using namespace injector;
-    const bool post_ops_ok_
-            = post_ops_ok({isa, {eltwise, binary, sum}, jcp.post_ops, &dst_d});
-    if (!post_ops_ok_) return status::unimplemented;
+    const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(isa,
+            {eltwise, binary, sum}, jcp.post_ops, &dst_d, false, false, false));
+    VDISPATCH_CONV_IC(post_ops_ok_, VERBOSE_UNSUPPORTED_POSTOP);
 
     args_ok = true && jcp.oc % simd_w == 0 && jcp.ic % simd_w == 0
             && jcp.f_pad == 0 && jcp.t_pad == 0 && jcp.l_pad == 0
             && jcp.stride_w == 1 && jcp.stride_h == 1 && jcp.stride_d == 1
             && jcp.ow == jcp.iw && jcp.oh == jcp.ih && jcp.od == jcp.id
             && jcp.kd == 1 && jcp.kh == 1 && jcp.kw == 1;
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(args_ok, VERBOSE_BAD_PARAM, "");
 
     jcp.bia_dt = jcp.with_bias ? cd.bias_desc.data_type : data_type::undef;
     jcp.dst_dt = cd.dst_desc.data_type;
+    jcp.sum_dt = post_ops.get_sum_dt(jcp.dst_dt);
 
     jcp.ic_block = jcp.oc_block = simd_w;
 
@@ -760,7 +780,7 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
             && (jcp.oh <= size_threshold && jcp.ow <= size_threshold)) {
         if (jcp.os <= SMALL_SPATIAL && jcp.oc * jcp.ic < L2_size)
             max_regs = min_regs = 3;
-        jcp.ur = nstl::min(max_regs, jcp.os);
+        jcp.ur = nstl::min<dim_t>(max_regs, jcp.os);
     } else {
         const int spatial = jcp.od * jcp.oh;
         jcp.ur = 1;
@@ -772,7 +792,7 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
             }
         }
         if (jcp.ur == 1) {
-            jcp.ur = nstl::min(max_regs, jcp.os);
+            jcp.ur = nstl::min<dim_t>(max_regs, jcp.os);
             int os_tail = jcp.os % max_regs;
             for (int i = max_regs; i >= min_regs; i--) {
                 int i_tail = jcp.os % i;
@@ -845,7 +865,7 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
     bcast_blocking = div_up(jcp.mb * jcp.ngroups * nb_bcast,
                              div_up(jcp.nthr, jcp.load_grp_count))
             * jcp.bcast_block;
-    bcast_blocking = nstl::min(jcp.bcast_dim, bcast_blocking);
+    bcast_blocking = nstl::min<dim_t>(jcp.bcast_dim, bcast_blocking);
     bcast_blocking = rnd_up(bcast_blocking, jcp.bcast_block);
 
     int space_for_bcast = (L2_capacity - /* kernel_size - */
@@ -875,7 +895,7 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
             && jcp.reduce_dim % jcp.reduce_block == 0;
 
     assert(params_ok && "parameter values are inconsistent");
-    if (!params_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(params_ok, VERBOSE_BLOCKING_FAIL, "bad parameters");
 
     jcp.ur_tail = (jcp.with_dw_conv ? jcp.ow : jcp.bcast_dim) % jcp.ur;
 
@@ -893,12 +913,10 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
     // miniumum size of load dim chunk for work distribution within threads
     jcp.nb_load_chunk = 1;
 
-    const auto &oscales = attr.output_scales_;
-    jcp.is_oc_scale = oscales.mask_ == 1 << 1;
-
-    // only common and per-oc-channel scales are supported
-    const bool oscales_ok = one_of(oscales.mask_, 0, 1 << 1);
-    if (!oscales_ok) return status::unimplemented;
+    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
+    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
+    jcp.is_oc_scale = wei_scales.mask_ != 0;
+    jcp.dst_scale = !dst_scales.has_default_values();
 
     jcp.wei_adj_scale
             = (weights_d.extra().flags & memory_extra_flags::scale_adjust)
@@ -914,10 +932,11 @@ void jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_scratchpad(
         const jit_1x1_conv_conf_t &jcp, const primitive_attr_t &attr) {
     using namespace dnnl::impl::memory_tracking::names;
 
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        dim_t count = nstl::max<dim_t>(attr.output_scales_.count_, 8);
-        scratchpad.book<float>(key_conv_adjusted_scales, count);
-    }
+    const int wei_mask = attr.scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const dim_t scales_count
+            = wei_mask == 0 ? 1 : static_cast<dim_t>(jcp.oc) * jcp.ngroups;
+    const dim_t count = nstl::max<dim_t>(scales_count, 8);
+    scratchpad.book<float>(key_conv_adjusted_scales, count);
 }
 
 template struct _jit_uni_x8s8s32x_1x1_conv_kernel<avx2, Ymm>;
