@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -39,15 +39,32 @@ using namespace nstl;
     (pd()->with_groups() ? (d).blk_off((g), __VA_ARGS__) \
                          : (d).blk_off(__VA_ARGS__))
 
-template <cpu_isa_t isa, data_type_t src_type, data_type_t dst_type>
-status_t
-jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_2d(
+template <cpu_isa_t isa>
+const float *jit_uni_x8s8s32x_convolution_fwd_t<isa>::adjust_oscales(
+        const memory_tracking::grantor_t &scratchpad, const float *src_scales,
+        const float *wei_scales) const {
+    auto loc_scales = scratchpad.template get<float>(key_conv_adjusted_scales);
+    int wei_mask = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    float factor = (pd()->jcp_.signed_input && (!pd()->jcp_.has_vnni))
+            ? 1.f / pd()->jcp_.wei_adj_scale
+            : 1.0f;
+    if (wei_mask == 0) {
+        utils::array_set(loc_scales, src_scales[0] * wei_scales[0] * factor, 8);
+    } else {
+        for (dim_t c = 0; c < pd()->OC(); c++)
+            loc_scales[c] = src_scales[0] * wei_scales[c] * factor;
+    }
+    return loc_scales;
+}
+
+template <cpu_isa_t isa>
+status_t jit_uni_x8s8s32x_convolution_fwd_t<isa>::execute_forward_2d(
         const exec_ctx_t &ctx) const {
     const auto &jcp = pd()->jcp_;
-    auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
-    auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
+    auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
+    auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
-    auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    auto dst = CTX_OUT_MEM(char *, DNNL_ARG_DST);
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(pd()->jcp_.post_ops, ctx);
 
@@ -59,32 +76,24 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_2d(
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
 
-    const size_t bia_dt_size = pd()->with_bias()
-            ? types::data_type_size(pd()->desc()->bias_desc.data_type)
-            : 0;
+    const size_t bia_dt_size
+            = pd()->with_bias() ? types::data_type_size(bias_d.data_type()) : 0;
+    const size_t dst_dt_size = types::data_type_size(dst_d.data_type());
 
     assert(jcp.ch_block == 1);
     assert(jcp.nb_ch_blocking == 1);
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
     assert(jcp.nb_ch % jcp.nb_ch_blocking == 0);
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        auto local_scales = ctx.get_scratchpad_grantor().template get<float>(
-                key_conv_adjusted_scales);
-        size_t count = pd()->attr()->output_scales_.count_;
-        float factor = 1.f / pd()->jcp_.wei_adj_scale;
-        if (count == 1) {
-            utils::array_set(local_scales, oscales[0] * factor, 8);
-        } else {
-            for (size_t c = 0; c < count; c++)
-                local_scales[c] = oscales[c] * factor;
-        }
-        oscales = local_scales;
-    }
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = adjust_oscales(
+            ctx.get_scratchpad_grantor(), src_scales, wei_scales);
 
     size_t offset = weights_d.size() - weights_d.additional_buffer_size();
-    auto w = const_cast<wei_data_t *>(weights);
+    auto w = const_cast<char *>(weights);
     const int32_t *compensation = (jcp.signed_input)
             ? reinterpret_cast<int32_t *>(&w[offset])
             : nullptr;
@@ -143,7 +152,8 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_2d(
                 const int32_t *compensation_w
                         = (jcp.signed_input) ? compensation + g_oc : nullptr;
 
-                auto dst_w = dst + dst_d.blk_off(n, g_oc, oh_s, ow_s);
+                auto dst_w = dst
+                        + dst_dt_size * dst_d.blk_off(n, g_oc, oh_s, ow_s);
                 auto src_w = src + src_d.blk_off(n, g_ic, ih_s, iw_s);
                 auto wht_w = weights + wht_blk_off(weights_d, g, ocb, 0);
 
@@ -152,10 +162,10 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_2d(
                 for (int oj = oh_s, ij = ih_s; oj < oh_e;
                         ++oj, ij += jcp.stride_h) {
                     int dilate_h = jcp.dilate_h + 1;
-                    int i_t_overflow
-                            = nstl::min(jcp.kh, div_up(max(0, -ij), dilate_h));
+                    int i_t_overflow = nstl::min(
+                            jcp.kh, div_up(nstl::max(0, -ij), dilate_h));
                     int i_b_overflow = nstl::min(jcp.kh,
-                            div_up(max(0,
+                            div_up(nstl::max(0,
                                            ij - jcp.ih + (jcp.kh - 1) * dilate_h
                                                    + 1),
                                     dilate_h));
@@ -181,18 +191,18 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_2d(
                     p.oc_blocks = ocb;
                     p.kh_padding = kh_padding;
                     p.scales = scales;
+                    p.dst_scale = dst_scales;
                     p.t_overflow = i_t_overflow;
                     p.b_overflow = i_b_overflow;
                     p.owb = owb;
 
-                    p.oc_l_off = g_oc;
                     p.post_ops_binary_rhs_arg_vec
                             = post_ops_binary_rhs_arg_vec.data();
                     p.dst_orig = dst;
 
                     (*kernel_)(&p);
                     src_w += src_h_stride * jcp.stride_h;
-                    dst_w += dst_h_stride;
+                    dst_w += dst_dt_size * dst_h_stride;
                 }
             }
             switch (jcp.loop_order) {
@@ -216,15 +226,14 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_2d(
     return status::success;
 }
 
-template <cpu_isa_t isa, data_type_t src_type, data_type_t dst_type>
-status_t
-jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_1d(
+template <cpu_isa_t isa>
+status_t jit_uni_x8s8s32x_convolution_fwd_t<isa>::execute_forward_1d(
         const exec_ctx_t &ctx) const {
     const auto &jcp = pd()->jcp_;
-    auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
-    auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
+    auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
+    auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
-    auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    auto dst = CTX_OUT_MEM(char *, DNNL_ARG_DST);
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(pd()->jcp_.post_ops, ctx);
 
@@ -236,33 +245,25 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_1d(
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
 
-    const size_t bia_dt_size = pd()->with_bias()
-            ? types::data_type_size(pd()->desc()->bias_desc.data_type)
-            : 0;
+    const size_t bia_dt_size
+            = pd()->with_bias() ? types::data_type_size(bias_d.data_type()) : 0;
+    const size_t dst_dt_size = types::data_type_size(dst_d.data_type());
 
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
     assert(jcp.nb_ch % jcp.nb_ch_blocking == 0);
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        auto local_scales = ctx.get_scratchpad_grantor().template get<float>(
-                key_conv_adjusted_scales);
-        size_t count = pd()->attr()->output_scales_.count_;
-        float factor = 1.f / pd()->jcp_.wei_adj_scale;
-        if (count == 1) {
-            utils::array_set(local_scales, oscales[0] * factor, 8);
-        } else {
-            for (size_t c = 0; c < count; c++)
-                local_scales[c] = oscales[c] * factor;
-        }
-        oscales = local_scales;
-    }
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = adjust_oscales(
+            ctx.get_scratchpad_grantor(), src_scales, wei_scales);
 
     size_t extra_data_offset
             = weights_d.size() - weights_d.additional_buffer_size();
     size_t ch_offset = jcp.is_depthwise ? jcp.nb_ch * jcp.ch_block
                                         : jcp.ngroups * jcp.oc;
-    auto w = const_cast<wei_data_t *>(weights);
+    auto w = const_cast<char *>(weights);
     const int32_t *compensation = (jcp.signed_input)
             ? reinterpret_cast<int32_t *>(&w[extra_data_offset])
             : nullptr;
@@ -270,6 +271,7 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_1d(
             ? reinterpret_cast<int32_t *>(&w[extra_data_offset])
                     + (jcp.signed_input ? ch_offset : 0)
             : nullptr;
+
     int oc_chunks = jcp.nb_oc / jcp.nb_oc_blocking;
     int nb_groups = jcp.nb_ch / jcp.nb_ch_blocking;
     int group_block = jcp.ch_block;
@@ -316,17 +318,17 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_1d(
                     = jcp.src_zero_point ? zp_compensation + g_oc : nullptr;
             p.src_zero_point = jcp.src_zero_point ? src_zero_point : nullptr;
             p.dst_zero_point = jcp.dst_zero_point ? dst_zero_point : nullptr;
-            p.dst = dst + dst_d.blk_off(n, g_oc, ow_s);
+            p.dst = dst + dst_dt_size * dst_d.blk_off(n, g_oc, ow_s);
             p.src = src + src_d.blk_off(n, g_ic, iw_s);
             p.filt = weights + wht_blk_off(weights_d, gb, ocb, 0);
             p.scales = &oscales[jcp.is_oc_scale * g_oc];
+            p.dst_scale = dst_scales;
             p.oc_blocks = jcp.is_depthwise ? gb : ocb;
             p.kh_padding = jcp.kh;
             p.t_overflow = 0;
             p.b_overflow = 0;
             p.owb = owb;
 
-            p.oc_l_off = g_oc;
             p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec.data();
             p.dst_orig = dst;
 
@@ -357,14 +359,14 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_1d(
     return status::success;
 }
 
-template <cpu_isa_t isa, data_type_t src_type, data_type_t dst_type>
-status_t jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type,
-        dst_type>::execute_forward_2d_dw(const exec_ctx_t &ctx) const {
+template <cpu_isa_t isa>
+status_t jit_uni_x8s8s32x_convolution_fwd_t<isa>::execute_forward_2d_dw(
+        const exec_ctx_t &ctx) const {
     const auto &jcp = pd()->jcp_;
-    auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
-    auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
+    auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
+    auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
-    auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    auto dst = CTX_OUT_MEM(char *, DNNL_ARG_DST);
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(pd()->jcp_.post_ops, ctx);
 
@@ -376,9 +378,9 @@ status_t jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type,
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
 
-    const size_t bia_dt_size = pd()->with_bias()
-            ? types::data_type_size(pd()->desc()->bias_desc.data_type)
-            : 0;
+    const size_t bia_dt_size
+            = pd()->with_bias() ? types::data_type_size(bias_d.data_type()) : 0;
+    const size_t dst_dt_size = types::data_type_size(dst_d.data_type());
 
     assert(jcp.ic_block == 1);
     assert(jcp.oc_block == 1);
@@ -387,23 +389,15 @@ status_t jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type,
     assert(jcp.nb_oc_blocking == 1);
     assert(jcp.nb_ch % jcp.nb_ch_blocking == 0);
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        auto local_scales = ctx.get_scratchpad_grantor().template get<float>(
-                key_conv_adjusted_scales);
-        size_t count = pd()->attr()->output_scales_.count_;
-        float factor = 1.f / pd()->jcp_.wei_adj_scale;
-        if (count == 1) {
-            utils::array_set(local_scales, oscales[0] * factor, 8);
-        } else {
-            for (size_t c = 0; c < count; c++)
-                local_scales[c] = oscales[c] * factor;
-        }
-        oscales = local_scales;
-    }
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = adjust_oscales(
+            ctx.get_scratchpad_grantor(), src_scales, wei_scales);
 
     size_t offset = weights_d.size() - weights_d.additional_buffer_size();
-    auto w = const_cast<wei_data_t *>(weights);
+    auto w = const_cast<char *>(weights);
     const int32_t *compensation = (jcp.signed_input)
             ? reinterpret_cast<int32_t *>(&w[offset])
             : nullptr;
@@ -415,7 +409,7 @@ status_t jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type,
     int group_block = jcp.ch_block;
 
     parallel_nd(jcp.mb, jcp.oh, jcp.nb_ow, nb_groups,
-            [&](int n, int oh_s, int owb, int gg) {
+            [&](dim_t n, dim_t oh_s, dim_t owb, dim_t gg) {
                 auto p = jit_conv_call_s();
 
                 size_t src_h_stride = src_d.blk_off(0, 0, 1);
@@ -433,17 +427,18 @@ status_t jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type,
                 const int32_t *compensation_w
                         = jcp.signed_input ? compensation + g : nullptr;
 
-                auto dst_w = dst + dst_d.blk_off(n, g, oh_s, ow_s);
+                auto dst_w
+                        = dst + dst_dt_size * dst_d.blk_off(n, g, oh_s, ow_s);
                 auto src_w = src + src_d.blk_off(n, g, ih_s, iw_s);
                 auto wht_w = weights + wht_blk_off(weights_d, gb, 0);
 
                 auto scales = &oscales[jcp.is_oc_scale * g];
 
                 int dilate_h = jcp.dilate_h + 1;
-                int i_t_overflow
-                        = nstl::min(jcp.kh, div_up(max(0, -ih_s), dilate_h));
+                int i_t_overflow = nstl::min(
+                        jcp.kh, div_up(nstl::max(0, -ih_s), dilate_h));
                 int i_b_overflow = nstl::min(jcp.kh,
-                        div_up(max(0,
+                        div_up(nstl::max(0,
                                        ih_s - jcp.ih + (jcp.kh - 1) * dilate_h
                                                + 1),
                                 dilate_h));
@@ -467,11 +462,11 @@ status_t jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type,
                 p.oc_blocks = gb;
                 p.kh_padding = kh_padding;
                 p.scales = scales;
+                p.dst_scale = dst_scales;
                 p.t_overflow = i_t_overflow;
                 p.b_overflow = i_b_overflow;
                 p.owb = owb;
 
-                p.oc_l_off = g;
                 p.post_ops_binary_rhs_arg_vec
                         = post_ops_binary_rhs_arg_vec.data();
                 p.dst_orig = dst;
@@ -481,15 +476,14 @@ status_t jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type,
     return status::success;
 }
 
-template <cpu_isa_t isa, data_type_t src_type, data_type_t dst_type>
-status_t
-jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_3d(
+template <cpu_isa_t isa>
+status_t jit_uni_x8s8s32x_convolution_fwd_t<isa>::execute_forward_3d(
         const exec_ctx_t &ctx) const {
     const auto &jcp = pd()->jcp_;
-    auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
-    auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
+    auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
+    auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
-    auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    auto dst = CTX_OUT_MEM(char *, DNNL_ARG_DST);
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(pd()->jcp_.post_ops, ctx);
 
@@ -501,32 +495,24 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_3d(
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
 
-    const size_t bia_dt_size = pd()->with_bias()
-            ? types::data_type_size(pd()->desc()->bias_desc.data_type)
-            : 0;
+    const size_t bia_dt_size
+            = pd()->with_bias() ? types::data_type_size(bias_d.data_type()) : 0;
+    const size_t dst_dt_size = types::data_type_size(dst_d.data_type());
 
     assert(jcp.ch_block == 1);
     assert(jcp.nb_ch_blocking == 1);
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
     assert(jcp.nb_ch % jcp.nb_ch_blocking == 0);
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        auto local_scales = ctx.get_scratchpad_grantor().template get<float>(
-                key_conv_adjusted_scales);
-        size_t count = pd()->attr()->output_scales_.count_;
-        float factor = 1.f / pd()->jcp_.wei_adj_scale;
-        if (count == 1) {
-            utils::array_set(local_scales, oscales[0] * factor, 8);
-        } else {
-            for (size_t c = 0; c < count; c++)
-                local_scales[c] = oscales[c] * factor;
-        }
-        oscales = local_scales;
-    }
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = adjust_oscales(
+            ctx.get_scratchpad_grantor(), src_scales, wei_scales);
 
     size_t offset = weights_d.size() - weights_d.additional_buffer_size();
-    auto w = const_cast<wei_data_t *>(weights);
+    auto w = const_cast<char *>(weights);
     const int32_t *compensation = (jcp.signed_input)
             ? reinterpret_cast<int32_t *>(&w[offset])
             : nullptr;
@@ -584,10 +570,10 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_3d(
                 int iw_s = ow_s * jcp.stride_w;
                 int id_s = -jcp.f_pad + od_s * jcp.stride_d;
                 int dilate_d = jcp.dilate_d + 1;
-                int d_f_overflow
-                        = nstl::min(jcp.kd, div_up(max(0, -id_s), dilate_d));
+                int d_f_overflow = nstl::min(
+                        jcp.kd, div_up(nstl::max(0, -id_s), dilate_d));
                 int d_back_overflow = nstl::min(jcp.kd,
-                        div_up(max(0,
+                        div_up(nstl::max(0,
                                        id_s - jcp.id + (jcp.kd - 1) * dilate_d
                                                + 1),
                                 dilate_d));
@@ -606,7 +592,9 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_3d(
                 p.dst_zero_point
                         = jcp.dst_zero_point ? dst_zero_point : nullptr;
 
-                auto dst_w = dst + dst_d.blk_off(n, g_oc, od_s, oh_s, ow_s);
+                auto dst_w = dst
+                        + dst_dt_size
+                                * dst_d.blk_off(n, g_oc, od_s, oh_s, ow_s);
                 auto src_w = src + src_d.blk_off(n, g_ic, id_s, ih_s, iw_s)
                         + d_f_overflow * dilate_d * src_d_stride;
                 auto wht_w = weights + wht_blk_off(weights_d, g, ocb, 0)
@@ -620,10 +608,10 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_3d(
                 for (int oj = oh_s, ij = ih_s; oj < oh_e;
                         ++oj, ij += jcp.stride_h) {
                     int dilate_h = jcp.dilate_h + 1;
-                    int i_t_overflow
-                            = nstl::min(jcp.kh, div_up(max(0, -ij), dilate_h));
+                    int i_t_overflow = nstl::min(
+                            jcp.kh, div_up(nstl::max(0, -ij), dilate_h));
                     int i_b_overflow = nstl::min(jcp.kh,
-                            div_up(max(0,
+                            div_up(nstl::max(0,
                                            ij - jcp.ih + (jcp.kh - 1) * dilate_h
                                                    + 1),
                                     dilate_h));
@@ -642,20 +630,20 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_3d(
                     p.kh_padding = kh_padding;
                     p.kd_padding = kd_padding;
                     p.scales = scales;
+                    p.dst_scale = dst_scales;
                     p.t_overflow = i_t_overflow;
                     p.b_overflow = i_b_overflow;
                     p.f_overflow = d_f_overflow;
                     p.back_overflow = d_back_overflow;
                     p.owb = owb;
 
-                    p.oc_l_off = g_oc;
                     p.post_ops_binary_rhs_arg_vec
                             = post_ops_binary_rhs_arg_vec.data();
                     p.dst_orig = dst;
 
                     (*kernel_)(&p);
                     src_w += src_h_stride * jcp.stride_h;
-                    dst_w += dst_h_stride;
+                    dst_w += dst_dt_size * dst_h_stride;
                 }
             }
             switch (jcp.loop_order) {
@@ -681,22 +669,8 @@ jit_uni_x8s8s32x_convolution_fwd_t<isa, src_type, dst_type>::execute_forward_3d(
     return status::success;
 }
 
-template struct jit_uni_x8s8s32x_convolution_fwd_t<sse41, s8, u8>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<sse41, u8, u8>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<sse41, s8, s8>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<sse41, u8, s8>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<sse41, s8, s32>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<sse41, u8, s32>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<sse41, s8, f32>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<sse41, u8, f32>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<avx2, s8, u8>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<avx2, u8, u8>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<avx2, s8, s8>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<avx2, u8, s8>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<avx2, s8, s32>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<avx2, u8, s32>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<avx2, s8, f32>;
-template struct jit_uni_x8s8s32x_convolution_fwd_t<avx2, u8, f32>;
+template struct jit_uni_x8s8s32x_convolution_fwd_t<sse41>;
+template struct jit_uni_x8s8s32x_convolution_fwd_t<avx2>;
 
 } // namespace x64
 } // namespace cpu
