@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -121,10 +121,10 @@ status_t jit_uni_lrn_fwd_t<isa, d_type>::execute_forward(
     const auto ker_last = ker_last_.get();
 
     if (dat_tag == nChw8c && ls == 5 && ak == lrn_across_channels) {
-        parallel_nd(N, C / VECTOR_LENGTH, [&](int n, int c8) {
+        parallel_nd(N, C / VECTOR_LENGTH, [&](dim_t n, dim_t c8) {
             const auto offset = n * HW * C + c8 * HW * VECTOR_LENGTH;
-            jit_args_fwd_t args {
-                    &src[offset], &dst[offset], &ws[offset], nullptr};
+            auto ws_ptr = ws ? &ws[offset] : nullptr;
+            jit_args_fwd_t args {&src[offset], &dst[offset], ws_ptr, nullptr};
             if (c8 == 0)
                 (*ker_first)(&args);
             else if (c8 == C / VECTOR_LENGTH - 1)
@@ -134,20 +134,22 @@ status_t jit_uni_lrn_fwd_t<isa, d_type>::execute_forward(
         });
     } else if (one_of(dat_tag, nhwc, nChw8c, nChw16c)
             && ak == lrn_within_channel) {
-        parallel_nd(N, C / VECTOR_LENGTH, [&](int n, int c) {
+        parallel_nd(N, C / VECTOR_LENGTH, [&](dim_t n, dim_t c) {
             const std::size_t offset = dat_tag == nhwc
                     ? n * HW * C + c * VECTOR_LENGTH
                     : n * HW * C + c * HW * VECTOR_LENGTH;
-            jit_args_fwd_t args {&src[offset], &dst[offset], &ws[offset],
-                    &ws[offset + N * C * HW]};
+            auto ws0_ptr = ws ? &ws[offset] : nullptr;
+            auto ws1_ptr = ws ? &ws[offset + N * C * HW] : nullptr;
+            jit_args_fwd_t args {&src[offset], &dst[offset], ws0_ptr, ws1_ptr};
             (*ker)(&args);
         });
     } else if (dat_tag == nchw && ls == 5 && ak == lrn_across_channels) {
         parallel_nd(N, (HW + VECTOR_LENGTH - 1) / VECTOR_LENGTH,
-                [&](int n, int hw8) {
+                [&](dim_t n, dim_t hw8) {
                     const auto offset = n * HW * C + hw8 * VECTOR_LENGTH;
+                    auto ws0_ptr = ws ? &ws[offset] : nullptr;
                     jit_args_fwd_t args {
-                            &src[offset], &dst[offset], &ws[offset], nullptr};
+                            &src[offset], &dst[offset], ws0_ptr, nullptr};
 
                     if ((hw8 + 1) * VECTOR_LENGTH > HW)
                         (*ker_last)(&args);
@@ -155,10 +157,10 @@ status_t jit_uni_lrn_fwd_t<isa, d_type>::execute_forward(
                         (*ker)(&args);
                 });
     } else { // nhwc
-        parallel_nd(N, HW, [&](int n, int hw) {
+        parallel_nd(N, HW, [&](dim_t n, dim_t hw) {
             const auto offset = n * HW * C + hw * C;
-            jit_args_fwd_t args {
-                    &src[offset], &dst[offset], &ws[offset], nullptr};
+            auto ws_ptr = ws ? &ws[offset] : nullptr;
+            jit_args_fwd_t args {&src[offset], &dst[offset], ws_ptr, nullptr};
             (*ker)(&args);
         });
     }
@@ -170,45 +172,60 @@ status_t jit_uni_lrn_fwd_t<isa, d_type>::pd_t::init(engine_t *engine) {
     using namespace prop_kind;
     using namespace alg_kind;
 
-    const memory_desc_wrapper data_d(src_md());
-    const bool ok = true && mayiuse(isa) && is_fwd()
-            && everyone_is(d_type, data_d.data_type()) && !has_zero_dim_memory()
-            && data_d.ndims() == 4 && data_d.dims()[1] % VECTOR_LENGTH == 0
-            && data_d.dims()[1] >= 2 * VECTOR_LENGTH && desc()->lrn_beta == 0.75
-            && attr()->has_default_values();
-    if (!ok) return unimplemented;
+    const memory_desc_wrapper src_d(src_md());
+    const memory_desc_wrapper dst_d(dst_md());
+
+    VDISPATCH_LRN(is_fwd(), VERBOSE_BAD_PROPKIND);
+
+    // disabling verbose dispatch checks for unsupported isa for better readability
+    if (!mayiuse(isa)) return status::unimplemented;
+
+    VDISPATCH_LRN(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+    VDISPATCH_LRN(everyone_is(d_type, src_d.data_type(), dst_d.data_type()),
+            VERBOSE_UNSUPPORTED_DT);
+    VDISPATCH_LRN(attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+    VDISPATCH_LRN(set_default_formats_common(), VERBOSE_UNSUPPORTED_TAG);
+    VDISPATCH_LRN(src_d == dst_d, VERBOSE_INCONSISTENT_MDS, "src", "dst");
+    VDISPATCH_LRN(src_d.ndims() == 4, VERBOSE_BAD_NDIMS, "src", src_d.ndims());
+    VDISPATCH_LRN(src_d.dims()[1] % VECTOR_LENGTH == 0
+                    && src_d.dims()[1] >= 2 * VECTOR_LENGTH,
+            "src has inconsistent dimensions with vector length");
+    VDISPATCH_LRN(desc()->lrn_beta == 0.75, VERBOSE_BAD_PARAM, "lrn_beta");
 
     dat_tag_ = memory_desc_matches_one_of_tag(
             *src_md(), nChw16c, nChw8c, nchw, nhwc);
 
-    const int HW = data_d.dims()[2] * data_d.dims()[3];
+    const int HW = src_d.dims()[2] * src_d.dims()[3];
 
     const bool args_ok_across = true && desc()->alg_kind == lrn_across_channels
             && desc()->local_size == 5 && one_of(dat_tag_, nChw8c, nchw, nhwc)
-            && everyone_is(data_type::f32, data_d.data_type())
+            && everyone_is(data_type::f32, src_d.data_type())
             /* SSE41: prevent loads smaller than the size of xmm registers,
          * otherwise it will result in an illegal memory read (seg-fault)
          * due to protected memory. */
             && IMPLICATION(isa == sse41 && dat_tag_ == nchw, HW >= 4)
-            && isa != avx512_common;
+            && !is_superset(isa, avx512_core);
 
     const int jit_max_local_size = 5; // bigger size triggers too big code size
     const bool args_ok_within = true && desc()->alg_kind == lrn_within_channel
             && desc()->local_size <= (jit_max_local_size <= MAX_LOCAL_SIZE
                                ? jit_max_local_size
                                : MAX_LOCAL_SIZE)
-            && data_d.dims()[2] >= desc()->local_size
-            && data_d.dims()[3] >= desc()->local_size
-            && IMPLICATION(d_type == data_type::bf16, mayiuse(avx512_core))
-            && (isa == avx512_common ? one_of(dat_tag_, nhwc, nChw16c)
-                                     : one_of(dat_tag_, nhwc, nChw8c));
+            && src_d.dims()[2] >= desc()->local_size
+            && src_d.dims()[3] >= desc()->local_size
+            && IMPLICATION(d_type == data_type::bf16,
+                    mayiuse(avx512_core) || mayiuse(avx2_vnni_2))
+            && IMPLICATION(d_type == data_type::f16,
+                    mayiuse(avx512_core_fp16) || mayiuse(avx2_vnni_2))
+            && (is_superset(isa, avx512_core) ? one_of(dat_tag_, nhwc, nChw16c)
+                                              : one_of(dat_tag_, nhwc, nChw8c));
 
     const auto status
             = args_ok_across || args_ok_within ? success : unimplemented;
 
     if (desc()->prop_kind == forward_training && status == success) {
         dims_t ws_dims = {MB(), C(), H(), 2 * W()};
-        dnnl_memory_desc_init_by_tag(&ws_md_, 4, ws_dims, d_type, dat_tag_);
+        memory_desc_init_by_tag(ws_md_, 4, ws_dims, d_type, dat_tag_);
     }
 
     return status;
@@ -288,7 +305,7 @@ status_t jit_uni_lrn_bwd_t<isa, d_type>::execute_backward(
 
     if (one_of(dat_tag, nhwc, nChw8c, nChw16c)
             && ak == alg_kind::lrn_within_channel) {
-        parallel_nd(N, C / VECTOR_LENGTH, [&](int n, int c) {
+        parallel_nd(N, C / VECTOR_LENGTH, [&](dim_t n, dim_t c) {
             const std::size_t offset = dat_tag == nhwc
                     ? n * H * W * C + c * VECTOR_LENGTH
                     : n * H * W * C + c * H * W * VECTOR_LENGTH;
@@ -297,7 +314,7 @@ status_t jit_uni_lrn_bwd_t<isa, d_type>::execute_backward(
             (*ker)(&args);
         });
     } else if (use_h_parallelism) {
-        parallel_nd(N, C / VECTOR_LENGTH, H, [&](int n, int c8, int h) {
+        parallel_nd(N, C / VECTOR_LENGTH, H, [&](dim_t n, dim_t c8, dim_t h) {
             const std::size_t offset = n * C * H * W
                     + c8 * H * W * VECTOR_LENGTH + h * W * VECTOR_LENGTH;
             jit_args_bwd_t args {&src[offset], &diff_dst[offset], &ws[offset],
@@ -312,7 +329,7 @@ status_t jit_uni_lrn_bwd_t<isa, d_type>::execute_backward(
                 (*ker)(&args);
         });
     } else {
-        parallel_nd(N, C / VECTOR_LENGTH, [&](int n, int c8) {
+        parallel_nd(N, C / VECTOR_LENGTH, [&](dim_t n, dim_t c8) {
             const std::size_t offset
                     = n * C * H * W + c8 * H * W * VECTOR_LENGTH;
             jit_args_bwd_t args {&src[offset], &diff_dst[offset], &ws[offset],
@@ -335,49 +352,69 @@ status_t jit_uni_lrn_bwd_t<isa, d_type>::pd_t::init(engine_t *engine) {
     using namespace prop_kind;
     using namespace alg_kind;
 
-    const memory_desc_wrapper data_d(src_md());
-    const bool ok = true && mayiuse(isa) && !is_fwd()
-            && utils::everyone_is(d_type, data_d.data_type())
-            && set_default_formats_common() && !has_zero_dim_memory()
-            && data_d.ndims() == 4 && data_d.dims()[1] % VECTOR_LENGTH == 0
-            && data_d.dims()[1] >= 2 * VECTOR_LENGTH && desc()->lrn_beta == 0.75
-            && attr()->has_default_values();
-    if (!ok) return unimplemented;
+    const memory_desc_wrapper src_d(src_md());
+    const memory_desc_wrapper diff_src_d(diff_src_md());
+    const memory_desc_wrapper diff_dst_d(diff_dst_md());
+
+    VDISPATCH_LRN(!is_fwd(), VERBOSE_BAD_PROPKIND);
+
+    // disabling verbose dispatch checks for unsupported isa for better readability
+    if (!mayiuse(avx512_core)) return status::unimplemented;
+
+    VDISPATCH_LRN(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+    VDISPATCH_LRN(utils::everyone_is(d_type, src_d.data_type(),
+                          diff_src_d.data_type(), diff_dst_d.data_type()),
+            VERBOSE_UNSUPPORTED_DT);
+    VDISPATCH_LRN(src_d.ndims() == 4, VERBOSE_BAD_NDIMS, "src", src_d.ndims());
+    VDISPATCH_LRN(attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+    VDISPATCH_LRN(set_default_formats_common(), VERBOSE_UNSUPPORTED_TAG);
+    VDISPATCH_LRN(src_d == diff_dst_d, VERBOSE_INCONSISTENT_MDS, "src", "dst");
+    VDISPATCH_LRN(diff_dst_d == diff_src_d, VERBOSE_INCONSISTENT_MDS,
+            "diff_src", "diff_dst");
+    VDISPATCH_LRN((src_d.dims()[1] % VECTOR_LENGTH == 0
+                          && src_d.dims()[1] >= 2 * VECTOR_LENGTH),
+            "src has inconsistent dimensions with vector length");
+    VDISPATCH_LRN(desc()->lrn_beta == 0.75, VERBOSE_BAD_PARAM, "lrn_beta");
 
     dat_tag_ = memory_desc_matches_one_of_tag(
             *src_md(), nChw16c, nChw8c, nchw, nhwc);
 
     const dims_t ws_dims = {MB(), C(), H(), 2 * W()};
-    dnnl_memory_desc_init_by_tag(&ws_md_, 4, ws_dims, d_type, dat_tag_);
+    memory_desc_init_by_tag(ws_md_, 4, ws_dims, d_type, dat_tag_);
 
-    if (!compare_ws(hint_fwd_pd_)) return unimplemented;
+    VDISPATCH_LRN(compare_ws(hint_fwd_pd_), VERBOSE_WS_MISMATCH);
 
     const bool args_ok_across = true && desc()->alg_kind == lrn_across_channels
             && desc()->local_size == 5 && utils::one_of(dat_tag_, nChw8c)
-            && everyone_is(data_type::f32, data_d.data_type())
-            && isa != avx512_common;
+            && everyone_is(data_type::f32, src_d.data_type())
+            && !is_superset(isa, avx512_core);
 
     const int jit_max_local_size = 5; // bigger size triggers too big code size
     const bool args_ok_within = true && desc()->alg_kind == lrn_within_channel
             && desc()->local_size <= (jit_max_local_size <= MAX_LOCAL_SIZE
                                ? jit_max_local_size
                                : MAX_LOCAL_SIZE)
-            && data_d.dims()[2] >= desc()->local_size
-            && data_d.dims()[3] >= desc()->local_size
+            && src_d.dims()[2] >= desc()->local_size
+            && src_d.dims()[3] >= desc()->local_size
             && IMPLICATION(d_type == data_type::bf16, mayiuse(avx512_core))
-            && (isa == avx512_common ? one_of(dat_tag_, nhwc, nChw16c)
-                                     : one_of(dat_tag_, nhwc, nChw8c));
+            && IMPLICATION(d_type == data_type::f16, mayiuse(avx512_core_fp16))
+            && (isa == avx512_core ? one_of(dat_tag_, nhwc, nChw16c)
+                                   : one_of(dat_tag_, nhwc, nChw8c));
 
     return args_ok_across || args_ok_within ? success : unimplemented;
 }
 
-template struct jit_uni_lrn_fwd_t<avx512_common, dnnl::impl::data_type::f32>;
-template struct jit_uni_lrn_fwd_t<avx512_common, dnnl::impl::data_type::bf16>;
-template struct jit_uni_lrn_fwd_t<avx2, dnnl::impl::data_type::f32>;
-template struct jit_uni_lrn_fwd_t<sse41, dnnl::impl::data_type::f32>;
-template struct jit_uni_lrn_bwd_t<avx512_common, dnnl::impl::data_type::f32>;
-template struct jit_uni_lrn_bwd_t<avx512_common, dnnl::impl::data_type::bf16>;
-template struct jit_uni_lrn_bwd_t<avx2, dnnl::impl::data_type::f32>;
+template struct jit_uni_lrn_fwd_t<avx512_core, data_type::f32>;
+template struct jit_uni_lrn_fwd_t<avx512_core, data_type::bf16>;
+template struct jit_uni_lrn_fwd_t<avx512_core_fp16, data_type::f16>;
+template struct jit_uni_lrn_fwd_t<avx2_vnni_2, data_type::bf16>;
+template struct jit_uni_lrn_fwd_t<avx2_vnni_2, data_type::f16>;
+template struct jit_uni_lrn_fwd_t<avx2, data_type::f32>;
+template struct jit_uni_lrn_fwd_t<sse41, data_type::f32>;
+template struct jit_uni_lrn_bwd_t<avx512_core, data_type::f32>;
+template struct jit_uni_lrn_bwd_t<avx512_core, data_type::bf16>;
+template struct jit_uni_lrn_bwd_t<avx512_core_fp16, data_type::f16>;
+template struct jit_uni_lrn_bwd_t<avx2, data_type::f32>;
 
 } // namespace x64
 } // namespace cpu
