@@ -452,9 +452,7 @@ public:
                 dims[i] *= ir_utils::safe_divide(padded_dim(d), tg_block);
             }
         }
-        auto &tg_idxs = ir_builder_t::tg_idxs();
-        set_kernel_grid(grid_info_t(
-                dims, std::vector<expr_t>(tg_idxs.begin(), tg_idxs.end())));
+        set_kernel_grid(grid_info_t(dims, ir_builder_t::tg_idx));
     }
 
     void init_thread_group_grid(const std::array<pvar_tile_t, 3> &grid) {
@@ -463,7 +461,7 @@ public:
             for (auto &d : grid[i])
                 dims[i] *= thread_group_dim(d);
         }
-        set_thread_group_grid(grid_info_t(dims, "thr_idx"));
+        set_thread_group_grid(grid_info_t(dims, ir_builder_t::thr_idx));
     }
 
 protected:
