@@ -855,7 +855,7 @@ void kernel_desc_t::show_help() {
 }
 
 grid_t create_thread_group_grid(const kernel_desc_t &desc) {
-    grid_t grid(jit::ir_builder_t::tg_idxs());
+    grid_t grid(jit::ir_builder_t::tg_idx);
     auto set = [&](const pvar_t dim, int idx) {
         grid.add_mapping(dim, desc.use_stream_k ? 0 : idx);
     };
@@ -890,7 +890,7 @@ grid_t create_thread_group_grid(const kernel_desc_t &desc) {
 }
 
 grid_t create_thread_grid(const kernel_desc_t &desc) {
-    grid_t grid("thr_idx");
+    grid_t grid(jit::ir_builder_t::thr_idx);
     switch (desc.prop) {
         case prop_kind::forward:
             grid.add_mapping(pvars::oc, 0);
