@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 * Copyright 2020 Codeplay Software Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -23,7 +23,7 @@
 
 #include "common/type_helpers.hpp"
 #include "gpu/nvidia/cudnn_inner_product_impl.hpp"
-#include "gpu/nvidia/sycl_cuda_engine.hpp"
+#include "gpu/nvidia/engine.hpp"
 #include "gpu/nvidia/sycl_cuda_scoped_context.hpp"
 #include "gpu/nvidia/sycl_cuda_utils.hpp"
 
@@ -55,6 +55,9 @@ protected:
         switch (cudnn_dt) {
             case CUDNN_DATA_FLOAT: blas_dt = CUDA_R_32F; return status::success;
             case CUDNN_DATA_HALF: blas_dt = CUDA_R_16F; return status::success;
+            case CUDNN_DATA_BFLOAT16:
+                blas_dt = CUDA_R_16BF;
+                return status::success;
             case CUDNN_DATA_INT8: blas_dt = CUDA_R_8I; return status::success;
             case CUDNN_DATA_INT8x4: blas_dt = CUDA_R_8I; return status::success;
             default: return status::unimplemented;
@@ -73,11 +76,9 @@ struct cudnn_gemm_inner_product_fwd_impl_t
     cudnnTensorDescriptor_t y_acc_desc_;
     bool need_reorder_;
 
-    bool ip_using_scratchpad() const override { return (use_acc_dst_ > 0); }
-    virtual bool need_to_transform_filter() const { return need_reorder_; }
-
-    virtual status_t init(engine_t *, inner_product_pd_t *pd, bool with_relu,
-            bool with_eltwise, bool with_sum, bool need_reorder) override {
+    virtual status_t init(impl::engine_t *, inner_product_pd_t *pd,
+            bool with_relu, bool with_eltwise, bool with_sum,
+            bool need_reorder) override {
         need_reorder_ = need_reorder;
         // GEMM is column major, here the data is row major.
         // By switching the weight and source we convert the row major to
@@ -121,8 +122,6 @@ struct cudnn_gemm_inner_product_fwd_impl_t
                 || (with_bias_
                         && pd->weights_md(1)->data_type
                                 != pd->dst_md()->data_type));
-        // this must be applied on bias if exists.
-        output_scales_ = pd->attr()->output_scales_.scales_[0]; // alpha
         with_sum_ = with_sum;
         // scaling factor to add the previous destination value to the current
         // computation. This is equivalent of
@@ -165,7 +164,8 @@ struct cudnn_gemm_inner_product_fwd_impl_t
         if (use_acc_dst_) {
             pd->scratchpad_registry().registrar().book(
                     memory_tracking::names::key_iprod_int_dat_in_acc_dt,
-                    memory_desc_wrapper(pd->dst_md()).size(), size_t(1));
+                    memory_desc_wrapper(pd->dst_md()).nelems(),
+                    types::data_type_size(pd->desc()->accum_data_type));
             CHECK(create_and_set_tensor_descriptor(&y_acc_desc_,
                     CUDNN_DATA_FLOAT, ndims_, dims_[io::dst],
                     strides_[io::dst]));
@@ -178,9 +178,10 @@ struct cudnn_gemm_inner_product_fwd_impl_t
 
     void execute(cudnnHandle_t cudnn_handle, cublasHandle_t cublas_handle,
             const std::vector<void *> &args) const override {
-        assert(args.size() == 7);
+        assert(args.size() == 9);
         auto x = args[0], w = args[1], b = args[2], y = args[3],
-             workspace = args[4];
+             workspace = args[4], src_scale = args[6], wei_scale = args[7],
+             dst_scale = args[8];
         auto w_arg = w;
         if (need_reorder_) {
             void *transformed_w = args[5];
@@ -188,25 +189,60 @@ struct cudnn_gemm_inner_product_fwd_impl_t
             w_arg = transformed_w;
         }
         auto y_dst = use_acc_dst_ ? workspace : y;
-        auto sum_scale = use_acc_dst_ ? 0.0f : sum_scale_;
+
+        float scale = 1.0f;
+        if (src_scale || wei_scale) {
+            if (src_scale) {
+                float host_src_scale = 1.0f;
+                CUDA_EXECUTE_FUNC(cuMemcpy, (CUdeviceptr)&host_src_scale,
+                        (CUdeviceptr)src_scale, sizeof(float));
+                scale *= host_src_scale;
+            }
+            if (wei_scale) {
+                float host_wei_scale = 1.0f;
+                CUDA_EXECUTE_FUNC(cuMemcpy, (CUdeviceptr)&host_wei_scale,
+                        (CUdeviceptr)wei_scale, sizeof(float));
+                scale *= host_wei_scale;
+            }
+        }
+
+        // If sum for int8 is requested, first, convert dst memory into acc
+        // memory into f32 and using gemm API add it.
+        if (with_sum_ && use_acc_dst_) {
+            float one = 1.0f;
+            float zero = 0.0f;
+            CUDNN_EXECUTE_FUNC(cudnnTransformTensor, cudnn_handle, &one,
+                    tensor_descs_[io::dst], y, &zero, y_acc_desc_, y_dst);
+        }
+
         // do gemm
         CUBLAS_EXECUTE_FUNC(cublasGemmEx, cublas_handle, trans_a_, trans_b_, m_,
-                n_, k_, &output_scales_, w_arg, a_type_, lda_, x, b_type_, ldb_,
-                &sum_scale, y_dst, c_type_, ldc_, compute_type_, algo_);
+                n_, k_, &scale, w_arg, a_type_, lda_, x, b_type_, ldb_,
+                &sum_scale_, y_dst, c_type_, ldc_, compute_type_, algo_);
 
         if (with_bias_) {
-
-            CUDNN_EXECUTE_FUNC(cudnnAddTensor, cudnn_handle, &output_scales_,
-                    tensor_descs_[io::bia], b, &alpha_, y_acc_desc_, y_dst);
-        }
-        if (use_acc_dst_) {
-            CUDNN_EXECUTE_FUNC(cudnnTransformTensor, cudnn_handle, &alpha_,
-                    y_acc_desc_, y_dst, &sum_scale_, tensor_descs_[io::dst], y);
+            float alpha = 1.0f;
+            CUDNN_EXECUTE_FUNC(cudnnAddTensor, cudnn_handle, &alpha,
+                    tensor_descs_[io::bia], b, &alpha, y_acc_desc_, y_dst);
         }
         if (with_eltwise_) {
             CUDNN_EXECUTE_FUNC(cudnnActivationForward, cudnn_handle, act_desc_,
-                    &alpha_, tensor_descs_[io::dst], y, &beta_,
-                    tensor_descs_[io::dst], y);
+                    &alpha_, y_acc_desc_, y_dst, &beta_, y_acc_desc_, y_dst);
+        }
+
+        if (dst_scale) {
+            float host_dst_scale = 1.0f;
+            CUDA_EXECUTE_FUNC(cuMemcpy, (CUdeviceptr)&host_dst_scale,
+                    (CUdeviceptr)dst_scale, sizeof(float));
+            float inv_scale = 1.0f / host_dst_scale;
+            CUDNN_EXECUTE_FUNC(cudnnScaleTensor, cudnn_handle, y_acc_desc_,
+                    y_dst, &inv_scale);
+        }
+        if (use_acc_dst_) {
+            float alpha = 1.0f;
+            float zero = 0.0f;
+            CUDNN_EXECUTE_FUNC(cudnnTransformTensor, cudnn_handle, &alpha,
+                    y_acc_desc_, y_dst, &zero, tensor_descs_[io::dst], y);
         }
     }
 
@@ -227,9 +263,6 @@ struct cudnn_gemm_inner_product_fwd_impl_t
             case alg_kind::eltwise_logistic:
                 act_mode = CUDNN_ACTIVATION_SIGMOID;
                 break;
-            case alg_kind::eltwise_bounded_relu:
-                act_mode = CUDNN_ACTIVATION_CLIPPED_RELU;
-                break;
             default: return status::unimplemented;
         }
         CHECK(CUDNN_EXECUTE_FUNC_S(cudnnSetActivationDescriptor, act_desc_,
@@ -246,9 +279,7 @@ struct cudnn_gemm_inner_product_bwd_data_impl_t
       public cudnn_conv_filter_adjustment_base_t {
     bool need_reorder_;
 
-    virtual bool need_to_transform_filter() const { return need_reorder_; }
-
-    virtual status_t init(engine_t *, inner_product_pd_t *pd,
+    virtual status_t init(impl::engine_t *, inner_product_pd_t *pd,
             bool /*with_relu*/, bool /*with_eltwise*/, bool /*with_sum */,
             bool need_reorder) override {
         need_reorder_ = need_reorder;
@@ -318,8 +349,6 @@ struct cudnn_gemm_inner_product_bwd_weights_impl_t
     bool wie_tr_;
     bool need_reorder_;
 
-    virtual bool need_to_transform_filter() const { return need_reorder_; }
-
     virtual ~cudnn_gemm_inner_product_bwd_weights_impl_t() {
         if (reduceTensorDesc_) {
             CUDNN_EXECUTE_FUNC_V(
@@ -334,7 +363,7 @@ struct cudnn_gemm_inner_product_bwd_weights_impl_t
                 CUDNN_REDUCE_TENSOR_NO_INDICES, CUDNN_32BIT_INDICES);
         return status::success;
     }
-    virtual status_t init(engine_t *engine, inner_product_pd_t *pd,
+    virtual status_t init(impl::engine_t *engine, inner_product_pd_t *pd,
             bool /*with_relu*/, bool /*with_eltwise*/, bool /*with_sum */,
             bool need_reorder) override {
         need_reorder_ = need_reorder;
@@ -404,12 +433,12 @@ struct cudnn_gemm_inner_product_bwd_weights_impl_t
                     strides_[io::bia]));
             CHECK(create_and_set_reduce_descriptor());
 
-            auto &sycl_engine = *utils::downcast<sycl_cuda_engine_t *>(engine);
-            stream_t *service_stream;
+            auto &sycl_engine = *utils::downcast<nvidia::engine_t *>(engine);
+            impl::stream_t *service_stream;
             CHECK(sycl_engine.get_service_stream(service_stream));
 
             auto cuda_stream
-                    = utils::downcast<sycl_cuda_stream_t *>(service_stream);
+                    = utils::downcast<nvidia::stream_t *>(service_stream);
             auto handle = cuda_stream->get_cudnn_handle();
 
             // get the required workspace size
