@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -24,18 +24,26 @@
 #include "common.hpp"
 #include "dnn_types.hpp"
 #include "dnnl_common.hpp"
-#include "dnnl_memory.hpp"
 
-#include "conv/conv_common.hpp"
+#include "conv/conv.hpp"
 
 namespace conv_dw_fusion {
 
 using desc_t = conv::desc_t;
 using prb_t = conv::prb_t;
 using alg_t = conv::alg_t;
-using dt_conf_t = conv::dt_conf_t;
+using cfg_t = conv::cfg_t;
 
-int doit(const prb_t *prb, res_t *res);
+int createit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+        const prb_t *prb, res_t *res);
+int check_cacheit(
+        std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+        const prb_t *prb, res_t *res);
+int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+        const prb_t *prb, res_t *res);
+void get_fused_conv_dst_dims(const int ndims,
+        const attr_t::post_ops_t::entry_t &po_entry, const dnnl_dims_t dims,
+        dnnl_dims_t fused_conv_dst_dims);
 
 } // namespace conv_dw_fusion
 
