@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2021 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,123 +14,56 @@
 * limitations under the License.
 *******************************************************************************/
 
-#include "tests/test_thread.hpp"
+#include "utils/parallel.hpp"
 
-#include "conv/conv_common.hpp"
+#include "conv/ref_conv.hpp"
 
 namespace conv {
 
-template <typename get_args_func>
-void exec_conv(get_args_func get_args, const prb_t *prb, dnnl_primitive_t c_ref,
-        dnn_mem_t &src_m, dnn_mem_t &wei_m, dnn_mem_t &bia_m,
-        dnn_mem_t &dst_m) {
-    const_dnnl_primitive_desc_t pd_ref;
-    dnnl_engine_t engine_ref;
+void compute_ref_direct_fwd(const prb_t *prb, const args_t &args) {
+    const dnn_mem_t &src_m = args.find(DNNL_ARG_SRC);
+    const dnn_mem_t &wei_m = args.find(DNNL_ARG_WEIGHTS);
+    const dnn_mem_t &bia_m = args.find(DNNL_ARG_BIAS);
+    const dnn_mem_t &dst_m = args.find(DNNL_ARG_DST);
+    const dnn_mem_t &src_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC);
+    const dnn_mem_t &wei_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS);
+    const dnn_mem_t &dst_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
+    const dnn_mem_t &src_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const dnn_mem_t &wei_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_WEIGHTS);
+    const dnn_mem_t &dst_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
 
-    SAFE_V(dnnl_primitive_get_primitive_desc(c_ref, &pd_ref));
-    SAFE_V(dnnl_primitive_desc_query(
-            pd_ref, dnnl_query_engine, 0, &engine_ref));
-    const auto &scratchpad_md = *dnnl_primitive_desc_query_md(
-            pd_ref, dnnl_query_exec_arg_md, DNNL_ARG_SCRATCHPAD);
+    const bool has_src_scale = !prb->attr.scales.get(DNNL_ARG_SRC).is_def();
+    const bool has_wei_scale = !prb->attr.scales.get(DNNL_ARG_WEIGHTS).is_def();
+    const bool has_dst_scale = !prb->attr.scales.get(DNNL_ARG_DST).is_def();
+    const int wei_scale_mask = prb->attr.scales.get_mask(
+            DNNL_ARG_WEIGHTS, dnnl_convolution, wei_m.ndims(), prb->has_groups);
+    const int src_scale_mask = prb->attr.scales.get_mask(
+            DNNL_ARG_SRC, dnnl_convolution, src_m.ndims(), prb->has_groups);
+    const int dst_scale_mask = prb->attr.scales.get_mask(
+            DNNL_ARG_DST, dnnl_convolution, dst_m.ndims(), prb->has_groups);
 
-    auto src_ref = dnn_mem_t::create_from_host_ptr(
-            src_m.md_, engine_ref, (void *)src_m);
-    auto wei_ref = dnn_mem_t::create_from_host_ptr(
-            wei_m.md_, engine_ref, (void *)wei_m);
-    dnn_mem_t bia_ref;
-    if (prb->dir & FLAG_BIA)
-        bia_ref = dnn_mem_t::create_from_host_ptr(
-                bia_m.md_, engine_ref, (void *)bia_m);
-    auto dst_ref = dnn_mem_t::create_from_host_ptr(
-            dst_m.md_, engine_ref, (void *)dst_m);
-    dnn_mem_t scratchpad(scratchpad_md, engine_ref);
+    assert(IMPLICATION(
+            has_src_scale, src_scales.nelems() == 1 || src_scale_mask == 3));
+    assert(IMPLICATION(
+            has_dst_scale, dst_scales.nelems() == 1 || dst_scale_mask == 2));
 
-    args_t args = get_args(prb, src_ref, wei_ref, bia_ref, dst_ref);
-    args.set(DNNL_ARG_SCRATCHPAD, scratchpad);
-    SAFE_V(execute_and_wait(c_ref, args));
-}
+    const bool has_src_zp = !prb->attr.zero_points.get(DNNL_ARG_SRC).is_def();
+    const bool has_wei_zp
+            = !prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).is_def();
+    const bool has_dst_zp = !prb->attr.zero_points.get(DNNL_ARG_DST).is_def();
+    const int src_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_SRC).policy);
+    const int wei_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).policy);
+    const int dst_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_DST).policy);
 
-args_t get_args_conv_fwd(const prb_t *prb, dnn_mem_t &src_ref,
-        dnn_mem_t &wei_ref, dnn_mem_t &bia_ref, dnn_mem_t &dst_ref) {
-
-    args_t args;
-    args.set(DNNL_ARG_SRC, src_ref);
-    args.set(DNNL_ARG_WEIGHTS, wei_ref);
-    if (prb->dir & FLAG_BIA) args.set(DNNL_ARG_BIAS, bia_ref);
-    args.set(DNNL_ARG_DST, dst_ref);
-    return args;
-}
-
-args_t get_args_conv_bwd_d(const prb_t *prb, dnn_mem_t &src_ref,
-        dnn_mem_t &wei_ref, dnn_mem_t &bia_ref, dnn_mem_t &dst_ref) {
-
-    args_t args;
-    args.set(DNNL_ARG_DIFF_SRC, src_ref);
-    args.set(DNNL_ARG_WEIGHTS, wei_ref);
-    args.set(DNNL_ARG_DIFF_DST, dst_ref);
-    return args;
-}
-
-args_t get_args_conv_bwd_w(const prb_t *prb, dnn_mem_t &src_ref,
-        dnn_mem_t &wei_ref, dnn_mem_t &bia_ref, dnn_mem_t &dst_ref) {
-
-    args_t args;
-    args.set(DNNL_ARG_SRC, src_ref);
-    args.set(DNNL_ARG_DIFF_WEIGHTS, wei_ref);
-    if (prb->dir & FLAG_BIA) args.set(DNNL_ARG_DIFF_BIAS, bia_ref);
-    args.set(DNNL_ARG_DIFF_DST, dst_ref);
-    return args;
-}
-
-void compute_ref_fwd(const prb_t *prb, dnnl_primitive_t c_ref, dnn_mem_t &src_m,
-        dnn_mem_t &wei_m, dnn_mem_t &bia_m,
-        const std::vector<dnn_mem_t> &binary_po, dnn_mem_t &dst_m) {
-    if (c_ref) {
-        exec_conv(get_args_conv_fwd, prb, c_ref, src_m, wei_m, bia_m, dst_m);
-        return;
-    }
-    if (prb->alg == WINO && prb->cfg[SRC].dt == dnnl_f32) {
-        compute_wino_ref_fwd(prb, src_m, wei_m, bia_m, dst_m);
-    } else {
-        compute_ref_direct_fwd(prb, src_m, wei_m, bia_m, binary_po, dst_m);
-    }
-}
-
-void compute_ref_bwd_d(const prb_t *prb, dnnl_primitive_t c_ref,
-        dnn_mem_t &diff_src_m, dnn_mem_t &wei_m, dnn_mem_t &bia_m,
-        const std::vector<dnn_mem_t> &binary_po, dnn_mem_t &diff_dst_m) {
-    if (c_ref) {
-        exec_conv(get_args_conv_bwd_d, prb, c_ref, diff_src_m, wei_m, bia_m,
-                diff_dst_m);
-        return;
-    }
-    if (prb->alg == WINO && prb->cfg[SRC].dt == dnnl_f32) {
-        compute_wino_ref_bwd_d(prb, diff_src_m, wei_m, bia_m, diff_dst_m);
-    } else {
-        compute_ref_direct_bwd_d(
-                prb, diff_src_m, wei_m, bia_m, binary_po, diff_dst_m);
-    }
-}
-
-void compute_ref_bwd_w(const prb_t *prb, dnnl_primitive_t c_ref,
-        dnn_mem_t &src_m, dnn_mem_t &diff_wei_m, dnn_mem_t &diff_bia_m,
-        dnn_mem_t &diff_dst_m) {
-    if (c_ref) {
-        exec_conv(get_args_conv_bwd_w, prb, c_ref, src_m, diff_wei_m,
-                diff_bia_m, diff_dst_m);
-        return;
-    }
-    if (prb->alg == WINO && prb->cfg[SRC].dt == dnnl_f32) {
-        compute_wino_ref_bwd_w(prb, src_m, diff_wei_m, diff_bia_m, diff_dst_m);
-    } else {
-        compute_ref_direct_bwd_w(
-                prb, src_m, diff_wei_m, diff_bia_m, diff_dst_m);
-    }
-}
-
-void compute_ref_direct_fwd(const prb_t *prb, dnn_mem_t &src_m,
-        dnn_mem_t &wei_m, dnn_mem_t &bia_m,
-        const std::vector<dnn_mem_t> &binary_po, dnn_mem_t &dst_m) {
     /* help compiler optimize the code */
     const int64_t MB = prb->mb, G = prb->g, OC = prb->oc, IC = prb->ic;
     const int64_t OCG = OC / G, ICG = IC / G;
@@ -143,6 +76,8 @@ void compute_ref_direct_fwd(const prb_t *prb, dnn_mem_t &src_m,
     const int64_t DH = prb->dh + 1;
     const int64_t DW = prb->dw + 1;
 
+    int wei_zp = (has_wei_zp && (wei_zp_mask == 0)) ? wei_zps.get_elem(0) : 0;
+
     auto ker = [&](float &d, int64_t g, int64_t mb, int64_t oc, int64_t od,
                        int64_t oh, int64_t ow) {
         const float *__restrict src_loc
@@ -163,18 +98,26 @@ void compute_ref_direct_fwd(const prb_t *prb, dnn_mem_t &src_m,
                     for (int64_t ic = 0; ic < ICG; ++ic) {
                         int64_t src_off = ((ic * ID + id) * IH + ih) * IW + iw;
                         int64_t wei_off = ((ic * KD + kd) * KH + kh) * KW + kw;
-                        float s = src_loc[src_off];
-                        maybe_zero_point(prb->attr, s, prb->src_zp,
-                                g * ICG + ic, DNNL_ARG_SRC);
-                        d += s * wei_loc[wei_off];
+                        float src_scale = 1.f;
+                        if (has_src_scale)
+                            src_scale = src_scales.get_elem(
+                                    src_scale_mask > 0 ? g * ICG + ic : 0);
+                        int src_zp = has_src_zp ? src_zps.get_elem(
+                                             src_zp_mask > 0 ? g * ICG + ic : 0)
+                                                : 0;
+                        const float s = src_loc[src_off];
+                        const float w = wei_loc[wei_off];
+                        const float d_tmp
+                                = ((s - src_zp) * src_scale) * (w - wei_zp);
+                        d += d_tmp;
                     }
                 }
             }
         }
     };
 
-    std::vector<int> v_bin_po_mask = prb->attr.post_ops.get_binary_po_masks();
-    dnnl::impl::parallel_nd(G, MB, OCG, OD, OH, OW,
+    auto v_po_masks = prb->attr.post_ops.get_po_masks();
+    benchdnn_parallel_nd(G, MB, OCG, OD, OH, OW,
             [&](int64_t g, int64_t mb, int64_t oc, int64_t od, int64_t oh,
                     int64_t ow) {
                 const size_t dst_off = dst_off_f(prb, mb, g, oc, od, oh, ow);
@@ -183,33 +126,76 @@ void compute_ref_direct_fwd(const prb_t *prb, dnn_mem_t &src_m,
                 float conv_res = 0;
                 ker(conv_res, g, mb, oc, od, oh, ow);
 
+                // apply scale as:
+                //    dst = src_scale * wei_scale * conv(src - zp_src, wei)
+                float wei_scale = 1.f, dst_scale = 1.f;
+                if (has_wei_scale)
+                    wei_scale = wei_scales.get_elem(
+                            wei_scale_mask > 0 ? g * OCG + oc : 0);
+                if (has_dst_scale)
+                    dst_scale = 1.f
+                            / dst_scales.get_elem(
+                                    dst_scale_mask > 0 ? g * OCG + oc : 0);
+
+                conv_res *= wei_scale;
+
                 if (prb->dir & FLAG_BIA) {
                     const size_t bia_off = bia_off_f(prb, g, oc);
                     conv_res += ((float *)bia_m)[bia_off];
                 }
 
-                maybe_oscale(prb->attr, conv_res, prb->scales, g * OCG + oc);
+                const auto v_po_vals
+                        = prepare_po_vals(dst_m, args, v_po_masks, dst_off);
 
-                std::vector<float> v_binary_vals;
-                v_binary_vals.reserve(v_bin_po_mask.size());
-                for (size_t d = 0; d < v_bin_po_mask.size(); ++d) {
-                    auto bin_po_offset
-                            = dst_m.get_scale_idx(dst_off, v_bin_po_mask[d]);
-                    float binary_val = binary_po[d].get_elem(bin_po_offset);
-                    v_binary_vals.push_back(binary_val);
-                }
-                maybe_post_ops(prb->attr, conv_res, dst, v_binary_vals);
+                maybe_post_ops(prb->attr, conv_res, dst, v_po_vals);
 
-                maybe_zero_point(prb->attr, conv_res, prb->dst_zp, g * OCG + oc,
-                        DNNL_ARG_DST, true);
-
-                dst = conv_res;
+                int dst_zp = has_dst_zp
+                        ? dst_zps.get_elem(dst_zp_mask > 0 ? g * OCG + oc : 0)
+                        : 0;
+                dst = conv_res * dst_scale + dst_zp;
+                maybe_round(
+                        prb->attr, DNNL_ARG_DST, dst, dst_off, prb->dst_dt());
             });
 }
 
-void compute_ref_direct_bwd_d(const prb_t *prb, dnn_mem_t &diff_src_m,
-        dnn_mem_t &wei_m, dnn_mem_t &bia_m,
-        const std::vector<dnn_mem_t> &binary_po, dnn_mem_t &diff_dst_m) {
+void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
+    const dnn_mem_t &diff_src_m = args.find(DNNL_ARG_DIFF_SRC);
+    const dnn_mem_t &wei_m = args.find(DNNL_ARG_WEIGHTS);
+    const dnn_mem_t &bia_m = args.find(DNNL_ARG_BIAS);
+    const dnn_mem_t &diff_dst_m = args.find(DNNL_ARG_DIFF_DST);
+    const dnn_mem_t &src_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC);
+    const dnn_mem_t &wei_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS);
+    const dnn_mem_t &dst_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
+    const dnn_mem_t &src_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const dnn_mem_t &wei_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_WEIGHTS);
+    const dnn_mem_t &dst_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
+
+    const bool has_src_scale = !prb->attr.scales.get(DNNL_ARG_SRC).is_def();
+    const bool has_wei_scale = !prb->attr.scales.get(DNNL_ARG_WEIGHTS).is_def();
+    const bool has_dst_scale = !prb->attr.scales.get(DNNL_ARG_DST).is_def();
+    assert(IMPLICATION(has_src_scale, src_scales.nelems() == 1));
+    assert(IMPLICATION(has_dst_scale, dst_scales.nelems() == 1));
+    float src_scale = has_src_scale ? src_scales.get_elem(0) : 1.f;
+    float dst_scale = has_dst_scale ? 1.f / dst_scales.get_elem(0) : 1.f;
+    const int wei_scale_mask = prb->attr.scales.get_mask(
+            DNNL_ARG_WEIGHTS, dnnl_convolution, wei_m.ndims(), prb->has_groups);
+
+    const bool has_src_zp = !prb->attr.zero_points.get(DNNL_ARG_SRC).is_def();
+    const bool has_wei_zp
+            = !prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).is_def();
+    const bool has_dst_zp = !prb->attr.zero_points.get(DNNL_ARG_DST).is_def();
+    const int src_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_SRC).policy);
+    const int wei_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).policy);
+    const int dst_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_DST).policy);
 
     /* help compiler optimize the code */
     const int64_t MB = prb->mb, G = prb->g, OC = prb->oc, IC = prb->ic;
@@ -223,21 +209,11 @@ void compute_ref_direct_bwd_d(const prb_t *prb, dnn_mem_t &diff_src_m,
     const int64_t DH = prb->dh + 1;
     const int64_t DW = prb->dw + 1;
 
+    int wei_zp = (has_wei_zp && (wei_zp_mask == 0)) ? wei_zps.get_elem(0) : 0;
+
     enum { precompute_size = 16 };
     const bool fast = MAX3(KD, KH, KW) <= precompute_size;
 
-    // from bwd pov zp src from fwd is zp diff dst and
-    // zp dst is zp dst is zp diff_src
-    const auto map_arg_to_zp_arg = [](int num) {
-        switch (num) {
-            case DNNL_ARG_DIFF_DST: return DNNL_ARG_SRC;
-            case DNNL_ARG_DIFF_SRC: return DNNL_ARG_DST;
-            default: assert(false && "map_arg_to_zp_arg unsupported arg");
-        }
-
-        return -1;
-    };
-
     /* pre-computes arrays of oh(ow) and kh(kw) for traversing in kernel */
     auto precompute_ok
             = [](int64_t i, int64_t O, int64_t K, int64_t S, int64_t P,
@@ -271,17 +247,24 @@ void compute_ref_direct_bwd_d(const prb_t *prb, dnn_mem_t &diff_src_m,
 
         for_(int64_t d = 0; d < num_d; ++d)
         for_(int64_t h = 0; h < num_h; ++h)
-        for (int64_t w = 0; w < num_w; ++w) {
-            for (int64_t oc = 0; oc < OCG; ++oc) {
-                const int64_t diff_dst_off
-                        = ((oc * OD + od[d]) * OH + oh[h]) * OW + ow[w];
-                const int64_t wei_off
-                        = ((oc * ICG * KD + kd[d]) * KH + kh[h]) * KW + kw[w];
-                float diff_dst_val = diff_dst_loc[diff_dst_off];
-                maybe_zero_point(prb->attr, diff_dst_val, prb->src_zp,
-                        g * OCG + oc, map_arg_to_zp_arg(DNNL_ARG_DIFF_DST));
-                ds += diff_dst_val * wei_loc[wei_off];
-            }
+        for_(int64_t w = 0; w < num_w; ++w)
+        for (int64_t oc = 0; oc < OCG; ++oc) {
+            const int64_t diff_dst_off
+                    = ((oc * OD + od[d]) * OH + oh[h]) * OW + ow[w];
+            const int64_t wei_off
+                    = ((oc * ICG * KD + kd[d]) * KH + kh[h]) * KW + kw[w];
+            int src_zp = has_src_zp
+                    ? src_zps.get_elem(src_zp_mask > 0 ? g * OCG + oc : 0)
+                    : 0;
+            float diff_dst_val
+                    = (diff_dst_loc[diff_dst_off] - src_zp) * src_scale;
+
+            float wei_scale = 1.f;
+            if (has_wei_scale)
+                wei_scale = wei_scales.get_elem(
+                        wei_scale_mask > 0 ? g * ICG + ic : 0);
+            float wei_val = (wei_loc[wei_off] - wei_zp) * wei_scale;
+            ds += diff_dst_val * wei_val;
         }
     };
 
@@ -309,20 +292,27 @@ void compute_ref_direct_bwd_d(const prb_t *prb, dnn_mem_t &diff_src_m,
                                 = ((oc * OD + od) * OH + oh) * OW + ow;
                         const int64_t wei_off
                                 = ((oc * ICG * KD + kd) * KH + kh) * KW + kw;
-                        float diff_dst_val = diff_dst_loc[diff_dst_off];
-                        maybe_zero_point(prb->attr, diff_dst_val, prb->src_zp,
-                                g * OCG + oc,
-                                map_arg_to_zp_arg(DNNL_ARG_DIFF_DST));
+                        int src_zp = has_src_zp ? src_zps.get_elem(
+                                             src_zp_mask > 0 ? g * OCG + oc : 0)
+                                                : 0;
+                        float diff_dst_val
+                                = (diff_dst_loc[diff_dst_off] - src_zp)
+                                * src_scale;
 
-                        ds += diff_dst_val * wei_loc[wei_off];
+                        float wei_scale = 1.f;
+                        if (has_wei_scale)
+                            wei_scale = wei_scales.get_elem(
+                                    wei_scale_mask > 0 ? g * ICG + ic : 0);
+                        float wei_val = (wei_loc[wei_off] - wei_zp) * wei_scale;
+                        ds += diff_dst_val * wei_val;
                     }
                 }
             }
         }
     };
 
-    std::vector<int> v_bin_po_mask = prb->attr.post_ops.get_binary_po_masks();
-    dnnl::impl::parallel_nd(G, MB, ICG, ID, IH, IW,
+    auto v_po_masks = prb->attr.post_ops.get_po_masks();
+    benchdnn_parallel_nd(G, MB, ICG, ID, IH, IW,
             [&](int64_t g, int64_t mb, int64_t ic, int64_t id, int64_t ih,
                     int64_t iw) {
                 size_t src_off = src_off_f(prb, mb, g, ic, id, ih, iw);
@@ -337,26 +327,23 @@ void compute_ref_direct_bwd_d(const prb_t *prb, dnn_mem_t &diff_src_m,
                     const size_t bia_off = (size_t)g * ICG + ic;
                     conv_res += ((float *)bia_m)[bia_off];
                 }
-                maybe_oscale(prb->attr, conv_res, prb->scales, g * ICG + ic);
 
-                std::vector<float> v_binary_vals;
-                v_binary_vals.reserve(v_bin_po_mask.size());
-                for (size_t d = 0; d < v_bin_po_mask.size(); ++d) {
-                    auto bin_po_offset = diff_src_m.get_scale_idx(
-                            src_off, v_bin_po_mask[d]);
-                    float binary_val = binary_po[d].get_elem(bin_po_offset);
-                    v_binary_vals.push_back(binary_val);
-                }
-                maybe_post_ops(prb->attr, conv_res, ds, v_binary_vals);
-                maybe_zero_point(prb->attr, conv_res, prb->dst_zp, g * ICG + ic,
-                        map_arg_to_zp_arg(DNNL_ARG_DIFF_SRC), true);
+                const auto v_po_vals = prepare_po_vals(
+                        diff_src_m, args, v_po_masks, src_off);
 
-                ds = conv_res;
+                maybe_post_ops(prb->attr, conv_res, ds, v_po_vals);
+
+                int dst_zp = has_dst_zp
+                        ? dst_zps.get_elem(dst_zp_mask > 0 ? g * ICG + ic : 0)
+                        : 0;
+                ds = conv_res * dst_scale + dst_zp;
             });
 }
 
-void compute_ref_bwd_weights(const prb_t *prb, dnn_mem_t &src_m,
-        dnn_mem_t &diff_wei_m, dnn_mem_t &diff_dst_m) {
+void compute_ref_bwd_weights(const prb_t *prb, const args_t &args) {
+    const dnn_mem_t &src_m = args.find(DNNL_ARG_SRC);
+    const dnn_mem_t &diff_wei_m = args.find(DNNL_ARG_DIFF_WEIGHTS);
+    const dnn_mem_t &diff_dst_m = args.find(DNNL_ARG_DIFF_DST);
     /* help compiler optimize the code */
     const int64_t MB = prb->mb, G = prb->g, OC = prb->oc, IC = prb->ic;
     const int64_t OCG = OC / G, ICG = IC / G;
@@ -407,7 +394,7 @@ void compute_ref_bwd_weights(const prb_t *prb, dnn_mem_t &src_m,
         }
     };
 
-    dnnl::impl::parallel_nd(G, OCG, ICG, KD, KH, KW,
+    benchdnn_parallel_nd(G, OCG, ICG, KD, KH, KW,
             [&](int64_t g, int64_t oc, int64_t ic, int64_t kd, int64_t kh,
                     int64_t kw) {
                 size_t wei_off = wei_off_f(prb, g, oc, ic, kd, kh, kw);
@@ -417,14 +404,15 @@ void compute_ref_bwd_weights(const prb_t *prb, dnn_mem_t &src_m,
             });
 }
 
-void compute_ref_bwd_bias(
-        const prb_t *prb, dnn_mem_t &diff_bia_m, dnn_mem_t &diff_dst_m) {
+void compute_ref_bwd_bias(const prb_t *prb, const args_t &args) {
+    const dnn_mem_t &diff_bia_m = args.find(DNNL_ARG_DIFF_BIAS);
+    const dnn_mem_t &diff_dst_m = args.find(DNNL_ARG_DIFF_DST);
     /* help compiler optimize the code */
     const int64_t MB = prb->mb, G = prb->g, OC = prb->oc;
     const int64_t OCG = OC / G;
     const int64_t OD = prb->od, OH = prb->oh, OW = prb->ow;
 
-    dnnl::impl::parallel_nd(G, OCG, [&](int64_t g, int64_t oc) {
+    benchdnn_parallel_nd(G, OCG, [&](int64_t g, int64_t oc) {
         size_t bia_off = bia_off_f(prb, g, oc);
         double sum = 0;
 
@@ -439,11 +427,62 @@ void compute_ref_bwd_bias(
     });
 }
 
-void compute_ref_direct_bwd_w(const prb_t *prb, dnn_mem_t &src_m,
-        dnn_mem_t &diff_wei_m, dnn_mem_t &diff_bia_m, dnn_mem_t &diff_dst_m) {
-    compute_ref_bwd_weights(prb, src_m, diff_wei_m, diff_dst_m);
+void compute_ref_direct_bwd_w(const prb_t *prb, const args_t &args) {
+    compute_ref_bwd_weights(prb, args);
     if (!(prb->dir & FLAG_BIA)) return;
-    compute_ref_bwd_bias(prb, diff_bia_m, diff_dst_m);
+    compute_ref_bwd_bias(prb, args);
+}
+
+void compute_ref_fwd(
+        const prb_t *prb, const args_t &args, dnnl_primitive_t prim_ref) {
+    if (prim_ref) {
+        SAFE_V(execute_and_wait(prim_ref, args));
+        return;
+    }
+
+    if (prb->alg == WINO && prb->get_dt(SRC) == dnnl_f32) {
+        compute_wino_ref_fwd(prb, args);
+    } else {
+        compute_ref_direct_fwd(prb, args);
+    }
+}
+
+void compute_ref_bwd_d(
+        const prb_t *prb, const args_t &args, dnnl_primitive_t prim_ref) {
+    if (prim_ref) {
+        SAFE_V(execute_and_wait(prim_ref, args));
+        return;
+    }
+
+    if (prb->alg == WINO && prb->get_dt(SRC) == dnnl_f32) {
+        compute_wino_ref_bwd_d(prb, args);
+    } else {
+        compute_ref_direct_bwd_d(prb, args);
+    }
+}
+
+void compute_ref_bwd_w(
+        const prb_t *prb, const args_t &args, dnnl_primitive_t prim_ref) {
+    if (prim_ref) {
+        SAFE_V(execute_and_wait(prim_ref, args));
+        return;
+    }
+
+    if (prb->alg == WINO && prb->get_dt(SRC) == dnnl_f32) {
+        compute_wino_ref_bwd_w(prb, args);
+    } else {
+        compute_ref_direct_bwd_w(prb, args);
+    }
+}
+
+void compute_ref(
+        const prb_t *prb, const args_t &args, dnnl_primitive_t prim_ref) {
+    if (prb->dir & FLAG_FWD)
+        compute_ref_fwd(prb, args, prim_ref);
+    else if (prb->dir == BWD_D)
+        compute_ref_bwd_d(prb, args, prim_ref);
+    else if (prb->dir & FLAG_BWD && prb->dir & FLAG_WEI)
+        compute_ref_bwd_w(prb, args, prim_ref);
 }
 
 } // namespace conv
