@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -26,12 +26,21 @@
 #include "common.hpp"
 #include "dnn_types.hpp"
 #include "dnnl_common.hpp"
-#include "dnnl_memory.hpp"
-#include "perf_report.hpp"
+#include "utils/cfg.hpp"
+#include "utils/perf_report.hpp"
+#include "utils/settings.hpp"
 
 namespace pool {
 
-enum alg_t { MAX, AVG_NP, AVG_P };
+enum alg_t {
+    undef,
+    max,
+    avg_np,
+    avg_p,
+    pooling_max = max,
+    pooling_avg_exclude_padding = avg_np,
+    pooling_avg_include_padding = avg_p,
+};
 alg_t str2alg(const char *str);
 const char *alg2str(alg_t alg);
 dnnl_alg_kind_t alg2alg_kind(alg_t alg);
@@ -46,7 +55,7 @@ struct desc_t {
     int64_t pd, ph, pw;
     int64_t pd_r, ph_r, pw_r; // End side padding for each dimension
 
-    const char *name;
+    std::string name;
     int ndims;
 
     // Initialize dependent opposite-side paddings values from the shape
@@ -57,6 +66,14 @@ struct desc_t {
         pd_r = opp_pad(id, od, kd, dd, sd, pd);
     }
 
+    dims_t src_dims() const;
+    dims_t dst_dims() const;
+    dims_t strides() const;
+    dims_t kernel() const;
+    dims_t dilations() const;
+    dims_t padding() const;
+    dims_t padding_r() const;
+
 private:
     int64_t opp_pad(
             int64_t i, int64_t o, int64_t k, int64_t d, int64_t s, int64_t p) {
@@ -67,33 +84,7 @@ private:
 int str2desc(desc_t *desc, const char *str);
 std::ostream &operator<<(std::ostream &s, const desc_t &d);
 
-/** configuration structure, that controls initial data filling + error check
- *
- * dt defines pooling precision
- *
- * for each type (SRC and DST) the values are filled as follows:
- * if (rand() > f_sparsity) then:
- *     v <-- f_base // it is guaranteed each kernel window
- *                  // has at least one non-zero element
- * else:
- *     v <-- f_min + rand() * f_step % (f_max - f_min)
- *
- * on final check the resulting values should be in [min .. max] range, the
- * relative difference should not exceed eps
- */
-typedef struct dt_conf_t {
-    dnnl_data_type_t dt;
-    double min, max; /* representative */
-    int f_min, f_max; /* fill range */
-    double eps; /* acceptable error */
-} _dt_conf_t[DAT_TOTAL];
-
-extern const _dt_conf_t conf_f32;
-
-const dt_conf_t *str2cfg(const char *str);
-std::ostream &operator<<(std::ostream &s, const dt_conf_t *cfg);
-
-struct settings_t {
+struct settings_t : public base_settings_t {
     settings_t() = default;
 
     // ctor to save certain fields from resetting
@@ -104,62 +95,102 @@ struct settings_t {
     desc_t desc {};
 
     std::vector<dir_t> dir {FWD_D};
-    std::vector<const dt_conf_t *> cfg {conf_f32};
+    std::vector<std::vector<dnnl_data_type_t>> dt {{dnnl_f32}};
     std::vector<std::string> tag {tag::abx};
-    std::vector<alg_t> alg {MAX};
-    std::vector<int64_t> mb {0};
-    std::vector<attr_t::post_ops_t> post_ops {attr_t::post_ops_t()};
-    std::vector<dnnl_scratchpad_mode_t> scratchpad_mode {
-            dnnl_scratchpad_mode_library};
+    std::vector<alg_t> alg {max};
 
-    const char *perf_template_csv
-            = "perf,%engine%,%impl%,%name%,%dir%,%cfg%,%tag%,%alg%,%DESC%,%-"
-              "time%,%0time%";
-    const char *perf_template_def
-            = "perf,%engine%,%impl%,%name%,%prb%,%-time%,%0time%";
-    const char *perf_template = perf_template_def;
+    const char *perf_template_csv() const {
+        static const std::string args = "%dir%,%sdt%,%tag%,%alg%";
+        return perf_template_csv_base(args);
+    }
 
     void reset() { *this = settings_t(perf_template); }
+
+    bool has_single_setup() const override {
+        return dir.size() == 1 && dt.size() == 1 && tag.size() == 1
+                && alg.size() == 1 && base_settings_t::has_single_setup();
+    }
 };
 
 struct prb_t : public desc_t {
-    prb_t(const desc_t &desc, dir_t dir, const dt_conf_t *cfg,
-            const std::string &tag, alg_t alg, const attr_t &attr,
-            int64_t mb = 0)
+    // A ctor with common interface across all drivers.
+    prb_t(const settings_t &s)
+        : prb_t(s.desc, s.dir[0], s.dt[0], s.tag[0], s.alg[0], s.mb[0],
+                s.attributes.front(), s.ctx_init[0], s.ctx_exe[0],
+                s.impl_filter) {
+        SAFE_V(s.has_single_setup() ? OK : FAIL);
+    }
+
+    prb_t(const desc_t &desc, dir_t dir,
+            const std::vector<dnnl_data_type_t> &dt, const std::string &tag,
+            alg_t alg, int64_t mb, const attr_t &attr,
+            const thr_ctx_t &ctx_init, const thr_ctx_t &ctx_exe,
+            const impl_filter_t &impl_filter)
         : desc_t(desc)
         , dir(dir)
-        , cfg(cfg)
+        , dt(dt)
         , tag(tag)
         , alg(alg)
+        , user_mb(mb)
         , attr(attr)
-        , user_mb(mb) {
+        , ctx_init(ctx_init)
+        , ctx_exe(ctx_exe)
+        , impl_filter(impl_filter) {
         if (mb) this->mb = mb;
+
+        // Broadcast data types if needed
+        if (dt.size() == 1) {
+            const auto val = dt[0]; // Need a copy here.
+            this->dt.assign(2, val);
+        }
+
+        repro = set_repro_line(); // must be last in ctor to collect right info
     }
-    ~prb_t() {}
 
     dir_t dir;
-    const dt_conf_t *cfg;
+    std::vector<dnnl_data_type_t> dt;
     std::string tag;
     alg_t alg;
-    attr_t attr;
     int64_t user_mb;
+    bool inplace = false; // Lacks placement, always considered `false`.
+    attr_t attr;
+    thr_ctx_t ctx_init, ctx_exe;
+    impl_filter_t impl_filter;
 
-    BENCHDNN_DISALLOW_COPY_AND_ASSIGN(prb_t);
-};
-std::ostream &operator<<(std::ostream &s, const prb_t &prb);
-
-struct perf_report_t : public base_perf_report_t {
-    using base_perf_report_t::base_perf_report_t;
-
-    void report(const prb_t *prb, const res_t *res, const char *prb_str) {
-        p_ = prb;
-        tag_ = normalize_tag(p_->tag, p_->ndims);
-        base_report(res, prb_str);
+    int64_t kernel_size() const { return kd * kh * kw; }
+    bool has_ker_in_pad() const {
+        bool ker_in_pad_d = pd >= kd || pd_r >= kd;
+        bool ker_in_pad_h = ph >= kh || ph_r >= kh;
+        bool ker_in_pad_w = pw >= kw || pw_r >= kw;
+        return ker_in_pad_d || ker_in_pad_h || ker_in_pad_w;
     }
 
-    void dump_alg(std::ostream &s) const override { s << alg2str(p_->alg); }
+    dnnl_data_type_t src_dt() const { return dt[0]; }
+    dnnl_data_type_t dst_dt() const { return dt[1]; }
+    dnnl_data_type_t get_dt(data_kind_t data_kind) const;
 
-    void dump_cfg(std::ostream &s) const override { s << p_->cfg; }
+    // Used to construct memory desc when dimensions are runtime since such mds
+    // can't be used directly from query and memory objects can't be constructed.
+    benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> get_md(int arg) const {
+        assert(!"No runtime dimensions support for this driver!");
+        return make_benchdnn_dnnl_wrapper<dnnl_memory_desc_t>(nullptr);
+    }
+
+    const char *str() const { return repro.c_str(); }
+
+private:
+    std::string repro;
+
+    std::string set_repro_line();
+};
+
+struct perf_report_t : public base_perf_report_t {
+    perf_report_t(const prb_t *prb, const char *perf_template)
+        : base_perf_report_t(perf_template)
+        , p_(prb)
+        , tag_(normalize_tag(p_->tag, p_->ndims)) {}
+
+    void dump_alg(std::ostream &s) const override { s << alg2str(p_->alg); }
 
     void dump_desc(std::ostream &s) const override {
         s << static_cast<const desc_t &>(*p_);
@@ -182,15 +213,27 @@ struct perf_report_t : public base_perf_report_t {
     }
 
     const int64_t *user_mb() const override { return &p_->user_mb; }
-    const char *name() const override { return p_->name; }
+    const std::vector<dnnl_data_type_t> *sdt() const override {
+        return &p_->dt;
+    }
+    const attr_t *attr() const override { return &p_->attr; }
+    const thr_ctx_t *ctx_init() const override { return &p_->ctx_init; }
+    const thr_ctx_t *ctx_exe() const override { return &p_->ctx_exe; }
+    const std::string *name() const override { return &p_->name; }
     const dir_t *dir() const override { return &p_->dir; }
     const std::string *tag() const override { return &tag_; }
 
 private:
-    const prb_t *p_ = NULL;
+    const prb_t *p_;
     std::string tag_;
 };
 
+struct cfg_t : public base_cfg_t {
+    cfg_t(const prb_t *prb, const std::vector<data_kind_t> &kinds);
+
+    cfg_entry_t::cfg_map_t get_cfg_map(data_kind_t kind) const override;
+};
+
 inline int64_t src_off_f(const prb_t *prb, int64_t mb, int64_t ic, int64_t id,
         int64_t ih, int64_t iw) {
     return (((mb * prb->ic + ic) * prb->id + id) * prb->ih + ih) * prb->iw + iw;
@@ -230,28 +273,32 @@ inline int64_t get_num_summands(
     auto ih_end_excluded = ih_end > IH ? (ih_end - IH - 1) / (DH + 1) + 1 : 0;
     auto iw_end_excluded = iw_end > IW ? (iw_end - IW - 1) / (DW + 1) + 1 : 0;
 
-    return prb->alg == AVG_P ? KD * KH * KW
+    return prb->alg == avg_p ? prb->kernel_size()
                              : (KD - id_start_excluded - id_end_excluded)
                     * (KH - ih_start_excluded - ih_end_excluded)
                     * (KW - iw_start_excluded - iw_end_excluded);
 }
 
-void compute_ref_fwd(const prb_t *prb, const dnn_mem_t &src,
-        const std::vector<dnn_mem_t> &binary_po, dnn_mem_t &dst, dnn_mem_t &ws);
-void compute_ref_bwd(const prb_t *prb, dnn_mem_t &diff_src,
-        const dnn_mem_t &diff_dst, const dnn_mem_t &ws);
+dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args);
+void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
+        const args_t &ref_args);
+std::vector<int> supported_exec_args(dir_t dir);
+int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
+        dnnl_primitive_t prim, const prb_t *prb, res_t *res,
+        dnnl_primitive_t prim_ref = nullptr);
 
-int compare_src(
-        const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res);
-int compare_dst(
-        const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res);
-int fill_src(
-        const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res);
-int fill_dst(
-        const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res);
-int fill_ws(const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res);
+void skip_unimplemented_prb(const prb_t *prb, res_t *res);
+void skip_invalid_prb(const prb_t *prb, res_t *res);
+void compute_ref(const prb_t *prb, const args_t &args,
+        dnnl_primitive_t prim_ref = nullptr);
 
-int doit(const prb_t *prb, res_t *res);
+int createit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+        const prb_t *prb, res_t *res);
+int check_cacheit(
+        std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+        const prb_t *prb, res_t *res);
+int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+        const prb_t *prb, res_t *res);
 int bench(int argc, char **argv);
 
 } // namespace pool
