@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,6 +14,8 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <sstream>
+
 #include "dnnl_common.hpp"
 #include "dnnl_debug.hpp"
 
@@ -21,21 +23,28 @@
 
 namespace shuffle {
 
-std::ostream &operator<<(std::ostream &s, const prb_t &prb) {
+std::string prb_t::set_repro_line() {
+    std::stringstream s;
     dump_global_params(s);
     settings_t def;
 
-    if (canonical || prb.dir != def.dir[0]) s << "--dir=" << prb.dir << " ";
-    if (canonical || prb.dt != def.dt[0]) s << "--dt=" << prb.dt << " ";
-    if (canonical || prb.tag != def.tag[0]) s << "--tag=" << prb.tag << " ";
-    if (canonical || prb.group != def.group[0])
-        s << "--group=" << prb.group << " ";
-    if (canonical || prb.axis != def.axis[0]) s << "--axis=" << prb.axis << " ";
+    if (canonical || dir != def.dir[0]) s << "--dir=" << dir << " ";
+    if (canonical || dt != def.dt[0]) s << "--dt=" << dt << " ";
+    if (canonical || tag != def.tag[0]) s << "--tag=" << tag << " ";
+    if (canonical || group != def.group[0]) s << "--group=" << group << " ";
+    if (canonical || axis != def.axis[0]) s << "--axis=" << axis << " ";
 
-    s << prb.attr;
-    s << prb.dims;
+    s << attr;
+    if (canonical || ctx_init != def.ctx_init[0])
+        s << "--ctx-init=" << ctx_init << " ";
+    if (canonical || ctx_exe != def.ctx_exe[0])
+        s << "--ctx-exe=" << ctx_exe << " ";
+    if (canonical || !impl_filter.is_def() || !global_impl_filter.is_def())
+        s << impl_filter;
 
-    return s;
+    s << static_cast<prb_dims_t>(*this);
+
+    return s.str();
 }
 
 } // namespace shuffle
