@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -121,7 +121,14 @@ CPU_INST_TEST_CASE(TestGEMM_smalln,
         test_params {'n', 't', 8, 512, 2048, 1.0f, 1.0f, 2048, 2048, 512},
         test_params {'n', 't', 8, 2048, 512, 1.0f, 1.0f, 512, 512, 2048});
 
+CPU_INST_TEST_CASE(TestGEMM_stkmem,
+        test_params {'n', 'n', 2, 48, 83, 1.0f, 0.0f, 83, 48, 48},
+        test_params {'n', 'n', 2, 48, 200, 1.0f, 0.0f, 200, 48, 48},
+        test_params {'n', 'n', 2, 16, 251, 1.0f, 0.0f, 251, 16, 16},
+        test_params {'n', 'n', 2, 16, 256, 1.0f, 0.0f, 256, 16, 16});
+
 #if defined(FP32) || defined(BF16BF16F32)
+#if !BUILD_GEMM_KERNELS_NONE
 INST_TEST_CASE(TestGEMM_packed,
         test_params {'t', 'n', 3, 2, 1, 1.0, 0.0, 2, 5, 8, {}, {false, true},
                 true, dnnl_invalid_arguments},
@@ -192,6 +199,7 @@ INST_TEST_CASE(TestGEMM_packed,
         make_test_params_pack({false, true}, 't', 'n', 200, 300, 8000, 1.0f,
                 3.0f, 200, 300, 300));
 #endif
+#endif
 
 #elif defined(BF16BF16BF16)
 
@@ -248,6 +256,7 @@ constexpr test_igemm_params fix_no_offsets = {'F', false, false, false};
 constexpr test_igemm_params col_no_offsets = {'C', false, false, false};
 constexpr test_igemm_params row_no_offsets = {'R', false, false, false};
 
+#if !BUILD_GEMM_KERNELS_NONE
 INST_TEST_CASE(TestGEMM_expected_failures,
         test_params {'t', 'n', 3, 2, 1, 1.0, 0.0, 2, 5, 8, {}, {}, true,
                 dnnl_invalid_arguments},
@@ -284,6 +293,13 @@ INST_TEST_CASE(TestGEMM_expected_failures,
                 true, dnnl_invalid_arguments},
         test_params {'n', 'd', 3, 2, 1, 1.0, 0.0, 3, 3, 3, {}, {false, true},
                 true, dnnl_invalid_arguments});
+#endif
+
+CPU_INST_TEST_CASE(TestGEMM_stkmem,
+        test_params {'n', 'n', 10, 4000, 2, 1.0, 0.0, 2, 4000, 4000,
+                fix_use_all_offsets},
+        test_params {'n', 'n', 10, 5000, 2, 1.0, 0.0, 2, 5000, 5000,
+                fix_use_all_offsets});
 
 INST_TEST_CASE(TestGEMM_general_cases_fix_offset,
         test_params {'N', 'n', 30, 20, 10, 1.0, 0.0, 60, 50, 80, fix_use_oc},
@@ -633,6 +649,7 @@ CPU_INST_TEST_CASE(TestGEMM_fractional_scales_row_offset,
         test_params {'n', 'n', 2, 2, 10000, 1.66f, 2.33f, 10000, 2, 2,
                 row_no_offsets});
 
+#ifndef DNNL_ENABLE_MEM_DEBUG
 CPU_INST_TEST_CASE(TestGEMV,
         test_params {'n', 'n', 2000, 1, 1000, 1.0f, 0.0f, 1000, 1, 1,
                 fix_no_offsets},
@@ -720,6 +737,7 @@ CPU_INST_TEST_CASE(TestGEMV_kblocking,
         test_params {'t', 'n', 1, 550, 7000, 1.0f, 1.0f, 7000, 550, 550,
                 fix_no_offsets});
 
+#if !BUILD_GEMM_KERNELS_NONE
 CPU_INST_TEST_CASE(TestGEMM_packed,
         make_test_params_pack({false, true}, 'N', 'n', 30, 20, 10, 1.0f, 1.0f,
                 60, 50, 80, fix_use_oc),
@@ -816,6 +834,8 @@ CPU_INST_TEST_CASE(TestGEMM_packed,
                 1, 200, 200, fix_no_offsets),
         make_test_params_pack({false, true}, 'n', 'T', 1, 200, 200, 1.0f, 0.0f,
                 200, 200, 200, fix_no_offsets));
+#endif
+#endif
 
 CPU_INST_TEST_CASE(TestGEMM_heavy,
         test_params {'n', 'n', 3000, 3000, 3000, 1.0, 0.0, 3000, 3000, 3000,
@@ -827,6 +847,7 @@ CPU_INST_TEST_CASE(TestGEMM_heavy,
         test_params {'t', 't', 3000, 3000, 3000, 1.0, 0.0, 3000, 3000, 3000,
                 fix_use_oc});
 
+#if !BUILD_GEMM_KERNELS_NONE
 CPU_INST_TEST_CASE(TestGEMM_packed_heavy,
         make_test_params_pack({false, true}, 'n', 'n', 3000, 3000, 3000, 1.0f,
                 0.0f, 3000, 3000, 3000, fix_use_oc),
@@ -860,5 +881,5 @@ CPU_INST_TEST_CASE(TestGEMM_packed_heavy,
                 3.0f, 8000, 8000, 200, row_use_oc),
         make_test_params_pack({false, true}, 't', 'n', 200, 300, 8000, 1.0f,
                 0.0f, 200, 300, 300, col_use_oc));
-
+#endif
 #endif
