@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -39,13 +39,13 @@ void compute_ref_inner_product_bwd_data(int ndims,
     const memory::desc diff_dst_d = diff_dst.get_desc();
     const memory::desc weights_d = weights.get_desc();
     const memory::desc diff_src_d = diff_src.get_desc();
-    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.data);
-    const dnnl::impl::memory_desc_wrapper weights_mdw(weights_d.data);
-    const dnnl::impl::memory_desc_wrapper diff_src_mdw(diff_src_d.data);
+    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.get());
+    const dnnl::impl::memory_desc_wrapper weights_mdw(weights_d.get());
+    const dnnl::impl::memory_desc_wrapper diff_src_mdw(diff_src_d.get());
 
     bool has_spatial = ipd.kh > 1 || ipd.kw > 1;
     if (ndims == 5) has_spatial = has_spatial || ipd.kd > 1;
-    auto padded_ic = diff_src_d.data.padded_dims[1];
+    auto padded_ic = diff_src_d.get_padded_dims()[1];
 
     dnnl::impl::parallel_nd(ipd.mb, ipd.ic, [&](memory::dim n, memory::dim ic) {
         if (has_spatial) {
@@ -100,8 +100,9 @@ protected:
                              p.weights_format, p.diff_dst_format),
                 "Unsupported format tag");
         SKIP_IF_CUDA(p.ndims > 5, "Unsupported number of dimensions");
+        SKIP_IF_GENERIC(true, "Primitive not implemented");
         catch_expected_failures(
-                [=]() { Test(); }, p.expect_to_fail, p.expected_status);
+                [&]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
 
     bool cuda_check_format_tags(memory::format_tag diff_src_format,
@@ -135,6 +136,11 @@ protected:
     void Test() {
         auto p = ::testing::TestWithParam<inprod_test_params_t>::GetParam();
         test_inner_product_descr_t ipd = p.test_ipd;
+
+        // ip specific types and values
+        using pd_t = inner_product_backward_data::primitive_desc;
+        using hint_pd_t = inner_product_forward::primitive_desc;
+
         bool has_spatial = ipd.kh > 1 || ipd.kw > 1;
         if (p.ndims == 5) has_spatial = has_spatial || ipd.kd > 1;
 
@@ -166,19 +172,17 @@ protected:
                 = create_md({ipd.mb, ipd.oc}, data_type, p.diff_dst_format);
 
         // Create inner product forward (hint for backward)
-        auto ip_fwd_desc = inner_product_forward::desc(prop_kind::forward,
-                ip_diff_src_desc, ip_weights_desc, ip_diff_dst_desc);
-        auto ip_fwd_pdesc
-                = inner_product_forward::primitive_desc(ip_fwd_desc, eng);
+        auto ip_fwd_pdesc = hint_pd_t(eng, prop_kind::forward, ip_diff_src_desc,
+                ip_weights_desc, ip_diff_dst_desc);
 
         // Create inner product backward
-        auto ip_desc = inner_product_backward_data::desc(
-                ip_diff_src_desc, ip_weights_desc, ip_diff_dst_desc);
+        auto ip_primitive_desc = pd_t(eng, ip_diff_src_desc, ip_weights_desc,
+                ip_diff_dst_desc, ip_fwd_pdesc);
 
-        auto ip_primitive_desc = inner_product_backward_data::primitive_desc(
-                ip_desc, eng, ip_fwd_pdesc);
-        ip_primitive_desc = inner_product_backward_data::primitive_desc(
-                ip_primitive_desc.get()); // test construction from a C pd
+        allows_attr_t aa {false}; // doesn't support anything
+        test_bwd_pd_constructors<pd_t, hint_pd_t>(ip_primitive_desc,
+                ip_fwd_pdesc, aa, ip_diff_src_desc, ip_weights_desc,
+                ip_diff_dst_desc);
 
         auto ip_diff_src
                 = test::make_memory(ip_primitive_desc.diff_src_desc(), eng);
@@ -207,6 +211,9 @@ protected:
                 ip_primitive_desc.query_md(query::exec_arg_md, DNNL_ARG_WEIGHTS)
                 == ip_primitive_desc.weights_desc());
 
+        ASSERT_EQ(ip_primitive_desc.get_prop_kind(), prop_kind::backward_data);
+
+        EXPECT_ANY_THROW(inner_product_backward_data(ip_primitive_desc, {}));
         inner_product_backward_data(ip_primitive_desc)
                 .execute(strm,
                         {{DNNL_ARG_DIFF_DST, ip_diff_dst},
