@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,6 +14,8 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <memory>
+
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
@@ -64,8 +66,8 @@ void compute_ref_resampling_fwd(const resampling_test_params_t &p,
     const memory::desc src_d = src_m.get_desc();
     const memory::desc dst_d = dst_m.get_desc();
 
-    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.data);
-    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.data);
+    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.get());
+    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.get());
 
     auto pd = p.test_pd;
     auto padded_c = src_mdw.padded_dims()[1];
@@ -129,8 +131,8 @@ void compute_ref_resampling_bwd(const resampling_test_params_t &p,
     const memory::desc diff_src_d = diff_src_m.get_desc();
     const memory::desc diff_dst_d = diff_dst_m.get_desc();
 
-    const dnnl::impl::memory_desc_wrapper diff_src_mdw(diff_src_d.data);
-    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.data);
+    const dnnl::impl::memory_desc_wrapper diff_src_mdw(diff_src_d.get());
+    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.get());
 
     auto pd = p.test_pd;
     auto padded_c = diff_src_mdw.padded_dims()[1];
@@ -208,6 +210,7 @@ private:
     std::shared_ptr<test_memory> src, dst, diff_src, diff_dst;
     std::shared_ptr<memory::desc> src_desc, dst_desc;
     std::vector<float> factors;
+    std::vector<float> expected_factors;
     resampling_forward::primitive_desc resampling_pd;
 
     resampling_test_params_t p;
@@ -219,7 +222,13 @@ protected:
         return impl::utils::one_of(
                 tag, dnnl_abc, dnnl_abcd, dnnl_acb, dnnl_acdb);
     }
+    bool generic_supported_format_tag(memory::format_tag tag) {
+        return impl::utils::one_of(tag, dnnl_abc, dnnl_abcd, dnnl_acb,
+                dnnl_acdb, dnnl_format_tag_any);
+    }
     void SetUp() override {
+        SKIP_IF_HIP(
+                true, "Resampling operator is not supported by hip backend");
         p = ::testing::TestWithParam<decltype(p)>::GetParam();
         SKIP_IF_CUDA(p.aalgorithm == algorithm::resampling_nearest,
                 "nearet algorithm is not supported for cudnn backend");
@@ -227,9 +236,11 @@ protected:
                 "cudnn resampling backend does not support 5d tensor");
         SKIP_IF_CUDA(!cuda_supported_format_tag(p.src_format),
                 "Unsupported format tag");
+        SKIP_IF_GENERIC(!generic_supported_format_tag(p.src_format),
+                "Unsupported format tag");
 
         catch_expected_failures(
-                [=]() { Test(); }, p.expect_to_fail, p.expected_status);
+                [&]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
 
     void Test() {
@@ -262,31 +273,43 @@ protected:
         }
 
         memory::data_type data_type = data_traits<data_t>::data_type;
-        src_desc.reset(new memory::desc(src_dims, data_type, p.src_format));
-        dst_desc.reset(new memory::desc(dst_dims, data_type, p.src_format));
+        src_desc = std::make_shared<memory::desc>(
+                src_dims, data_type, p.src_format);
+        dst_desc = std::make_shared<memory::desc>(
+                dst_dims, data_type, p.src_format);
+
+        expected_factors.resize(src_desc->get_ndims() - 2);
+        for (int i = 0; i < src_desc->get_ndims() - 2; i++) {
+            expected_factors[i] = (double)dst_desc->get_dims()[2 + i]
+                    / src_desc->get_dims()[2 + i];
+        }
 
         Forward();
         Backward();
     }
 
     void Forward() {
-        auto resampling_desc = resampling_forward::desc(
-                p.aprop_kind, p.aalgorithm, *src_desc, *dst_desc);
-
-        resampling_pd
-                = resampling_forward::primitive_desc(resampling_desc, eng);
+        resampling_pd = resampling_forward::primitive_desc(
+                eng, p.aprop_kind, p.aalgorithm, *src_desc, *dst_desc);
         resampling_pd = resampling_forward::primitive_desc(
                 resampling_pd.get()); // test construction from a C pd
 
-        if (true) {
-            auto resampling_desc_no_dst = resampling_forward::desc(p.aprop_kind,
-                    p.aalgorithm, factors, resampling_pd.src_desc());
-            auto resampling_pd_no_dst = resampling_forward::primitive_desc(
-                    resampling_desc_no_dst, eng);
+        {
+            auto resampling_desc_no_dst
+                    = resampling_forward::primitive_desc(eng, p.aprop_kind,
+                            p.aalgorithm, factors, resampling_pd.src_desc());
+            auto resampling_pd_no_dst
+                    = resampling_forward::primitive_desc(eng, p.aprop_kind,
+                            p.aalgorithm, factors, resampling_pd.src_desc());
             ASSERT_EQ(
                     resampling_pd.dst_desc(), resampling_pd_no_dst.dst_desc());
+            ASSERT_EQ(resampling_pd_no_dst.get_factors(), expected_factors);
         }
 
+        ASSERT_EQ(resampling_pd.get_prop_kind(), p.aprop_kind);
+        ASSERT_EQ(resampling_pd.get_algorithm(), p.aalgorithm);
+        ASSERT_EQ(resampling_pd.get_factors(), expected_factors);
+
         auto src = test::make_memory(resampling_pd.src_desc(), eng);
         auto dst = test::make_memory(resampling_pd.dst_desc(), eng);
         auto dst_ref = test::make_memory(resampling_pd.dst_desc(), eng);
@@ -294,6 +317,7 @@ protected:
         fill_data<data_t>(src.get_desc().get_size() / sizeof(data_t), src);
         check_zero_tail<data_t>(1, src);
 
+        EXPECT_ANY_THROW(resampling_forward(resampling_pd, {}));
         resampling_forward(resampling_pd)
                 .execute(strm, {{DNNL_ARG_SRC, src}, {DNNL_ARG_DST, dst}});
         strm.wait();
@@ -306,11 +330,8 @@ protected:
     }
 
     void Backward() {
-        auto resampling_bwd_desc = resampling_backward::desc(
-                p.aalgorithm, factors, *src_desc, *dst_desc);
-
-        auto resampling_bwd_pd = resampling_backward::primitive_desc(
-                resampling_bwd_desc, eng, resampling_pd);
+        auto resampling_bwd_pd = resampling_backward::primitive_desc(eng,
+                p.aalgorithm, factors, *src_desc, *dst_desc, resampling_pd);
 
         auto diff_src
                 = test::make_memory(resampling_bwd_pd.diff_src_desc(), eng);
@@ -319,11 +340,16 @@ protected:
         auto diff_src_ref
                 = test::make_memory(resampling_bwd_pd.diff_src_desc(), eng);
 
+        ASSERT_EQ(resampling_bwd_pd.get_prop_kind(), prop_kind::backward_data);
+        ASSERT_EQ(resampling_bwd_pd.get_algorithm(), p.aalgorithm);
+        ASSERT_EQ(resampling_bwd_pd.get_factors(), expected_factors);
+
         fill_data<data_t>(
                 diff_dst.get_desc().get_size() / sizeof(data_t), diff_dst);
         check_zero_tail<data_t>(1, diff_dst);
         check_zero_tail<data_t>(1, diff_src);
 
+        EXPECT_ANY_THROW(resampling_backward(resampling_bwd_pd, {}));
         resampling_backward(resampling_bwd_pd)
                 .execute(strm,
                         {{DNNL_ARG_DIFF_SRC, diff_src},
