@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -37,6 +37,8 @@ protected:
 };
 
 TEST_F(iface_sum_test_t, SumTestDstDataTypeCompliance) {
+    SKIP_IF_HIP(true, "Sum operator is not supported by HIP");
+
     using dt = memory::data_type;
 
     const dt src_dt = dt::s8;
@@ -48,16 +50,17 @@ TEST_F(iface_sum_test_t, SumTestDstDataTypeCompliance) {
     for (dt dst_dt : {dt::undef, dt::s8, dt::s32, dt::f32}) {
         sum::primitive_desc sum_pd;
         SKIP_FOR_LOOP_CUDA(dst_dt == dt::s32, "Unsupported data_type");
+        SKIP_FOR_LOOP_GENERIC(dst_dt == dt::s32, "Unsupported data_type");
         if (dst_dt != dt::undef) {
             memory::desc dst_md(shape, dst_dt, dst_tag);
             sum_pd = sum::primitive_desc(
-                    dst_md, {2., 2.}, {src_md, src_md}, eng);
+                    eng, dst_md, {2., 2.}, {src_md, src_md});
         } else {
-            sum_pd = sum::primitive_desc({2., 2.}, {src_md, src_md}, eng);
+            sum_pd = sum::primitive_desc(eng, {2., 2.}, {src_md, src_md});
         }
 
         dt expect_dst_dt = dst_dt == dt::undef ? src_dt : dst_dt;
-        ASSERT_EQ(sum_pd.dst_desc().data.data_type, expect_dst_dt);
+        ASSERT_EQ(sum_pd.dst_desc().get_data_type(), expect_dst_dt);
     }
 }
 
@@ -83,8 +86,8 @@ private:
             const std::vector<float> &scale, const memory &dst) {
         auto dst_data = map_memory<const dst_data_t>(dst);
         const auto &dst_d = dst.get_desc();
-        const auto dst_dims = dst_d.data.dims;
-        const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.data);
+        const auto dst_dims = dst_d.get_dims();
+        const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.get());
 
         std::vector<mapped_ptr_t<const src_data_t>> mapped_srcs;
         mapped_srcs.reserve(srcs.size());
@@ -101,9 +104,9 @@ private:
                     for (size_t num = 0; num < srcs.size(); num++) {
                         auto &src_data = mapped_srcs[num];
                         const auto &src_d = srcs[num].get_desc();
-                        const auto src_dims = src_d.data.dims;
+                        const auto src_dims = src_d.get_dims();
                         const dnnl::impl::memory_desc_wrapper src_mdw(
-                                src_d.data);
+                                src_d.get());
 
                         auto src_idx = w + src_dims[3] * h
                                 + src_dims[2] * src_dims[3] * c
@@ -140,7 +143,19 @@ protected:
                 dnnl_cdeba, dnnl_decab, dnnl_defcab, dnnl_aBc4b, dnnl_aBcd4b,
                 dnnl_aBcde4b);
     }
+    bool generic_supported_format_tag(memory::format_tag tag) {
+        return impl::utils::one_of(tag, dnnl_a, dnnl_ab, dnnl_abc, dnnl_abcd,
+                dnnl_abcde, dnnl_abcdef, dnnl_abdec, dnnl_acb, dnnl_acbde,
+                dnnl_acbdef, dnnl_acdb, dnnl_acdeb, dnnl_ba, dnnl_bac,
+                dnnl_bacd, dnnl_bca, dnnl_bcda, dnnl_bcdea, dnnl_cba, dnnl_cdba,
+                dnnl_cdeba, dnnl_decab, dnnl_defcab, dnnl_format_tag_any);
+    }
+    bool generic_supported_dt(memory::data_type dt) {
+        return impl::utils::one_of(
+                dt, dnnl_f32, dnnl_bf16, dnnl_f16, dnnl_s8, dnnl_u8);
+    }
     void SetUp() override {
+        SKIP_IF_HIP(true, "Sum operator is not supported by HIP");
         src_data_type = data_traits<src_data_t>::data_type;
         dst_data_type = data_traits<dst_data_t>::data_type;
         sum_test_params p
@@ -159,14 +174,29 @@ protected:
             SKIP_IF_CUDA(!cuda_supported_format_tag(p.srcs_format[i]),
                     "Unsupported format tag");
         }
+
+        SKIP_IF_GENERIC(
+                !generic_supported_dt(src_data_type), "Unsupported data type");
+        SKIP_IF_GENERIC(
+                !generic_supported_dt(dst_data_type), "Unsupported data type");
+        SKIP_IF_GENERIC(!generic_supported_format_tag(p.dst_format),
+                "Unsupported format tag");
+        for (size_t i = 0; i < p.srcs_format.size(); i++) {
+            SKIP_IF_GENERIC(!generic_supported_format_tag(p.srcs_format[i]),
+                    "Unsupported format tag");
+        }
+
         catch_expected_failures(
-                [=]() { Test(); }, p.expect_to_fail, p.expected_status);
+                [&]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
 
     void Test() {
         sum_test_params p
                 = ::testing::TestWithParam<sum_test_params>::GetParam();
 
+        // sum specific types and values
+        using pd_t = sum::primitive_desc;
+
         const auto num_srcs = p.srcs_format.size();
 
         auto eng = get_test_engine();
@@ -177,7 +207,37 @@ protected:
 
         for (size_t i = 0; i < num_srcs; i++) {
             auto desc = memory::desc(p.dims, src_data_type, p.srcs_format[i]);
-            auto src_memory = test::make_memory(desc, eng);
+            srcs_md.push_back(desc);
+        }
+
+        memory dst;
+        sum::primitive_desc sum_pd;
+
+        if (p.is_output_omitted) {
+            ASSERT_NO_THROW(sum_pd = pd_t(eng, p.scale, srcs_md));
+        } else {
+            auto dst_desc = memory::desc(p.dims, dst_data_type, p.dst_format);
+            sum_pd = pd_t(eng, dst_desc, p.scale, srcs_md);
+
+            ASSERT_EQ(sum_pd.dst_desc().get_ndims(), dst_desc.get_ndims());
+        }
+        dst = test::make_memory(sum_pd.dst_desc(), eng);
+
+        // test all pd ctors
+        auto aa = allows_attr_t {false};
+        if (p.is_output_omitted)
+            test_fwd_pd_constructors<pd_t>(sum_pd, aa, p.scale, srcs_md);
+        else {
+            auto dst_desc = memory::desc(p.dims, dst_data_type, p.dst_format);
+            test_fwd_pd_constructors<pd_t>(
+                    sum_pd, aa, dst_desc, p.scale, srcs_md);
+        }
+
+        for (size_t i = 0; i < num_srcs; i++) {
+            if (p.srcs_format[i] != memory::format_tag::any) {
+                ASSERT_TRUE(srcs_md[(int)i] == sum_pd.src_desc((int)i));
+            }
+            auto src_memory = test::make_memory(sum_pd.src_desc((int)i), eng);
             const size_t sz
                     = src_memory.get_desc().get_size() / sizeof(src_data_t);
             fill_data<src_data_t>(sz, src_memory);
@@ -189,33 +249,19 @@ protected:
                 int mant_digits
                         = dnnl::impl::nstl::numeric_limits<src_data_t>::digits;
                 int want_mant_digits = 3;
+                int digits_shift = mant_digits - want_mant_digits;
+                uint_type max_val = (uint_type)-1;
+                // Move left to keep mask value in uint_type range, move left
+                // to flush all digits but `want_mant_digits`.
+                uint_type mask = (max_val >> digits_shift) << digits_shift;
                 auto src_ptr = map_memory<src_data_t>(src_memory);
                 for (size_t i = 0; i < sz; i++) {
-                    uint_type mask = (uint_type)-1
-                            << (mant_digits - want_mant_digits);
                     *((uint_type *)&src_ptr[i]) &= mask;
                 }
             }
-            srcs_md.push_back(desc);
             srcs.push_back(src_memory);
         }
 
-        memory dst;
-        sum::primitive_desc sum_pd;
-
-        if (p.is_output_omitted) {
-            ASSERT_NO_THROW(
-                    sum_pd = sum::primitive_desc(p.scale, srcs_md, eng));
-        } else {
-            auto dst_desc = memory::desc(p.dims, dst_data_type, p.dst_format);
-            sum_pd = sum::primitive_desc(dst_desc, p.scale, srcs_md, eng);
-
-            ASSERT_EQ(sum_pd.dst_desc().data.ndims, dst_desc.data.ndims);
-        }
-        dst = test::make_memory(sum_pd.dst_desc(), eng);
-        // test construction from a C pd
-        sum_pd = sum::primitive_desc(sum_pd.get());
-
         ASSERT_TRUE(sum_pd.query_md(query::exec_arg_md, DNNL_ARG_DST)
                 == sum_pd.dst_desc());
         for (int i = 0; i < (int)srcs.size(); i++)
@@ -230,6 +276,7 @@ protected:
             dnnl::impl::parallel_nd(
                     (ptrdiff_t)sz, [&](ptrdiff_t i) { dst_data[i] = -32; });
         }
+        EXPECT_ANY_THROW(sum(sum_pd, {}));
         sum c(sum_pd);
         std::unordered_map<int, memory> args = {{DNNL_ARG_DST, dst}};
         for (int i = 0; i < (int)num_srcs; i++) {
@@ -318,7 +365,11 @@ static auto special_test_cases = []() {
             sum_test_params {{tag::nchw, tag::nChw8c}, tag::nchw, {1, 8, 4, 4},
                     {1.0f}, false, true, dnnl_invalid_arguments},
             sum_test_params {{tag::nchw, tag::nChw8c}, tag::nchw, {2, 8, 4, 4},
-                    {0.1f}, false, true, dnnl_invalid_arguments});
+                    {0.1f}, false, true, dnnl_invalid_arguments},
+            sum_test_params {{tag::any, tag::nchw}, tag::nchw, {1, 16, 1, 1},
+                    {2.0f, 3.0f}, false, true, dnnl_invalid_arguments},
+            sum_test_params {{tag::nchw, tag::any}, tag::nchw, {1, 16, 1, 1},
+                    {2.0f, 3.0f}, false, true, dnnl_invalid_arguments});
 };
 
 /* corner cases */
@@ -389,7 +440,7 @@ INST_TEST_CASE(sum_test_s32_omit_output, 1)
 INST_TEST_CASE_BF16(sum_test_bf16bf16_omit_output, 1)
 // Automatically created dst descriptor has bf16 data type so this test is not
 // valid: INST_TEST_CASE(sum_test_bf16f32_omit_output, 1)
-GPU_INST_TEST_CASE(sum_test_f16_omit_output, 1)
+INST_TEST_CASE(sum_test_f16_omit_output, 1)
 
 INST_TEST_CASE(sum_test_float, 0)
 INST_TEST_CASE(sum_test_u8, 0)
@@ -397,7 +448,7 @@ INST_TEST_CASE(sum_test_s8, 0)
 INST_TEST_CASE(sum_test_s32, 0)
 INST_TEST_CASE_BF16(sum_test_bf16bf16, 0)
 INST_TEST_CASE_BF16(sum_test_bf16f32, 0)
-GPU_INST_TEST_CASE(sum_test_f16, 0)
+INST_TEST_CASE(sum_test_f16, 0)
 
 #undef CPU_INST_TEST_CASE
 #undef GPU_INST_TEST_CASE
